function [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)

%EPSATBUILDFULLFILENAME permet d'obtenir le chemin d'accès a un fichier
%netCDF, de données de précipitations estimées d'après l'algorithme EPSAT,
%pour la date désirée.

%
%+
%
% =========================
% EPSATbuildfullfilename.m
% =========================
%
% .. function:: EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * ncfilesset (en caractères) : Type de fichier netCDF que l'on cherche,
%     'normal' ou 'extracted'.
%   * Annee (en caractères) : Année (AAAA) du jour d'étude.
%   * Mois (en caractères) : Numéro du mois (MM) du jour d'étude.
%   * Jour (en caractères) : Numéro du Jour d'étude (JJ).
%
% - Données de sorties :
%
%   * fullfilename : Chemin d'accès au fichier pour la date désirée.
%
% Cette fonction permet d'obtenir le chemin d'accès à un fichier netCDF,
% de données de précipitations estimées d'après l'algorithme EPSAT, pour la
% date désirée.
%
% EXAMPLES
% ========
%
% Voir la 'demo'.
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :ref:`varamma_startup.m`
%
% :func:`EPSATread`
%
% TODO
% ====
%
% add demo extracted
%
% test if file exists and handling error
%
% version in filename is not always v3.1-02c
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/EPSATbuildfullfilename.m $
%
% - fplod 20110804T112630Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * date is a reserved word : replaced by date_file
%
% - jaclod 2011-07-26
%
%   * Revision de la documentation et des commentaires.
%
% - jaclod 2011-07-21
%
%   * Adaptation pour les fichiers 'extracted'.
%
% - jaclod 2011-07-13
%
%   * Creation
%
%-
% Appel de la variable global 'PROJECT_ID' (Il faut auparavant avoir
% exécuté 'varamma_startup').
global PROJECT_ID

usage='[fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% Création d'une variable comportant la date nous permettant de
% sélectionner le fichier en question.
date_file = ([Annee Mois Jour]);

% Le répertoire ou se trouve le fichier, ainsi que son nom, différent selon
% qu'il soit 'normal' (fichier original) ou 'extracted' (fichier avec
% extraction des données spatiales).
switch ncfilesset

    case 'normal'

        % Répertoire contenant le fichier souhaité.
        repertoire = [ PROJECT_ID 'EPSAT/' Annee '/' Mois '/' ];

        % Création du chemin complet.
        fullfilename = [repertoire 'surf-rr_epsat-sg_multi-sat_010d_30min_' date_file '_v3.1-02c.nc'];

    case 'extracted'

         % Répertoire contenant le fichier souhaité.
        repertoire = [PROJECT_ID 'EPSAT/extracted/' Annee '/' Mois '/' ];

        % Création du chemin complet.
        fullfilename = [repertoire 'extracted-surf-rr_epsat-sg_multi-sat_010d_30min_' date_file '_v3.1-02c.nc'];

    otherwise

        error('La variable ncfilesset doit être ''normal'' ou ''extracted''.');

end

end

%!demo
%! varamma_startup;
%! more off;
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! ncfilesset = 'normal';
%! [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)
