#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
===============
amsu2ncdf_py.py
===============

SYNOPSIS
========

amsu2ncdf_py.py --fileamsu_t2 fileamsu_t2 --fileamsu_t4 fileamsu_t4

++ geobox

DESCRIPTION
===========

.. option:: --fileamsu_t2 <fileamsu_t2>
.. option:: --fileamsu_t4 <fileamsu_t4>

++ geobox

Write amsu_t2 file to netCDF

read :file:`${PROJECT_ID}/AMSU/{yyyy}/{mm}/{numch}_{yyyymmdd}_{geomin}_{geomax}_nadir.dat`
produced by ++:ref:`correct_nadir_amsu.pro`.

read :file:`${PROJECT_ID}/MSG/2006/06/01/msg_IR108_20060601.nc` that will
 be used for regrid.++

 write :file:`${PROJECT_ID}/AMSU/{yyyy}/{mm}/{numch}_{yyyymmdd}_{hhmmss}_{geomin}_{geomax}_nadir.nc`.

.. only:: man

   Figure is visible on PDF and HTML documents only.

.. only:: html or latex

    .. graphviz::

       digraph amsu2ncdf_py {

          amsu_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/yyyy/mm/numch_yyyymmdd_geomin_geomax.dat"];

          amsu_t4 [shape=ellipse,fontname=Courier,label="${PROJECT_OD}/yyyy/mm/amsua_amsub_yyyymmdd_geomin_geomax_nadir.nc"];
          msg [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/2006/06/01/msg_IR108_20060601.nc"];

          amsu2ncdf_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/amsu2ncdf_py.py",
          label="${PROJECT}/src/amsu2ncdf_py.py"];

          {amsu_t2} -> {amsu2ncdf_py} -> {amsu_t4}

         }

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest amsu2ncdf_py.py

Realistic example with VARAMMA file AMSU-B channel 3
:file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat`

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/AMSU//2011/04
   ln -s /homedata/eymard/varamma_d/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat ${PROJECT_ID}/AMSU//2011/04/
   #++ pas sur callisto fileamsu_t2=${PROJECT_ID}/AMSU/2011/04/b3_20110403_022w12s_022e42n.dat'
   fileamsu_t2=${PROJECT_ID}/AMSU/2006/08/a5_20060801_060w30s_050e45n.dat
   fileamsu_t4='ginette.nc'
   amsu2ncdf_py.py --fileamsu_t2 ${fileamsu_t2} --fileamsu_t4 ${fileamsu_t4}
   tlogd.sh amsu2ncdf_py_main

Same in debug mode

.. code-block:: bash

   python -m pdb ${PROJECT}/src/amsu2ncdf_py.py --fileamsu_t2 ${fileamsu_t2} --fileamsu_t4 ${fileamsu_t4}

SEE ALSO
========

:ref:`data_amsu`
++:ref:`data_msg`

Use: ++:func:`read_regular_grid`,
:func:`create_amsu_netcdf_py`, ++
:func:`geolocation_to_string_py`

Called by :func:`traite_amsuab_py` ++

Previous step : :func:`extract_amsu_py` ++

Next step : ... science !

TODO
====

finish migration

prévenir si des missing values dans tps

test with more than one uniq orbite in amsu_t2 file

EVOLUTIONS
==========

- fplod 20140813T093811Z callisto.locean-ipsl.upmc.fr (Linux)

  * going on migration thanks to
    http://stackoverflow.com/questions/6555101/numpy-masked-arrays-indicating-missing-values for mean with missing values

- fplod 20140806T155449Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from amsu2ncdf.pro
    sans msg, sans visu

"""
__all__ = ["amsu2ncdf_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

import numpy

import dateutil.parser

from geolocation_to_string_py import geolocation_to_string_py
from file_amsu_t2_to_mem_py import file_amsu_t2_to_mem_py  # ++ pourquoi
from create_amsu_netcdf_py import create_amsu_netcdf_py


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """read AMSU_T2 file and write to AMSU_T4 (netCDF)"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')
    parser.add_option('--fileamsu_t2',
                      help='filename of AMSU_T2 to be read',
                      dest='fileamsu_t2',
                      metavar='<fileamsu_t2>',
                      action='store')
    parser.add_option('--fileamsu_t4',
                      help='filename of AMSU_T4 to be written',
                      dest='fileamsu_t4',
                      metavar='<fileamsu_t2>',
                      action='store')

    return parser


def amsu2ncdf_py(fileamsu_t2, fileamsu_t4):

    """ ++ geobox
    """

    status = None

    logger = logging.getLogger()

    # ++ne pas utiliser les bornes lon et lat fournies en entrée car elles
    # peuvent être différentes de celle du fichier .dat puisqu'on peut
    # créer un fichier .nc extrait du .dat

    amsu_t2 = file_amsu_t2_to_mem_py(None, None, None, None, None, None, fileamsu_t2)

    hour = amsu_t2['data']['hour']
    fov = amsu_t2['data']['fov']
    lon = amsu_t2['data']['lon']
    lat = amsu_t2['data']['lat']
    tb1 = amsu_t2['data']['tb']

    # décodage du nb de points (nn)
    nn = len(tb1)

    # numero d'orbite
    norb = numpy.unique(amsu_t2['data']['desc'])

    # nombre d'orbites
    nbo = len(norb)

    # grille de sortie xaxis, yaxis :
    # --------------------------------
    xaxis = numpy.arange(((amsu_t2['header2']['lonmax'] - amsu_t2['header2']['lonmin']) / amsu_t2['header1']['resol'] + 1) * amsu_t2['header1']['resol']) + amsu_t2['header2']['lonmin']
    yaxis = numpy.arange(((amsu_t2['header2']['latmax'] - amsu_t2['header2']['latmin']) / amsu_t2['header1']['resol'] + 1) * amsu_t2['header1']['resol']) + amsu_t2['header2']['latmin']
    nlon = len(xaxis)
    nlat = len(yaxis)

    # calcul seuil pour remplir la nouvelle grille
    # --------------------------------------------
    # seuil latitude = max((variation de latitude sur un même fov)/2)
    # pour une orbite quelconque (on choisie la plus longue)
    # moyenne pour fov sur le bord (=min(fov)) et au milieu (=fix(max(fov)/2))
    torb = numpy.array(nbo, dtype='Float32')
    for i in range(0, nbo):
        inutile = numpy.where(amsu_t2['data']['desc'] == norb[i])[0]
        nb = len(inutile)
        if nbo == 1:  # note : python issue 0-d array
            torb = nb
        else:
            torb[i] = nb
    if nbo == 1:  # note : python issue 0-d array
        longorb = norb
    else:
        longorb = norb[numpy.where(torb == max(torb))]

    ind_bor = numpy.where(numpy.logical_and(amsu_t2['data']['fov'] == min(amsu_t2['data']['fov']), amsu_t2['data']['desc'] == longorb[0]))
    dif_bor = abs(lat[ind_bor[1:len(ind_bor) - 1]] - lat[ind_bor[0:len(ind_bor) - 2]])
    # arrondi au 0.1 supérieur pour lisser un peu mais pas trop
    seuil_lat = numpy.ceil(max([dif_bor]) / 2 * 10) / 10.
    logger.info('seuil_lat {0}'.format(seuil_lat))

    # seuil longitude = max((difference de longitude sur une ligne)/2)
    # pour une orbite quelconque (on choisit la plus longue)
    # pour une ligne quelconque (on choisit la plus longue)
    ind_orb = numpy.where(amsu_t2['data']['desc'] == longorb[0])
    dif_lon = lon[ind_orb[1:len(ind_orb) - 1]] - lon[ind_orb[0:len(ind_orb) - 2]]
    # arrondi au 0.1 supérieur pour lisser un peu mais pas trop
    seuil_lon = numpy.ceil(max(dif_lon[numpy.where(dif_lon < 1)]) / 2 * 10) / 10.
    logger.info('seuil_lon {0}'.format(seuil_lon))

    # valeur erreur -999.
    val_err = -999.
    # initialisation
    tbg = numpy.ones((nlon, nlat, nbo), dtype='Float32') * val_err
    temp = numpy.ones((nlon, nlat, nbo), dtype='Float32') * val_err
    hours_orb = numpy.array((nbo), dtype='Float32')

    ### pour chaque orbite ###
    rT = 6371  # rayon de la Terre
    for k in range(0, nbo):
        # trouve tous les indices de l'orbite
        ind_orb = numpy.where(amsu_t2['data']['desc'] == norb[k])
        ### pour chaque latitude ###
        for j in range(0, nlat):
            ### pour chaque longitude ###
            for i in range(0, nlon):
                ind_pt = numpy.where(numpy.logical_and(abs(yaxis[j] - lat[ind_orb]) < seuil_lat, abs(xaxis[i] - lon[ind_orb]) < seuil_lon))
                if ind_pt[0] != -1:
                    ###cherche le plus proche en km###
                    # lon/lat en radian
                    lat_rad = lat[ind_orb[ind_pt]] * numpy.pi/180.
                    lon_rad = lon[ind_orb[ind_pt]] * numpy.pi/180.
                    y_rad = yaxis[j] * numpy.pi/180.
                    x_rad = xaxis[i] * numpy.pi/180.
                    # calcul distance
                    dist = 2 * rT * arcsin(sqrt((sin((y_rad - lat_rad)/2.) * sin((y_rad - lat_rad)/2.))+(cos(x_rad) * cos(lon_rad) * sin((x_rad - lon_rad)/2.) * sin((x_rad - lon_rad)/2.))))
                    # selectionne le plus proche
                    distmin = numpy.where(dist == min(dist))
                    # attribut le point le plus proche
                    tbg[i, j, k] = tb1[ind_orb[ind_pt[distmin]]]
                    temp[i, j, k] = hour[ind_orb[ind_pt[distmin]]]
        tps = temp[:, :, k]
        ind = numpy.where(tps == val_err)[0]  # ++ pourquoi il y aurait des val_er dans les temps ?!
        if nbo == 1:  # note : python issue 0-d array
            hours_orb = numpy.ma.masked_values(tps, val_err).mean()
        else:
            hours_orb[k] = numpy.ma.masked_values(tps, val_err).mean()
        logger.info('orbite {0} finie'.format(norb[k]))

    #
    # Creation du fichier NetCDF :
    # ----------------------------

    look = 'filename'
    scale = 1.
    geomin = geolocation_to_string_py(amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmin'], look, scale)
    geomax = geolocation_to_string_py(amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmax'], look, scale)

    if not fileamsu_t4:
        project_id_env = os.environ['PROJECT_ID']
        yyyymmdd = dateutil.parser.parse('{0}'.format(amsu_t2['header1']['yyyymmdd']), ignoretz=True)
        yyyy = yyyymmdd.strftime('%Y')
        mm = yyyymmdd.strftime('%m')
        fileamsu_t4 = '{0}/AMSU/{1}/{2}/{3}_{4}_{5}_{6}_nadir.nc'.format(project_id_env, yyyy, mm, amsu_t2['header1']['yyyymmdd'], amsu_t2['header1']['numch'], geomin, geomax)

    result = create_amsu_netcdf_py(fileamsu_t4, amsu_t2['header1']['yyyymmdd'], hours_orb, xaxis, yaxis, tbg, val_err, amsu_t2['header1']['numch'])
    # ++ gestion d'erreur

    # comment the next line to produce an image
    #  goto, ret0
    #
    # ++ migration ret0:

    status = True

    return status


def amsu2ncdf_py_main():

    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['fileamsu_t2']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    fileamsu_t2 = fromcli.fileamsu_t2
    fileamsu_t4 = fromcli.fileamsu_t4

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('fileamsu_t2={0}'.format(fileamsu_t2))
    logger.info('fileamsu_t4={0}'.format(fileamsu_t4))

    result = amsu2ncdf_py(fileamsu_t2, fileamsu_t4)
    if not result:
        logger.error('pb amsu2ncdf_py', exc_info=True)
    else:
        logger.info('amsu2ncdf_py reach end; check {0}'.format(fileamsu_t4))

# Run main, if called from the command line
if __name__ == '__main__':
    amsu2ncdf_py_main()
