function [result] = compar_tbnoyau_image(ncfileset, Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup, Tb_LimNoyau_ini, dTb_LimNoyau, Tps_SeuilInf)

%COMPAR_TBNOYAU_IMAGE ouvre des figures permettant de comparer les images
%de précipitations cumulées estimées à partir d'EPSAT avec différentes
%intensités de l'activité nuageuse estimées à partir de MSG, tout en
%faisant varier le seuil de température de brillance délimitant le noyau
%des nuages convectifs.

%
%+
%
% ======================
% compar_tbnoyau_image.m
% ======================
%
% .. function:: compar_tbnoyau_image(Annee, Mois, Jour, Tb_SeuilSup_ini, dTb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * Annee (en caractères) : Année (AAAA) du jour d'étude.
%   * Mois (en caractères) : Numéro du mois (MM) du jour d'étude.
%   * Jour (en caractères) : Numéro du Jour d'étude (JJ).
%   * Tb_SeuilSup : Seuil de température de brillance maximale
%     délimitant un nuage convectif.
%   * Tb_LimNoyau_ini : Seuil de température de brillance délimitant le noyau
%     dans notre modèle à deux couches. Valeur de départ.
%   * dTb_LimNoyau : Variation du seuil de température de brillance du
%     noyau des nuages. Cette variation est appliquée à 4 reprises.
%   * Tps_SeuilInf : Critère de temps minimum de vie d'un nuage pour qu'il
%     soit pris en compte (en nombre de quarts d'heure, 3 par défaut).
%
% Ouverture de figures permettant de comparer les images de précipitations
% cumulées estimées à partir d'EPSAT avec différentes intensités de
% l'activité nuageuse estimées à partir de MSG, tout en faisant varier le
% seuil de température de brillance délimitant le noyau des nuages
% convectifs.
%
% EXAMPLES
% ========
%
% Voir "demo".
%
% SEE ALSO
% ========
%
% :ref:`data_msg`
%
% :ref:`data_epsat`
%
% :func:`MSGread`
%
% :func:`intensityMSG_day`
%
% :func:`plot_intensityMSG`
%
% :func:`EPSATread`
%
% :func:`cumulEPSAT_day`
%
% :func:`plot_cumulEPSAT`
%
% TODO
% ====
%
% externalize EPSATread MSGread (they have new parameters)
%
% check arg
%
% add demo with simul_epsat and simul_msg
%
% save figure
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/compar_tbnoyau_image.m $
%
% - fplod 20110805T141859Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * suppress interactivity
%
% - pinsard 2011-06-08T15:26:57Z loholt1.ipsl.polytechnique.fr (Linux)
%
%   * add box parameter
%
% - fplod 20110309T164608Z aedon.locean-ipsl.upmc.fr (Darwin)
%   
%   * suppress fontsize and fontname (known problem for octave)
%
% - fplod 20110304T084446Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * fix rst header
%
% - jaclod 20110303
%
%   * Ajout de la doc et de la démonstration
%
% - jaclod 20110303
%
%   * Création
%
%-
result=-1;
%
usage='[result] = compar_tbnoyau_image(ncfileset, Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup, Tb_LimNoyau_ini, dTb_LimNoyau, Tps_SeuilInf)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
%%%%%%%%%%%%%
% Paramètres
%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%
% Image du cumule EPSAT
%%%%%%%%%%%%%%%%%%%%%%%%

% On lit le fichier EPSAT correspondant à notre date.
[fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour);

[PluieEPSAT, Temps, Longitudes, Latitudes, Annee, Mois, Jour] = EPSATread(ncfilesset, fullfilename);

% On calcule le cumule des précipitations.
[CumulEPSAT] = cumulEPSAT_day(PluieEPSAT);

% On trace l'image des précipitations cumulées dans une première zone.
subplot(2,3,1);
plot_cumulEPSAT(CumulEPSAT, Annee, Mois, Jour, Longitudes, Latitudes);

% On modifie le titre.
title(['Precipitations cumulees en mm pour le ',...
    ([Annee '-' Mois '-' Jour])])

%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Images des intensités MSG
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% On lit le fichier MSG correspondant à notre date.
[TbMSG, Temps, Longitudes, Latitudes, Annee, Mois, Jour] = MSGread(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax);

% On boucle ensuite pour avoir différent paramètres et, pour chaque cas,
% calculer l'intensité et tracer l'image.
for i=1:5;

    % On calcule l'intensité.
    Tb_LimNoyau = Tb_LimNoyau_ini + (i-1)*dTb_LimNoyau;
    [ij_intensity, Tps_SeuilInf, Tb_SeuilSup, Tb_LimNoyau] = intensityMSG_day(Temps, TbMSG, Tps_SeuilInf, Tb_SeuilSup, Tb_LimNoyau);

    % On trace l'image de l'intensité.
    subplot(2,3,i+1);
    plot_intensityMSG(ij_intensity, Annee, Mois, Jour, Longitudes, Latitudes);

    % On modifie le titre.
    title(['Intensités (Seuil pour le noyau de ' num2str(Tb_LimNoyau) 'K)']);

end

result=0;

end

%!demo
%! varamma_startup;
%! more off
%! Annee = '2006';
%! Mois = '07';
%! Jour = '27';
%! latmin = 10;
%! latmax = 20;
%! lonmin = -20;
%! lonmax = -10;
%! Tb_SeuilSup = 235;
%! Tb_LimNoyau_ini = 180;
%! dTb_LimNoyau = 10;
%! Tps_SeuilInf = 3;
%! result = compar_tbnoyau_image(Annee, Mois, Jour, lonmin, lonmax, latmin, latmax, Tb_SeuilSup, Tb_LimNoyau_ini, dTb_LimNoyau, Tps_SeuilInf);
