#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

========================
create_amsu_netcdf_py.py
========================

DESCRIPTION
===========

Creation du fichier NetCDF contenant AMSU Brightness Temperatures

SEE ALSO
========

Called by :func:`amsu2ncdf_py`

EXAMPLES
========

.. code-block:: idl

   file_ncdf='ginette.nc'
   yyyy=2006
   mm=1
   dd=20
   hh=11
   mn=30
   ss=30
   lons=findgen(10)-10
   lats=findgen(10)+20
   datatab=findgen(10,10)
   masktab=findgen(10,10)
   create_amsu_netcdf, file_ncdf, amsu_t2['header1']['yyyymmdd'], hh, mn, ss, lons, lats, datatab, masktab

TODO
====

finish migration

EXAMPLES

doctest

cli

  * CF 1.5
    checker available on http://puma.nerc.ac.uk/cgi-bin/cf-checker.pl

EVOLUTIONS
==========

$Id$

$URL$

- fplod 20140813T092148Z callisto.locean-ipsl.upmc.fr (Linux)

  * going on migration from IDL to python
    thanks to Calendar functions in http://climate.ncas.ac.uk/~andy/python.html
    and http://nbviewer.ipython.org/github/cedadev/ceda_training_mod5/blob/master/03-Creating_NetCDF.ipynb for time writing

- fplod 20140812T162226Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from create_amsu_netcdf.pro

"""

import logging

import numpy
from netCDF4 import Dataset
from netCDF4 import netcdftime
from netCDF4 import num2date, date2num
from datetime import datetime, timedelta


def create_amsu_netcdf_py(file_ncdf, yyyymmdd, hours, lons, lats, datatab, val_err, numch):

    logger = logging.getLogger()

    #
    # ----------------------------
    # Declaration des variables
    # ----------------------------
    nblondata = len(lons)
    nblatdata = len(lats)
    time_units = 'hours since 1960-01-01 00:00:00'
    cdftime = netcdftime.utime(time_units)
    # ++ migration  yyyymmddhhmmss = yyyymmdd + timedelta(hours=hours)
    print('type hours {0}'.format(type(hours)))
    print('type hours.data {0}'.format(type(hours.data)))
    print('hours.data {0}'.format(hours.data))
    logging.warning('!!!! temps bidon migration')
    try:
        yyyymmddhhmmss = numpy.zeros(len(hours))
        for i in range(0, len(hours)):
            yyyymmddhhmmss[i] = datetime(2014, 01, 01) + timedelta(hours=hours[i])
        time = cdftime.date2num(yyyymmddhhmmss)
    except TypeError:
        yyyymmddhhmmss = datetime(2014, 01, 01) + timedelta(hours=float(hours.data))

    # ----------------------------
    # Ouverture du fichier NetCDF
    # ----------------------------
    #print, 'create file ', file_ncdf
    fid = Dataset(file_ncdf, 'w', format='NETCDF3_CLASSIC')

    # ----------------------------
    # Creation des dimensions
    # ----------------------------
    fid.createDimension('time', None)
    fid.createDimension('lon', nblondata)
    fid.createDimension('lat', nblatdata)

    # ----------------------------
    # Creation des Definitions
    # ----------------------------
    #
    # Definition des attributs globaux :
    # ----------------------------------
    fid.Conventions = 'CF-1.5'
    fid.title = 'AMSU Brightness Temperatures'
    #
    # Variable : Temps :
    # ------------------
    time = fid.createVariable('time', 'Float32', ('time'))
    time.long_name = 'Time'
    time.units = time_units
    time.calendar = 'standard'
    #++ migration time.actual_range = [min(yyyymmddhhmmss), max(yyyymmddhhmmss)], /double
    #
    # Variable : Latitude :
    # ---------------------
    lat = fid.createVariable('lat', 'Float32', ('lat'))
    lat.long_name = 'Latitude'
    lat.units = 'degrees_north'
    lat.actual_range = [min(lats), max(lats)]
    #
    # Variable : Longitude :
    # ----------------------
    lon = fid.createVariable('lon', 'Float32', ('lon'))
    lon.long_name = 'Longitude'
    lon.units = 'degrees_east'
    lon.actual_range = [min(lons), max(lons)]
    #
    # Variable : data :
    # -----------------
    data = fid.createVariable('tb{0}'.format(numch), 'Float32', ('lon', 'lat', 'time'), fill_value=True)   # ++ ordre
    nomcanal_upper = numch.split()[0][0].capitalize()
    nomcanal = numch.split()[0][0]
    data.long_name = 'AMSU-{0} Channel {1} Brightness Temperature'.format(nomcanal_upper, nomcanal)
    data.units = "Kelvin"
    #data.valid_range = [100,500]
    logger.debug('dans create_amsu_netcdf')
    # ++ migration help(datatab)
    datatab_novm = numpy.where(datatab != val_err)
    # ++ migration help(datatab_novm)
    #++ migration data.actual_range = [min(datatab[numpy.where(datatab != val_err)]), max(datatab[numpy.where(datatab != val_err)])]
    data.missing_value = val_err  # ++ regarder si egale a fill_value
    # ++ migration data._FillValue = val_err
    #
    # ----------------------------
    # write variables
    # ----------------------------
    time[:] = date2num(yyyymmddhhmmss, units=time.units, calendar=time.calendar)
    lat[:] = lats
    lon[:] = lons
    data[:] = datatab
    #
    # fermeture du fichier NetCDF :
    # -----------------------------
    # close the file.
    fid.close()

    return True
