function [ij_cumul_concat, time_concat, Longitudes, Latitudes] = cumulEPSAT_concat(ncfilesset, lonmin, lonmax, latmin, latmax, longresol, latresol, Annee, Mois_start, Jour_start, Mois_stop, Jour_stop)

%CUMULEPSAT_CONCAT permet d'analyser les données EPSAT et de concaténer les
%cumuls journaliers résultants. L'analyse s'effectue sur une seule
%saison, sur tous les fichiers ou entre deux dates prédéfinies.

%
%+
%
% ====================
% cumulEPSAT_concat.m
% ====================
%
% .. function:: cumulEPSAT_concat(ncfilesset, lonmin, lonmax, latmin, latmax, longresol, latresol, Annee, Mois_start, Jour_start, Mois_stop, Jour_stop)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * ncfilesset (en caractères) : Type de fichier netCDF que l'on cherche,
%     'normal' ou 'extracted'.
%   * lonmin : Borne inférieure des longitudes du domaine étudié.
%   * lonmax : Borne supérieure des longitudes du domaine étudié.
%   * latmin : Borne inférieure des latitudes du domaine étudié.
%   * latmax : Borne supérieure des latitudes du domaine étudié.
%   * longresol : Nouvelle résolution souhaitée (nombre de points) pour les
%     longitudes (0 pour garder la résolution d'origine).
%   * latresol : Nouvelle résolution souhaitée (nombre de points) pour les
%     latitudes (0 pour garder la résolution d'origine).
%   * Annee : Année de la saison d'étude.
%   * Mois_start (Facultatif) : Mois du premier jour d'étude pour la
%     sélection des fichiers.
%   * Jour_start (Facultatif) : Jour du premier jour d'étude pour la
%     sélection des fichiers.
%   * Mois_stop (Facultatif) : Mois du dernier jour d'étude pour la
%     sélection des fichiers.
%   * Jour_stop (Facultatif) : Jour du dernier jour d'étude pour la
%     sélection des fichiers.
%
% - Données de sorties :
%
%   * ij_cumul_concat : Matrice de données (temps, longitudes,
%     latitudes)contenant les cumuls de précipitations concaténées des
%     différents jours d'étude.
%   * time_concat : Variable de temps, en jour julien, pour identifier les
%     jours étudiés.
%   * Longitudes : Variable de longitudes dans le domaine sélectionné.
%   * Latitudes : Variable de latitudes dans le domaine sélectionné.
%
% Cette fonction permet d'analyser les données EPSAT et de concaténer les
% cumuls de précipitations résultants. L'analyse s'effectue sur une seule
% saison, sur tous les fichiers ou entre deux dates prédéfinies.
%
% EXAMPLES
% ========
%
% Voir la 'demo'.
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :ref:`varamma_startup.m`
%
% :func:`EPSATbuildfullfilename`
% :func:`extractedmatrixdata`
% :func:`spatialresolution`
% :func:`EPSATread`
% :func:`simul_epsat`
% :func:`cumulEPSAT_day`
%
% :func:`nonprecipperiods`
% :func:`MSGanalysiscrossvalidation`
% :func:`missingdaytonan`
%
% :func:`stagejasonjuillet`
%
% TODO
% ====
%
% usage of eomday
%
% improve optional arguments
%
% heavy loop for files handling ... more elegant ?
%
% coding rule and describe demo
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/cumulEPSAT_concat.m $
%
% - fplod 20110805T092658Z cratos (Linux)
%
%   * correct demo call
%   * remove varamma_startup call
%
% - jaclod 2011-07-29
%
%   * Révision finale du stage du mois de juillet 2011.
%
% - jaclod 2011-07-27
%
%   * Revision de la documentation et des commentaires.
%
% - jaclod 2011-07-21
%
%   * Création en se basant sur la fonction intensityMSG_concat déjà
%     existante.
%   * Compatibilité avec des fichiers 'extracted'.
%
%-

%%%%%%%%%%%%%%%%%
% Initialisation
%%%%%%%%%%%%%%%%%

% Appel de la variable global 'PROJECT_ID'.
global PROJECT_ID

usage='[ij_cumul_concat, time_concat, Longitudes, Latitudes] = cumulEPSAT_concat(ncfilesset, lonmin, lonmax, latmin, latmax, longresol, latresol, Annee, Mois_start, Jour_start, Mois_stop, Jour_stop)';
%
if nargin < nargin(mfilename) - 4
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end

% Sauvegarde de la variable 'Annee' sous forme de caractères.
annee = num2str(Annee);

%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Boucles sur les fichiers
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Les boucles se réalisent de différentes façons selon la sélection ou non
% de jours.

% Si le nombre d'arguments est supérieur a 8 c'est qu'une sélection est
% demandée. Sinon, on analyse tous les fichiers présent dans le répertoire
% EPSAT.
if nargin > 8;

    % Creation d'un vecteur comportant le nombre maximum de jours que l'on
    % a dans chaque mois.
    maxjourmois = [31 29 31 30 31 30 31 31 30 31 30 31];

    %%%%%% Boucle sur les mois %%%%%%
    for Mois = Mois_start : Mois_stop;

        % Récupération de la variable 'Mois' sous forme de caractères.
        if Mois < 10;
            mois = ['0' num2str(Mois)];
        else
            mois = num2str(Mois);
        end

        % S'il s'agit du premier mois, le jour de départ sera celui entre
        % en argument, sinon ce sera le premier du mois.
        if Mois == Mois_start;
            Jour_ini = Jour_start;
        else
            Jour_ini = 1;
        end

        % S'il s'agit du dernier mois, le jour de fin sera celui entre
        % en argument, sinon ce sera le dernier jour possible du mois.
        if Mois == Mois_stop;
            Jour_fin = Jour_stop;
        else
            Jour_fin = maxjourmois(Mois);
        end

        %%%%%% Boucle sur les jours %%%%%%
        for Jour = Jour_ini : Jour_fin;

            % Récupération de la variable 'Jour' sous forme de caractères.
            if Jour < 10;
                jour = ['0' num2str(Jour)];
            else
                jour = num2str(Jour);
            end

            % Creation du chemin d'accès complet au fichier correspondant.
            [fullfilename] = EPSATbuildfullfilename(ncfilesset, annee, mois, jour);

            %%%%%% Analyse du fichier %%%%%%

            % Vérification que le fichier du jour existe bien. Si ce n'est
            % pas le cas un message d'avertissement est donne. Sinon, on
            % peut faire l'analyse.
            % Cette vérification diffère selon le type de fichier en raison
            % d'un changement de répertoire et de nom.

            switch ncfilesset

                case 'normal'
                    repertoire = [PROJECT_ID 'EPSAT/' annee '/' mois '/'];

                case 'extracted'
                    repertoire = [PROJECT_ID 'EPSAT/extracted/' annee '/' mois '/'];

                otherwise
                    error('La variable ncfilesset doit etre ''normal'' ou ''extracted''.');

            end

            path(path,repertoire);

            switch ncfilesset

                case 'normal'
                    if exist(['surf-rr_epsat-sg_multi-sat_010d_30min_' annee mois jour '_v3.1-02c.nc'],'file') == 2;

                        % Message confirmant l'analyse du fichier.
                        disp(['iii : Analyse du fichier surf-rr_epsat-sg_multi-sat_010d_30min_' annee mois jour '_v3.1-02c.nc.']);

                        % Lecture du fichier netCDF, récupération des
                        % données et des variables.
                        [PluieEPSAT, Temps, Longitudes, Latitudes] = EPSATread(ncfilesset, fullfilename);

                        % Extraction des données.
                        [PluieEPSAT, Temps, Longitudes, Latitudes] = extractedmatrixdata(PluieEPSAT, Temps, Longitudes, Latitudes, Temps(1), Temps(size(Temps,1)), lonmin, lonmax, latmin, latmax);

                        %%%%%% Changement de résolution %%%%%%
                        % Si longresol et/ou latresol ont pour valeur 0,
                        % alors on récupère la valeur de la résolution
                        % initiale pour ne pas faire de changement.
                        if longresol == 0;
                            longresol = size(PluieEPSAT,2);
                        end

                        if latresol == 0;
                            latresol = size(PluieEPSAT,3);
                        end

                        % Si la résolution demandée est différente de celle
                        % initiale, alors on interpole les données avec la
                        % fonction spatial resolution.
                        if longresol ~= size(PluieEPSAT,2) || latresol ~= size(PluieEPSAT,3);
                            [Longitudes, Latitudes, PluieEPSAT] = spatialresolution(Longitudes, Latitudes, PluieEPSAT, longresol, latresol);
                        end

                        %%%%%% Calcul des cumuls %%%%%%
                        [cumulEPSAT] = cumulEPSAT_day(PluieEPSAT);

                        %%%%%% Concatenation %%%%%%
                        % Sauvegarde des cumuls (la dimension 3 est ici le
                        % temps).
                        if Mois == Mois_start && Jour == Jour_start;
                            ij_cumul_concat = cumulEPSAT;
                        else
                            ij_cumul_concat = cat(3,ij_cumul_concat,cumulEPSAT);
                        end

                        % Sauvegarde de l'information sur la date. On la
                        % sauve en jour julien, il faut donc convertir par
                        % rapport au temps fourni, en jours depuis le 1er
                        % janvier 1960.
                        Tempsjourjulien = Temps(1) + datenum(1960,01,01);
                        if Mois == Mois_start && Jour == Jour_start;
                            time_concat = Tempsjourjulien;
                        else
                            time_concat = cat(2,time_concat,Tempsjourjulien);
                        end

                    else
                        % Message d'avertissement en cas de non presence du
                        % fichier.
                        disp(['www : le fichier surf-rr_epsat-sg_multi-sat_010d_30min_' annee mois jour '_v3.1-02c.nc n''est pas present.']);
                    end

                case 'extracted'

                    if exist(['extracted-surf-rr_epsat-sg_multi-sat_010d_30min_' annee mois jour '_v3.1-02c.nc'],'file') == 2;

                        % Message confirmant l'analyse du fichier.
                        disp(['iii : Analyse du fichier extracted-surf-rr_epsat-sg_multi-sat_010d_30min_' annee mois jour '_v3.1-02c.nc.']);

                        % Lecture du fichier NetCDF, récupération des
                        % données et des variables.
                        [PluieEPSAT, Temps, Longitudes, Latitudes] = EPSATread(ncfilesset, fullfilename);

                        % Pas d'extraction des données.

                        %%%%%% Changement de resolution %%%%%%
                        % Si longresol et/ou latresol ont pour valeur 0,
                        % alors on récupère la valeur de la résolution
                        % initiale pour ne pas faire de changement.
                        if longresol == 0;
                            longresol = size(PluieEPSAT,2);
                        end

                        if latresol == 0;
                            latresol = size(PluieEPSAT,3);
                        end

                        % Si la resolution demandée est différente de celle
                        % initiale, alors on interpole les données avec la
                        % fonction spatial resolution.
                        if longresol ~= size(PluieEPSAT,2) || latresol ~= size(PluieEPSAT,3);
                            [Longitudes, Latitudes, PluieEPSAT] = spatialresolution(Longitudes, Latitudes, PluieEPSAT, longresol, latresol);
                        end

                        %%%%%% Calcul des cumuls %%%%%%
                        [cumulEPSAT] = cumulEPSAT_day(PluieEPSAT);

                        %%%%%% Concatenation %%%%%%
                        % Sauvegarde des cumuls (la dimension 3 est ici le
                        % temps).
                        if Mois == Mois_start && Jour == Jour_start;
                            ij_cumul_concat = cumulEPSAT;
                        else
                            ij_cumul_concat = cat(3,ij_cumul_concat,cumulEPSAT);
                        end

                        % Sauvegarde de l'information sur la date. On la
                        % sauve en jour julien, il faut donc convertir par
                        % rapport au temps fourni, en jour depuis le 1er
                        % janvier 1960.
                        Tempsjourjulien = Temps(1) + datenum(1960,01,01);
                        if Mois == Mois_start && Jour == Jour_start;
                            time_concat = Tempsjourjulien;
                        else
                            time_concat = cat(2,time_concat,Tempsjourjulien);
                        end

                    else
                        % Message d'avertissement en cas de non presence du
                        % fichier.
                        disp(['www : le fichier extracted-surf-rr_epsat-sg_multi-sat_010d_30min_' annee mois jour '_v3.1-02c.nc n''est pas present.']);
                    end

                otherwise
                    % Cas impossible. Erreur déjà donnée.
            end

        end

    end

else

    %%%%% Recherche des mois de données %%%%%
    % Répertoire correspondant aux fichiers EPSAT de l'année en question
    % différents selon le type de fichier.
    switch ncfilesset

        case 'normal'
            repertoireEPSATsaison = [PROJECT_ID 'EPSAT/' annee];

        case 'extracted'
            repertoireEPSATsaison = [PROJECT_ID 'EPSAT/extracted/' annee];

        otherwise
    end

    path(path,repertoireEPSATsaison);

    % On cherche les dossier dans ce dernier répertoire correspondant aux
    % mois dont nous disposons des données.
    listdir = dir(repertoireEPSATsaison);

    % Nombre de dossier. Attention les deux premiers résultats sont '.' et
    % '..' et ne compte donc pas comme dossier de fichiers.
    nbdir = numel(listdir)-2;

    %%%%% Boucle sur les mois %%%%%
    for numdir = 1 : nbdir;

        % Récupération du numero du mois sous forme de caractères.
        mois = listdir(numdir+2).name;

        % Répertoire du mois correspondant.
        repertoireEPSATmois = [repertoireEPSATsaison '/' mois];

        path(path,repertoireEPSATmois);

        %%%%% Recherche des fichiers netCDF %%%%%
        ext = '*.nc';
        chemin = fullfile(repertoireEPSATmois,ext);
        listncfiles = dir(chemin);

        nbfiles = numel(listncfiles);

        %%%%% Boucle sur les fichiers, les jours %%%%%
        for numfiles = 1 : nbfiles;

            % Récupération du nom du fichier et de son chemin d'accès
            % complet.
            fullfilename = [repertoireEPSATmois '/' listncfiles(numfiles).name];

            % Message confirmant l'analyse du fichier.
            disp(['iii : Analyse du fichier ' listncfiles(numfiles).name '.']);

            % Lecture du fichier NetCDF, récupération des données et
            % des variables.
            [PluieEPSAT, Temps, Longitudes, Latitudes] = EPSATread(ncfilesset, fullfilename);

            switch ncfilesset

                case 'normal'
                    % Extraction des données.
                    [PluieEPSAT, Temps, Longitudes, Latitudes] = extractedmatrixdata(PluieEPSAT, Temps, Longitudes, Latitudes, Temps(1), Temps(size(Temps,1)), lonmin, lonmax, latmin, latmax);

                case 'extracted'
                    % Pas d'extraction.

                otherwise
            end

            %%%%%% Changement de résolution %%%%%%
            % Si longresol et/ou latresol ont pour valeur 0, alors on
            % récupère la valeur de la résolution initiale pour ne pas
            % faire de changement.
            if longresol == 0;
                longresol = size(PluieEPSAT,2);
            end

            if latresol == 0;
                latresol = size(PluieEPSAT,3);
            end

            % Si la résolution demandée est différente de celle
            % initiale, alors on interpole les données avec la fonction
            % spatial resolution.
            if longresol ~= size(PluieEPSAT,2) || latresol ~= size(PluieEPSAT,3);
                [Longitudes, Latitudes, PluieEPSAT] = spatialresolution(Longitudes, Latitudes, PluieEPSAT, longresol, latresol);
            end

            %%%%%% Calcul des cumuls %%%%%%
            [cumulEPSAT] = cumulEPSAT_day(PluieEPSAT);

            %%%%%% Concatenation %%%%%%

            % Sauvegarde des cumuls (la dimension 3 est ici le temps).
            if numdir==1 && numfiles==1;
                ij_cumul_concat = cumulEPSAT;
            else
                ij_cumul_concat = cat(3,ij_cumul_concat,cumulEPSAT);
            end

            % Sauvegarde de l'information sur la date. On la sauve en jour
            % julien, il faut donc convertir par rapport au temps fourni,
            % en jours depuis le 1er janvier 1960.
            Tempsjourjulien = Temps(1) + datenum(1960,01,01);
            if numdir==1 && numfiles==1;
                time_concat = Tempsjourjulien;
            else
                time_concat = cat(2,time_concat,Tempsjourjulien);
            end

        end

    end

end

% On passe le temps en première dimension pour plus de concordance avec les
% autres variables.
ij_cumul_concat = permute(ij_cumul_concat,[3 1 2]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

%!demo
%! clear all
%! close all
%! varamma_startup
%! more off
%! [ij_cumul_concat, time_concat, Longitudes, Latitudes] = cumulEPSAT_concat('normal',-20, 0, 0, 20, 200, 200, 2006, 07, 26, 07, 31);
