function [CumulEPSAT] = cumulEPSAT_day(PluieEPSAT)

%CUMULEPSAT_DAY calcule le cumul des précipitations estimées d'après les
%données EPSAT sur tous les pas de temps (première dimension).

%
%+
%
% ================
% cumulEPSAT_day.m
% ================
%
% .. function:: cumulEPSAT_day(PluieEPSAT)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * PluieEPSAT : Données brutes de l'estimation des précipitations dont
%     les dimensions sont respectivement temps, longitudes et latitudes.
%
% - Données de sorties :
%
%   * CumulEPSAT : Cumul de précipitations sur tous les pas de temps dont
%     les dimensions sont respectivement longitudes et latitudes.
%
% Calcul du cumul des précipitations estimées d'apres les données EPSAT,
% sur tous les pas de temps.
%
% Dans le cas de données manquantes, une interpolation est réalisée.
%
% Plot of output data can be made using :func:`plot_cumulEPSAT`.
%
% EXAMPLES
% ========
%
% To compute cumul of precipitation from simulated EPSAT data:
%
% see demo 1
%
% To compute cumul of precipitation from a real EPSAT file:
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :ref:`varamma_startup.m`
%
% :func:`EPSATbuildfullfilename`
% :func:`EPSATread`
% :func:`extractedmatrixdata`
% :func:`simul_epsat`
%
% :func:`cumulEPSAT_concat`
% :func:`plot_cumulEPSAT`
%
% TODO
% ====
%
% hard coded missing value : should be set to NaN by EPSATread.
%
% argument checking dim, type
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/cumulEPSAT_day.m $
%
% - jaclod 2011-07-29
%
%   * Révision finale du stage du mois de juillet 2011.
%   * Modification du second exemple en raison de l'externalisation de
%     fonctions.
%
% - jaclod 2011-07-26
%
%   * Révision de la documentation et des commentaires.
%
% - jaclod 2011-07-22
%
%   * Correction de la valeur de données manquantes. Ce n'est plus -1 mais
%     -0.1 car le facteur d'échelle a maintenant été appliqué.
%   * Après vérification, AbsDonnees = find(PluieEPSAT(:,1,1) == -0.1) ne
%     fonctionne pas pour le jour test du 2 août 2006 alors que les données
%     du pas de temps 34 (16:45) sont -0.1 ...
%     Changement pour AbsDonnees = find(PluieEPSAT(:,1,1) < 0) qui
%     fonctionne.
%
% - pinsard 2011-06-08T14:28:49Z loholt1.ipsl.polytechnique.fr (Linux)
%
%   * correction of dimension order on data (time lon lat)
%
% - fplod 20110311T103227Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * udpate description
%   * squeeze for output 2D array (not 3D needed - no more time dimension)
%     if not pc
%
%     here is an example of sum on the first dimension of a 3D array on matlab
%     and octave on unix ::
%
%     .. code-block:: matlab
%
%        tab=zeros(2,3,4);
%        somme=sum(tab,1)
%        whos somme
%
%     .. parsed-literal::
%
%        Variables in the current scope:
%
%          Attr Name        Size                     Bytes  Class
%          ==== ====        ====                     =====  =====
%               somme       1x3x4                       96  double
%
%        Total is 12 elements using 96 bytes
%
%     while on pc (see jaclod on his pc with matlab 2010) somme is a 2D array !
%
% - fplod 20110310T155600Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add demo with :func:`simul_epsat`
%   * fix an error during suppression of permutation
%   * fix problem when no missing data
%
% - fplod 20110310T082921Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add reference to :func:`plot_cumulEPSAT`
%
% - fplod 20110309T155913Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * no more permutation needed (see :func:`EPSATread`)
%   * add some clean
%
% - fplod 20110304T085058Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * fix rst header
%
% - jaclod 2011-03-02
%
%   * Ajout de la documentation et de la demonstration.
%
% - jaclod 2011-02-25
%
%   * Creation
%
%-
usage='[CumulEPSAT] = cumulEPSAT_day(PluieEPSAT)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
%%%%%%%%%%%%%%%%
% Interpolation
%%%%%%%%%%%%%%%%

% On cherche pour quels pas de temps les données manquent (les données
% inférieures à 0 dont les données manquantes égales a -0.1).
AbsDonnees = find(PluieEPSAT(:,1,1) < 0);

if size(AbsDonnees,1)~=0;

    % On définit x, les abscisses de la fonction d'interpolation dont les
    % valeurs sont connues et xi, l'ensemble des abscisses connues ou non.
    xi = 1 : size(PluieEPSAT,1);
    x = xi;
    x(AbsDonnees) = 0;
    x = x(x~=0);

    % On supprime, dans la matrice de données, les valeurs manquantes (en
    % partant de la fin, sinon on ne supprime pas les bons pas de temps).
    for i = 1 : size(AbsDonnees,1)

        PluieEPSAT(AbsDonnees(size(AbsDonnees,1)-(i-1)),:,:) = [];

    end

    clear i

    PluieEPSAT_interp = interp1(x,PluieEPSAT,xi);

    clear x
    clear xi

    % Cumul des précipitations.
    CumulEPSAT = sum(PluieEPSAT_interp,1);
    clear PluieEPSAT_interp

else

    CumulEPSAT = sum(PluieEPSAT,1);

end

clear AbsDonnees

% Suppression de la dimension temporelle potentiellement résiduelle.
CumulEPSAT = squeeze(CumulEPSAT);

end

%!demo
%! % To compute cumul of precipitation from simulated EPSAT data:
%!
%! clear all
%! close all
%! varamma_startup
%! more off
%! index_simulation = uint8(2);
%! [lon_value, lat_value, datejul_value, datestr_value, epsat_value] = simul_epsat(index_simulation);
%! [CumulEPSAT] = cumulEPSAT_day(epsat_value);
%! whos

%!demo
%! % To compute cumul of precipitation for EPSAT file
%! % :file:`${PROJECT_ID}/EPSAT/2006/08/surf-rr_epsat-sg_multi-sat_010d_30min_200600801v3.1-02c.nc`::
%! varamma_startup;
%! more off
%! ncfilesset = 'normal';
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour);
%! [data, Temps, Longitudes, Latitudes] = EPSATread('normal', fullfilename);
%! mindim1 = min(Temps);
%! maxdim1 = max(Temps);
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! [data, Temps, Longitudes, Latitudes] = extractedmatrixdata(data, Temps, Longitudes, Latitudes, mindim1, maxdim1, lonmin, lonmax, latmin, latmax);
%! [Cumuldata] = cumulEPSAT_day(data);
%! whos
