; docformat = 'rst'
;+
;
; .. _extract_amsu.pro:
;
; =================
; extract_amsu.pro
; =================
;
; DESCRIPTION
; ===========
;
; prgm de lecture des fichiers AMSU (A et B)
;
; décode les noms des fichiers donnés en argument dans **files_list**
; dans la date choisie, puis appelle le prgm de lecture
;
; appelle :ref:`interpol_correc.pro`, qui fournit les fonctions de correction au
; nadir et les applique, en fonction du masque terre-mer issu de la
; carte de bathymétrie de S. Masson
;
; applique une interpolation tenant compte de la dimension de la tache
; au sol selon la position dans la fauchée
;
; écrit les données pour la zone (lat/long) choisie
;
; ++ le fichier de sortie contient dans l'ordre mm, jj (jour de
; l'annee), le temps (identique pour les points d'une même fauchée), le
; no satellite, le no de fov (1 a 40 ou 1 a 90), long, lat, tb
;
; Les valeurs manquantes sont codées par NaN.
;
; .. only:: man
;
;    Figure is visible on PDF and HTML documents only.
;
; .. only:: html or latex
;
;     .. graphviz::
;
;        digraph extract_amsu {
;
;           amsu_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/AMSU/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];
;           amsu_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/yyyy/mm/numch_yyyymmdd_geomin_geomax.dat"];
;
;           extract_amsua [shape=box,
;           fontname=Courier,
;           color=blue,
;           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/extract_amsua.pro",
;           label="${PROJECT}/src/extract_amsua.pro"];
;
;           {amsu_t1} -> {extract_amsua} -> {amsu_t2};
;       }
;
; :param numch:
; :param files_list:
; :param yyyy:
; :param mm:
; :param resol:
; :param lon_min:
; :param lon_max:
; :param lat_min:
; :param lat_max:
;
; EXAMPLES
; ========
;
; Creating an artificial list of AMSU-A files:
;
; .. code-block:: idl
;
;    files_list = strarr(2)
;    files_list[0] = project_id_env + '/AMSU/AMSUAN15/L1C/2006/2006_08_01/NSS.AMAX.NK.D06213.S0110.E0255.B4271112.GC.L1C'
;    files_list[1] = project_id_env + 'AMSU/AMSUAN15/L1C/2006/2006_08_01/NSS.AMAX.NK.D06213.S0250.E0437.B4271213.GC.L1C'
;
; This above tip can replace the call to :func:`search_amsufiles`.
;
; Pour ne garder que le dernier fichier:
;
; .. code-block:: idl
;
;    files_list = search_amsufiles(numch, yyyy, mm, dd)
;    help, files_list
;    files_list_last_array = strarr(1)
;    files_list_last_array[0] = files_list[N_ELEMENTS(files_list) - 1]
;    help, files_list_last_array
;    files_list = files_list_last_array
;    help, files_list
;
; Using AMSU-A channel 5:
;
; .. code-block:: idl
;
;    numch = 'a5'
;    yyyy=2006L
;    mm=8
;    dd=13
;    resol=1
;    lon_min=-25.
;    lon_max=25.
;    lat_min=-5.
;    lat_max=20.
;    files_list = search_amsufiles(numch, yyyy, mm, dd)
;    extract_amsu, numch, files_list, yyyy, mm, dd, resol, lon_min, lon_max, lat_min, lat_max
;    result = file_bathy_to_mem(lon_min, lon_max, lat_min, lat_max)
;    extract_amsua, numch, files_list, yyyy, mm, dd, resol, lon_min, lon_max, lat_min, lat_max
;
; :file:`${PROJECT_ID}/AMSU/2006/08/a5_20060813_025w05s_025e20n.dat`
; must have been created.
;
; SEE ALSO
; ========
;
; :ref:`data_amsu`
;
; called by :ref:`traite_amsuab.sh`
;
; previous step : :func:`search_amsufiles`
;
; use :ref:`read_amsua1c.pro`, :ref:`read_amsub1c.pro`,
; :func:`geolocation_to_string_idl`, :func:`mem_to_file_amsu_t2`
;
; next step :  ++
;
; TODO
; ====
;
; track why end with:
;
; .. parsed-literal::
;
;    % Program caused arithmetic error: Floating illegal operand
;
; traiter les longitudes autour de 180 degré (passage de 180 à -180)
; interpolswath ne gère pas cette situation
;
; lever le doute sur le contenu du fichier écrit par le printf (je (fplod)
; crains des zéros louches!!).louche
;
; ajouter MHS
;
; éviter la création d'un fichier vide (si pas de points dans la zone)
;
; tester la cohérence entre les fichiers présents dans files_list
; en terme de date avec yyyy, mm et dd donnés en paramètre
;
; improve log
;
; get rid of uppercase
;
; check args
;
; décrire la limitation avec les seuils
;
; EVOLUTIONS
; ==========
;
; $Id: extract_amsua.pro 513 2012-04-10 14:50:06Z pinsard $
;
; $URL: svn+ssh://lelod@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/extract_amsua.pro $
;
; - lelod 20120718
;
;   * extension de extract_amsua à amsu a et b validée sur un cas test
;     en direct (sans passer par traite_amsuab.sh) avec b3
;
; - lelod 20120418
;
;   *  neutralisation de la variable desc: remplacement par numéro de l'orbite lue
;
; - fplod 20120323
;
;   * homogenize examples to traite_amsuab.sh
;   * fix output name in example
;   * add a paramater files_list to remplace usage of :file:`list_file`
;   * replace usage of :file:`list_file` by the usage of the string array
;     files_list
;   * add a test if files_list no empty (see nofile label)
;   * reorder SEE ALSO and TODO
;
; - lelod 20120223
;
;   * modif lecture du fichier bathy (ETOPO) pour limiter le temps
;     nécessaire. Introduction de coordonnées limite dans initncdf, puis
;     domdef
;
;     Attention, si on change la zone, il faut changer les coordonnées!!!
;
; - lelod 20111215
;
;   * tests de fonctionnement effectués: la correction nadir est
;     correctement implémentée
;
;     l'interpolation de la fauchée fonctionne
;     mais attention test sur la longitude code en dur:
;     pbs avec les fauchées qui traversent le méridien 180 deg !!!!
;     l'interpolation ajoute des points parasites dans la zone ...
;
; - fplod 20111214T082300Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * remove old style writing output file
;   * forget idldoc
;
; - fplod 20111213T092550Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * fix syntax error : rewrite labfile label
;   * fix nocanal string to integer type
;   * fix example (missing resol in call)
;
; - lelod 20111211
;
;   * réintègre les modules de correct_nadir_amsu dans extract
;     prévu décodage de numch pour connaître le nom instrument et le
;     numéro du canal (pas fini, au début) --> à faire aussi dans les
;     ssprgm appelés (interpol_correc)
;     espère que c'est le dernier avatar de cette !!!!?XX^! de
;     chaîne de traitement AMSU
;
; - fplod 20111209T125312Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * chgt de terminologie
;   * ajout paramètre numch
;   * transition pour utilisation de :ref:`mem_to_file_amsu_t2`
;
; - lelod 20111121
;
;   * introduction du paramètre "descending" (orbite descendante 1,
;     montante 0) dans le fichier de sortie : à tester !
;
; - fplod 20110818T160435Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * no more /bdd in examples
;   * need to check if missing exist before NaN affection on idl7 cratos
;
; - pinsard 2011-08-18T10:34:53Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * correction of examples
;   * examples with 2 files in list_filea and list_fileb
;   * match technique from
;     http://cerebus.as.arizona.edu/~ioannis/teaching/idl/tutorial_strings.html
;   * révision du cas use_amsua=1 : la liste de fichiers amsu-a n'est ouverte
;     qu'une seule fois
;
; - pinsard 2011-08-18T09:10:08Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * move bad values from -9999.99 to NaN
;
; - pinsard 2011-08-17T14:58:13Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * use geolocation_to_string_idl
;
; - pinsard 2011-08-10T10:46:37Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * remove blank in output filename
;   * revision of artificial example with use_amsua=0
;   * add artificial example with use_amsua=1 and try (but fail) to
;     make it work on loholt1
;   * remove length path dependencies using IDL function file_basename
;   * use same date and box of examples in correct_nadir_amsu.pro
;
; - fplod 20110810T085644Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * revision of header
;
; - pinsard 2011-05-26T08:44:38Z loholt1.ipsl.polytechnique.fr (Linux)
;
;   * strmid(fich,51) is not anymore pertinent to isolate the correct information
;
;     It was ok when path as /bdd/AMSU-1C/AMSUAN15/L1C/2006/2006_07_28/:
;
;     .. code-block:: idl
;
;        c='/bdd/AMSU-1C/AMSUAN15/L1C/2006/2006_07_28/NSS.AMAX.NK.D06209.S1323.E1455.B4266162.GC.L1C'
;        b=strmid(c,51)
;        print, b
;        NK.D06209.S1323.E1455.B4266162.GC.L1C
;
;     but now path can be anything (PROJECT_ID environment variable)
;     so the correct way is to catch the last 36 characters
;
;     test sur skylla: fonctionne au moins pour un fichier!
;
;-
PRO extract_amsu, numch, files_list, yyyy, mm, dd, resol $
                 , lonmin, lonmax, latmin, latmax
;
compile_opt idl2, strictarrsubs
;
@cm_project
@common
s = size(SCOPE_TRACEBACK(/STRUCTURE),/DIMENSION)
routine = (SCOPE_TRACEBACK(/STRUCTURE))[s-1].Routine
key_performance=0

if key_performance EQ 1 THEN BEGIN
    time1 = SYSTIME(1)
ENDIF
;
; test if some files to read
CASE size(files_list,/DIMENSION) OF
   0L : BEGIN
     print, 'pas de fichiers'
     goto, nofile
        END
   ELSE : BEGIN
       print, 'nb de fichiers', size(files_list,/DIMENSION)
        END
ENDCASE
nb_file_array = size(files_list,/DIMENSION)
nb_file = (nb_file_array)[0]
;
nomcanal=strmid(numch,0,1)
nocanal=0
reads, strmid(numch,1,1),nocanal,format='(i1.1)'

tbmin=100
tbmax=350
PRINT, 'www : début programme extract_amsu',SYSTIME()

jpie = n_elements(xxe)
jpje = n_elements(yye)

; appel au ssprgm qui interpole les fonctions de correction (inutile
; de l'appeler à chaque fichier....)
; ou appeler pixelsize et réduire interpolcorrec à la partie
; correction nadir
interpol_correc,numch,nbpixel,cor_l,cor_s,swath,track
; ouverture des fichiers liste (année, mois, jour, tous satellites) pour
; chaque instrument
; boucle sur les éléments de la liste
desc=0
msg = report(['iii : ' + routine + ' : traitement du jour' $
      + string(yyyy, format='(1x,I4.4)') $
      + string(mm, format='(I2.2)') $
      + string(dd, format='(I2.2)')])
a = STRARR(nb_file)

IF key_performance EQ 1 THEN BEGIN
    msg = report(['ppp : ' + routine + ' :  durée intermédiaire démarrage boucle sur fichier ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
ENDIF

FOR ifile = 0, nb_file - 1 do begin
   if nomcanal eq 'a' then begin
      filea = files_list[ifile]
      COMMON amsua_header,ama_head
      COMMON amsua_data  ,ama_scan
   IF key_performance EQ 1 THEN BEGIN
       msg = report(['ppp : ' + routine + ' : durée intermédiaire ouverture et lecture du fichier' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
   ENDIF
      read_amsua1c,filea, flag1
      if (flag1 eq 0) then goto, labfile
      na=SIZE(reform(ama_scan.btemps[0,*,*]))
      na_scan=na[2]             ; nb lignes (fauchées) dans l'orbite
      n_scan=na_scan
      na_fov=na[1]              ; nb points dans la fauchée
      n=na
      nbpix= na_fov
      n_fov=na_fov
      nosat=ama_head.h_satid    ; numéro du satellite NOAA (15 a 19) ou METOP 2
     ; print,'NUMERO DU SATELLITE ',nosat
     ; print,'INSTRUMENT',ama_head.h_instrument
; on lit la structure et on extrait les infos: temps, longitude,
; latitude, et les Tbs des différents canaux (15 pour AMSUA)
      amalong=REFORM(ama_scan.latlon[1,*,*]/1.E4)
      amalati=REFORM(ama_scan.latlon[0,*,*]/1.E4)
      domain=where(amalong gt lonmin and amalong lt lonmax and amalati gt latmin and amalati lt latmax,ndom)
      msg = report(['iii : ' + routine + ' : nb de points du fichier dans le domaine géographique ' + string(ndom,format='(I8.8)')])
      IF key_performance EQ 1 THEN BEGIN
         msg = report(['ppp : ' + routine + ' : durée intermédiaire avant test ndom' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
      ENDIF
      if ndom eq 0 then goto, labfile

      amcha=REFORM(ama_scan.btemps[nocanal-1,*,*]/100.)
      amzeni=REFORM(ama_scan.angles[0,*,*]/100.)
      ttt=REFORM(ama_scan.scnlintime/3600000.)
      amaday=REFORM(ama_scan.scnlindy)
   endif
   if nomcanal eq 'b' then begin
      fileb = files_list[ifile]
      COMMON amsub_header,amb_head
      COMMON amsub_data  ,amb_scan
   IF key_performance EQ 1 THEN BEGIN
       msg = report(['ppp : ' + routine + ' : durée intermédiaire ouverture et lecture du fichier' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
   ENDIF
    read_amsub1c,fileb, flag2
     if (flag2 eq 0) then goto, labfile
      nb=SIZE(reform(amb_scan.btemps[0,*,*]))
      nb_scan=nb[2]             ; nb lignes (ou fauchées)
      n_scan=nb_scan
      nb_fov=nb[1]              ; nb pixels dans la fauchée
      nbpix = nb_fov          ; nb pixels dans la fauchée AMSUB
; extraction du canal traité dans un domaine enveloppant la zone choisie
      nosat=amb_head.h_satid    ; numéro du satellite NOAA (15 a 19) ou METOP 2
      print,'NUMERO DU SATELLITE ',nosat
      print,'INSTRUMENT',amb_head.h_instrument
; on lit la structure et on extrait les infos: temps, longitude,
; latitude, et les Tbs
      amalong=REFORM(amb_scan.latlon[1,*,*]/1.E4)
      amalati=REFORM(amb_scan.latlon[0,*,*]/1.E4)
      domain=where(amalong ge lonmin and amalong le lonmax and amalati ge latmin and amalati le latmax,ndom)
      msg = report(['iii : ' + routine + ' : nb de points du fichier dans le domaine géographique ' + string(ndom,format='(I8.8)')])
      IF key_performance EQ 1 THEN BEGIN
         msg = report(['ppp : ' + routine + ' : durée intermédiaire avant test ndom' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
      ENDIF
      if ndom eq 0 then goto, labfile

      amcha=REFORM(amb_scan.btemps[nocanal-1,*,*]/100.)
      amzeni=REFORM(amb_scan.angles[0,*,*]/100.)
      ttt=REFORM(amb_scan.scnlintime/3600000.)
      amaday=REFORM(amb_scan.scnlindy)
   endif

   ntime_1=fltarr(nbpix,n_scan)
   fovy_1=fltarr(nbpix,n_scan)
   for j=0L, nbpix-1L do begin
      fovy_1[j,*]=j+1           ; on remplit un tableau de nscan lignes avec les valeurs de no de pixel
   endfor
; extraction du canal traité dans un domaine enveloppant la zone choisie
   midpix=fix(nbpix/2)
   jnd=where(amalong[midpix,*] gt lonmin-15 and amalong[midpix,*] lt lonmax+15 and amalati[midpix,*] gt latmin-15 and amalati[midpix,*] lt latmax+15 ,nzon)
   msg = report(['iii : ' + routine + ' : nb de points du fichier dans le domaine étendu +15deg ' + string(nzon,format='(I8.8)')])
;   IF key_performance EQ 1 THEN BEGIN
;       msg = report(['ppp : ' + routine + ' : durée intermédiaire avant test nzon' $
;                   + string(SYSTIME(1)-time1,format='(F12.6)')])
 ;  ENDIF
   if nzon gt 1 then begin
      amalat=amalati[*,jnd]
      amalon=amalong[*,jnd]
      amch=amcha[*,jnd]
      amzen=amzeni[*,jnd]
      tt=ttt[jnd]
      dims = SIZE(amalon, /DIMENSIONS)
      print,'dimension tableaux extraits',dims
      n_scan=dims[1]
      PRINT,'il y a des données dans la region '
      amafov= fovy_1
      jour_ama=amaday[0]        ; jour du passage
      desc=desc+1
;
; correction nadir des données. En principe le 17 est éliminé des listes
         isat=0
         if nosat eq 15 then begin
             isat=0
             print,'no satellite',nosat, isat
         endif
         if nosat eq 16 then isat=1
         if nosat eq 18 then isat=2
         if nosat eq 19 then isat=3
         if nosat eq 2 then isat=4

      ch_nadir=fltarr(nbpix,nzon)
      landseamask=intarr(nbpix,nzon)+2 ; valeur hors zone sélectionnée
         IF key_performance EQ 1 THEN BEGIN
              msg = report(['ppp : ' + routine + ' : durée intermédiaire boucle sur les points du fichier, correction nadir ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
         ENDIF

      for isc=0L,nzon-1L do begin
;recherche de la zone xxe,yye englobant la fauchée
         utilex=where(xxe ge amalon[midpix,isc]-15 and xxe le amalon[midpix,isc]+15,nxu)
         if nxu ne 0 then begin
            xxutile=xxe[utilex]
            batutilx=bate[utilex,*]
            utiley=where(yye ge amalat[midpix,isc]-8 and yye le amalat[midpix,isc]+8,nyu)
            if nyu ne 0 then begin
               yyutile=yye[utiley]
               batutil=batutilx[*,utiley]

               for ifo=0,nbpix-1 do begin
; approximation: 1km= 1deg/100, et on considère le quart en lon et lat
; de la surface du pixel pour optimiser la localisation du centre du
; pixel attention approximation brutale des distances en fractions de degré
; recherche de la côte par rapport à la résolution AMSUA
                  xind=where (xxutile ge amalon[ifo,isc]-swath[ifo]/400. and xxutile le amalon[ifo,isc]+swath[ifo]/400.,nxlandsea)
                  yind=where (yyutile ge amalat[ifo,isc]-track[ifo]/400. and yyutile le amalat[ifo,isc]+track[ifo]/400.,nylandsea)
                  if(nxlandsea ne 0 and nylandsea ne 0) then begin
                     bate1=batutil[xind,*]
                     bate2=bate1[*,yind]
                     if (mean(bate2) le 0.5) then begin
                        ch_nadir[ifo,isc]=amch[ifo,isc]-cor_s[ifo,isat]
                        landseamask[ifo,isc]=0 ; mer
                     endif else begin
                        ch_nadir[ifo,isc]=amch[ifo,isc]-cor_l[ifo,isat]
                        landseamask[ifo,isc]=1 ; terre
                     endelse
                  endif
               endfor
            endif
         endif

      endfor
        IF key_performance EQ 1 THEN BEGIN
             msg = report(['ppp : ' + routine + ' : durée intermédiaire fin boucle ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
        ENDIF
      moych=fltarr(nbpix)
;print,'valeurs extrêmes tb',min(amch),min(ch_nadir),max(amch),max(ch_nadir)
; appel à interpolswath pour ajuster les pixels amsua sur une grille
; régulière dans la fauchée
; et sélection de la zone conservée
      nbgrid=0
      cont=0L
      chint=fltarr(1)
      for i=0L,nzon-1L do begin
         tb=ch_nadir[*,i]
         lon=amalon[*,i]
         lat=amalat[*,i]
         mask=landseamask[*,i]
         ind=where(tb gt tbmin,nbon)
; test pour éviter pb d'interpolation de longitude (méridien 180)
         if nbon eq nbpix then begin
            cont=cont+1L
            nbgrid1=nbgrid
            interpolswath,tb,lat,lon,mask,nomcanal,resol,nbgrid,tbgrid,latgrid,longrid,maskgrid
            fovgrid=indgen(n_elements(tbgrid))+1
;print,'wwwtest: nb points fov interpolés ',n_elements(tbgrid)
; sélection des taches au sol situées dans la zone d'intérêt
            zone=where((longrid ge lonmin) and (longrid le lonmax) $
                 and (latgrid ge latmin) and (latgrid le latmax) and (tbgrid gt tbmin) and (tbgrid lt tbmax), npt)
;print,'wwwtest: nb points zone d intérêt ',npt
            if npt ne 0 then begin
               if (n_elements(chint) LE 1) then begin
                  timeint=replicate(tt[i],npt)
                  chint=tbgrid[zone]
                  latint=latgrid[zone]
                  lonint=longrid[zone]
                  fovint=fovgrid[zone]
                  maskint=maskgrid[zone]
               endif else begin
                  chint=[chint,tbgrid[zone]]
                  latint=[latint,latgrid[zone]]
                  lonint=[lonint,longrid[zone]]
                  fovint=[fovint,fovgrid[zone]]
                  maskint=[maskint,maskgrid[zone]]
                  timeint=[timeint,replicate(tt[i],npt)]
               endelse
            endif else begin
                                ;print, 'www : aucun point dans la zone'
            endelse
         endif
      endfor
         IF key_performance EQ 1 THEN BEGIN
             msg = report(['ppp : ' + routine + ' : durée intermédiaire fin interpolation et sélection des points de la zone ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
         ENDIF

      nn=n_elements(chint)
         msg = report(['www : ' + routine + ' : nb points dans la zone en fin de traitement de l''orbite ' + string(nn,format='(I10.10)')])
      if nn gt 1 then begin
         for i=0L, nn-1L do begin
            if (n_elements(data) EQ 0) then begin
               data = { desc : desc $
                        , hour : timeint[i] $
                        , fov : fovint[i] $
                        , lon : lonint[i] $
                        , lat: latint[i] $
                        , landseamask: maskint[i] $
                        , tb : chint[i] $
                      }
            endif else begin
               data = [data , { desc : desc $
                                , hour : timeint[i] $
                                , fov : fovint[i] $
                                , lon : lonint[i] $
                                , lat: latint[i] $
                                , landseamask:  maskint[i] $
                                , tb : chint[i] $
                              }]
            endelse
         endfor
         desc=desc+1            ; incrémentation du numéro de l'orbite
      endif
;
; fin boucle sur les fichiers lus
   endif
   IF key_performance EQ 1 THEN BEGIN
       msg = report(['ppp : ' + routine + ' : durée intermédiaire au passage au fichier suivant ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
    ENDIF
labfile:
endfor
if (n_elements(data) NE 0) then begin
    ; write outputfile
    header1 = { yyyy : yyyy $
             ,  mm : mm $
             ,  dd : dd $
             ,  numch : numch $
             ,  nbpix : nbgrid $
             ,  resol : resol $
             }
header2= { lonmin: lonmin $
                      , lonmax: lonmax $
                      , latmin: latmin $
                      , latmax: latmax $
                     }

    amsu_t2={header1: header1, header2: header2, data: data}
    help, amsu_t2,/structure
    testfilename=''
    fileout = mem_to_file_amsu_t2(amsu_t2, testfilename)

endif else begin
    msg = report(['www : ' + routine + ' : no data to write'])
    goto, realend
endelse

nofile:
    msg = report(['www : ' + routine + ' : no data to read'])
    goto, realend

realend:
    IF key_performance EQ 1 THEN BEGIN
       msg = report(['ppp : ' + routine + ' : durée totale ' $
                   + string(SYSTIME(1)-time1,format='(F12.6)')])
       ENDIF
end
