#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

==================
extract_amsu_py_py
==================

SYNOPSIS
========

.. code-block:: bash

   extract_amsu_py.py --yyymmdd yyyymmdd --lonmin lonmin --lonmax lonmax --latmin latmin --latmax latmax --numch numch --filename filename --resol resol --testname testname

DESCRIPTION
===========

.. warning::

   not tested with AMSU-A

.. warning::

   this CLI exists only to consolidate :func:`traite_amusab_py`

   :func:`search_amsufiles_py` has to be used inside a main program
   for realistic amsu_t1 values

   :func:`file_correcb_to_mem_py` has to be used inside a main program
   for realistic correction values (here set to 0.)

   :func:`pixelsize_py` has to be used inside a main program
   for realistic swath values (here set to 1.)

.. option:: --lonmin <lonmin>
.. option:: --lonmax <lonmax>
.. option:: --latmin <latmin>
.. option:: --latmax <latmax>
.. option:: --numch <channel>
.. option:: --resol <resol>
.. option:: --filename <filename>
.. option:: --yyyymmdd <YYYYMMDD>
.. option:: --testname <testname>

   name of the test when running CLI : now fake correction

prgm de lecture des fichiers AMSU (A++ et B)

dÃ©code les noms des fichiers donnÃ©Ã©s en argument dans **files_list**
dans la date choisie, puis appelle le prgm de lecture

use correction produced by :func:`file_correcb_to_mem_py`.
use correction produced by :func:`pixelsize_py`.

If **testname** is set to '''test1''', correction is set to 0. everywhere and swath is set to 1. everywhere (silly !)

++applique une interpolation tenant compte de la dimension de la tache
au sol selon la position dans la fauchÃ©e

++Ã©crit les donnÃ©es pour la zone (lat/long) choisie

++ le fichier de sortie contient dans l'ordre mm, jj (jour de
l'annÃ©e), le temps (identique pour les points d'une mÃªme fauchÃ©e), le
no satellite, le no de fov (1 a 40 ou 1 a 90), long, lat, tb

Les valeurs manquantes sont codÃ©es par NaN.

.. only:: man

   Figure is visible on PDF and HTML documents only.

.. only:: html or latex

    .. graphviz::

       digraph extract_amsu_py {

          amsu_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/AMSU/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];
          amsu_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/yyyy/mm/numch_yyyymmdd_geomin_geomax.dat"];
          correc [shape=tab,fontname=Courier,label="numpy structured array++ correction array common 3D (lat, lon, fov)"];

          extract_amsu_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/extract_amsu_py.py",
          label="${PROJECT}/src/extract_amsu_py.py"];

          {amsu_t1 correc} -> {extract_amsua_py} -> {amsu_t2};
      }


EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest ${PROJECT}/src/extract_amsu_py.py

Realistic example with one VARAMMA file AMSU-B channel
:file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C`

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/
   ln -s /homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/
   numch='b3'
   yyyymmdd=20110403
   lonmin=-22
   lonmax=22
   latmin=-12
   latmax=42
   resol=1
   testname="test1"
   filename=${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C
   extract_amsu_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --lonmin ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax} --resol ${resol} --filename ${filename} --testname ${testname}
   tlogd.sh extract_amsu_py_main

Nothing to be written because only missing values in
amsu_t1['dataset']['amb1c_btemps'] b3

Another realistic example with one VARAMMA file AMSU-B channel
:file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C`

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006081303/
   ln -s /homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/
   numch='b3'
   yyyymmdd=20060813
   lonmin=-22
   lonmax=22
   #++lonmin=-130; lonmax=-115 # ++ pour debug
   latmin=-12
   latmax=42
   resol=1
   testname="test1"
   filename=${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C
   extract_amsu_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --lonmin ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax} --resol ${resol} --filename ${filename} --testname ${testname}
   tlogd.sh extract_amsu_py_main

:file:`${PROJECT_ID}/AMSU//2006/08/b3_20060813_022w12s_022e42n.dat` will be written if not already exists.

The same with debugger :

.. code-block:: bash

   python -m pdb ${PROJECT}/src/extract_amsu_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --lonmin ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax} --resol ${resol} --filename ${filename} --testname ${testname}

SEE ALSO
========

:ref:`data_amsu`

called by :ref:`traite_amsu_py.py`

previous step : :func:`search_amsufiles_py`

use :ref:`read_amsua1c_py`,
:func:`mem_to_file_amsu_t2_py`
:func:`interpolswathb_py`

use returned values of :func:`file_correcb_to_mem_py` called by :func:`traite_amsuab_py`

use returned values of :func:`pixelsize_py`  called by :func:`traite_amsuab_py`

next step :  ++

TODO
====

trop de log (mieux que pas de log) : stdout + logfile

finish migration check correction yyyymmdd

doctest avec correction plus realiste

faire plus malin comme données de test de correction test1

pythonize loops

http://matplotlib.org/users/shell.html pour que le dessin ne bloque pas
l'excution (et parametre pour choisir de faire/ ne pas faire  les dessins
cf. isinteractive() ou isatty() et ion()

documenter keyword with_plot

EVOLUTIONS
==========

$Id: extract_amsu_py.py 513 2012-04-10 14:50:06Z pinsard $

$URL: svn+ssh://lelod@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/extract_amsu_py.py $

- pinsard 20141020T155945Z climtest0.ipsl.polytechnique.fr (Linux)

  * fix numpy.append usage

- pinsard 20141020T131110Z climtest0.ipsl.polytechnique.fr (Linux)

  * change usage of file_correcb_to_mem_py output to be able to store multiple
    satellites

- pinsard 20141015T082710Z climtest0.ipsl.polytechnique.fr (Linux)

  * nearest cf. http://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array

- pinsard 20141014T134535Z climtest0.ipsl.polytechnique.fr (Linux)

  * going on migration off correction

- pinsard 20141003T131956Z climtest0.ipsl.polytechnique.fr (Linux)

  * add test on missing values amsu_t1['dataset']['amb1c_btemps'][:,:, ich]

- pinsard 20140827T140253Z loholt1.ipsl.polytechnique.fr (Linux)

  * going on migration with no interpolswath

- pinsard 20140811T145811Z loholt1.ipsl.polytechnique.fr (Linux)

  * add correction argument

- pinsard 20140808T091545Z loholt1.ipsl.polytechnique.fr (Linux)

  * suppression of nomcanal test. suppression of numch 'a' case.

- pinsard 20140807T154406Z loholt1.ipsl.polytechnique.fr (Linux)

  * reproduce modification from :file:`extract_amsub.pro` (not finished)
  * add migration of pixelsize

- pinsard 20140807T141633Z loholt1.ipsl.polytechnique.fr (Linux)

  * add test with :file:`/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C`
  * remove AMSU-A example and doctest

- fplod 20140805T090754Z callisto.locean-ipsl.upmc.fr (Linux)

  * fix array indices migration (IDL orders must be reversed)
  * start migration of structure for amsu_t2

- fplod 20140804T154812Z callisto.locean-ipsl.upmc.fr (Linux)

  * handle amsu-b (not finished)

- fplod 20140804T133045Z callisto.locean-ipsl.upmc.fr (Linux)

  * add migration of read_amsua1c

- fplod 20140801T131917Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from extract_amsu.pro

"""

__all__ = ["extract_amsu_py"]
__docformat__ = 'restructuredtext'

import os
import sys

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

import dateutil.parser

from find_nearest import find_nearest
from read_amsua1c_py import read_amsua1c_py
from plot_amsu_t1 import plot_amsu_t1_scan_fov
import numpy
from interpolswathb_py import interpolswathb_py
from file_correcb_to_mem_py import file_correcb_to_mem_py
from mem_to_file_amsu_t2_py import mem_to_file_amsu_t2_py
from geolocation_to_string_py import geolocation_to_string_py


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """process AMSU files for correction++"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--version',
                      action='store_true',
                      dest='version',
                      help='show version number and exit')

    parser.add_option('--yyyymmdd',
                      help='date to be found',
                      dest='yyyymmdd',
                      metavar='<YYYYMMDD>',
                      action='store')

    parser.add_option('--lonmin',
                      help='min longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmin')

    parser.add_option('--lonmax',
                      help='max longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmax')

    parser.add_option('--latmin',
                      help='min latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmin')

    parser.add_option('--latmax',
                      help='max latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmax')

    parser.add_option('--resol',
                      help='resol',
                      type='float',
                      default=None,
                      metavar='<RESOL>',
                      dest='resol')

    parser.add_option('--numch',
                      help='name of the channel',
                      dest='numch',
                      metavar='<NUMCH>',
                      action='store')

    parser.add_option('--filename',
                      help='filename',
                      dest='files_list',
                      metavar='<filename>',
                      action='append')

    parser.add_option('--testname',
                      help='name of dataset values for test',
                      dest='testname',
                      metavar='<testname>',
                      action='store')

    return parser


def extract_amsu_py(numch, files_list, yyyymmdd, resol, lonmin, lonmax, latmin, latmax, correcb, swath, with_plot=False):

    """

    amsu_t1 B3 file cf. :file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C`

    .. warning::

       this file must be linked or copy in ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C

       Too big to be in svn repository.

    >>> numch = 'b3'
    >>> yyyymmdd = dateutil.parser.parse('20110403', ignoretz=True)
    >>> lonmin = -22.
    >>> lonmax = 22.
    >>> latmin = -12.
    >>> latmax = 42.
    >>> resol = 1.
    >>> project_id_env = os.environ['PROJECT_ID']
    >>> files_list = []
    >>> files_list.append('{0}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C'.format(project_id_env))
    >>> nomsat = 'N15_AMSUB'
    >>> nocanal = int(numch.split()[0][1])
    >>> correcb_onesat = file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax)
    >>> correcb = {nomsat: correcb_onesat}
    >>> swath = numpy.ones(90, dtype='Float32')
    >>> result = extract_amsu_py(numch, files_list, yyyymmdd, resol, lonmin, lonmax, latmin, latmax, correcb, swath)
    +++ only missing values in btemps

    amsu_t1 B3 file cf. :file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C`

    .. warning::

       this file must be linked or copy in ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C

       Too big to be in svn repository.

    >>> numch = 'b3'
    >>> yyyymmdd = dateutil.parser.parse('20060813', ignoretz=True)
    >>> lonmin = -22.
    >>> lonmax = 22.
    >>> latmin = -12.
    >>> latmax = 42.
    >>> resol = 1.
    >>> project_id_env = os.environ['PROJECT_ID']
    >>> files_list = []
    >>> files_list.append('{0}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C'.format(project_id_env))
    >>> files_list.append('{0}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0305.E0452.B4288384.GC.L1C'.format(project_id_env)) # ++ fichier qui plante traite_amsu_py
    >>> nomsat = 'N15_AMSUB'
    >>> nocanal = int(numch.split()[0][1])
    >>> correcb_onesat = file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax)
    >>> correcb = {nomsat: correcb_onesat}
    >>> swath = numpy.ones(90, dtype='Float32')
    >>> result = extract_amsu_py(numch, files_list, yyyymmdd, resol, lonmin, lonmax, latmin, latmax, correcb, swath)
    +++

    """

    logger = logging.getLogger()

    if not files_list:
        logger.info('pas de fichiers')
        return
    else:
        logger.info('nb de fichiers {0}'.format(len(files_list)))

    look = 'figure'
    scale = 1.
    geomin = geolocation_to_string_py(lonmin, latmin, look, scale)
    geomax = geolocation_to_string_py(lonmax, latmax, look, scale)
    logger.info('zone : {0} {1}'.format(geomin, geomax))

    nomcanal = numch.split()[0][0]
    nocanal = int(numch.split()[0][1])

    tbmin = 100
    tbmax = 350

    # boucle sur les fichiers amsu_t1 donnés en arguments
    desc = 0
    logger.info('traitement du jour {0}'.format(yyyymmdd))

    for filename in files_list:
        fileb = filename
        (header, dataset) = read_amsua1c_py(fileb, nomcanal)
        if header is None or dataset is None:
            logger.warning('file {0} can not be read'.format(filename))
            continue
        amsu_t1 = {'header': header, 'dataset': dataset}

        # test if not only missing values in amb1c_btemps
        if (amsu_t1['dataset']['amb1c_btemps'][:, :, nocanal - 1] == -999999).all():
            msg = "only missing values in amsu_t1['dataset']['amb1c_btemps'][:,:, {0}] in file {1}".format(nocanal - 1, fileb)
            logging.info(msg)
            continue

        #if sys.stdout.isatty():  # ++ pas trs efficace
        if with_plot:  # ++ pas trs efficace
            # plot pour mise au point ++
            result = plot_amsu_t1_scan_fov(amsu_t1, '/dev/null',
                                           channel_selection=nocanal - 1,
                                           scan_selection=0,
                                           lonmin=lonmin,
                                           latmin=latmin,
                                           lonmax=lonmax,
                                           latmax=latmax)  # ++ tous les scans
        # n_scan = nb lignes (fauchÃ©es) dans l'orbite
        # n_fov  = nb points dans la fauchÃ©e
        n_scan, n_fov, n_ch = dataset['amb1c_btemps'].shape
        # extraction du canal traitÃ© dans un domaine enveloppant la zone choisie
        nosat = header['amb1c_h_satid']  # numÃ©ro du satellite NOAA (15 a 19) ou METOP 2
        logger.info('NUMERO DU SATELLITE {0}'.format(nosat))
        logger.info('INSTRUMENT {0}'.format(header['amb1c_h_instrument']))
        # on lit la structure et on extrait les infos: temps, longitude,
        # latitude, et les Tbs

        # We look if there iare some data inside the geobox
        amalong = dataset['amb1c_latlon'][:, :, 1] / 1.E4
        amalati = dataset['amb1c_latlon'][:, :, 0] / 1.E4
        mask_lat = (amalati >= latmin) & (amalati <= latmax)[0]
        mask_lon = (amalong >= lonmin) & (amalong <= lonmax)[0]
        amalong_ma = numpy.ma.masked_array(amalong, mask=mask_lon)
        # Does amalong have any non-masked elements?
        if not amalong_ma.mask.all():
            logger.info('data in lon; so going on')

        mask_lat_lon = mask_lat & mask_lon
        if (amsu_t1['dataset']['amb1c_latlon'][:, :, 0] < latmin).all() and (amsu_t1['dataset']['amb1c_latlon'][:, :, 0] > latmax).all() and (amsu_t1['dataset']['amb1c_latlon'][:, :, 1] < lonmin).all() and (amsu_t1['dataset']['amb1c_latlon'][:, :, 0] > lonmax).all():
            logger.info('everypoint out of {0} {1} (all)'.format(geomin, geomax))
            continue

        # ++ compare methode
        ndom = sum(sum(mask_lat_lon))
        logger.info('nb de points du fichier {0} dans le domaine gÃ©ographique {1}'.format(fileb, ndom))
        if ndom == 0:
            logger.info('everypoint out of {0} {1} (mask)'.format(geomin, geomax))
            continue

        amcha = dataset['amb1c_btemps'][:, :, nocanal - 1] / 100.
        amzeni = dataset['amb1c_angles'][:, :, 0] / 100.
        ttt = dataset['amb1c_scnlintime'] / 3600000.
        amaday = dataset['amb1c_scnlindy']  # a ne mangerait pas de pain de verifier que c'est bien le doy de yyyymmdd

        # extraction du canal traitÃ© dans un domaine enveloppant la zone choisie
        midfov = int(numpy.fix(n_fov / 2))
        marge = 15.
        geomin_marge = geolocation_to_string_py(lonmin - marge, latmin - marge, look, scale)
        geomax_marge = geolocation_to_string_py(lonmax + marge, latmax + marge, look, scale)
        logger.info('zone largie : {0} {1}'.format(geomin_marge, geomax_marge))

        mask_lat = (amalati[midfov, :] >= latmin - marge) & (amalati[midfov, :] <= latmax + marge)[0]
        mask_lon = (amalong[midfov, :] >= lonmin - marge) & (amalong[midfov, :] <= lonmax + marge)[0]
        mask_lat_lon = mask_lat & mask_lon
        jnd = numpy.where(mask_lat_lon)
        nzon = sum(mask_lat_lon)

        logger.info('nb de points du fichier {0} dans le domaine étendu avec {1} deg de marge : {2}'.format(fileb, marge, nzon))

        if nzon > 1:
            logger.info('some data of file {0} are in the enlarged domain {1} {2}'.format(fileb, geomin_marge, geomax_marge))
            amalat = amalati[mask_lat_lon, :]
            amalon = amalong[mask_lat_lon, :]
            amch = amcha[mask_lat_lon, :]
            amzen = amzeni[mask_lat_lon, :]
            tt = ttt[jnd]
            logger.info('dimension tableaux extraits {0}'.format(amalon.shape))
            jour_ama = amaday[0]        # jour du passage
            desc = desc + 1
            #
            # correction nadir des donnÃ©es. En principe le 17 est Ã©liminÃ© des listes
            isat = 0
            if nosat == 15:
                nomsat = 'N15_AMSUB'
            elif nosat == 16:
                nomsat = 'N16_AMSUB'
            elif nosat == 18:
                nomsat = 'N18_AMSUB'
            elif nosat == 19:
                nomsat = 'N19_AMSUB'
            elif nosat == 2:
                nomsat = 'M02_MHS'
            else:
                logger.warning('no satellite {0} not implemented'.format(nosat))
                logger.warning('file {0} not used for output'.format(fileb))
                continue

            ch_nadir = numpy.empty((nzon, n_fov), dtype='Float32')

            for isc in range(0, nzon):
                for ifo in range(0, n_fov):
                    # en principe, chasse au point de grille du fichier correc le plus
                    # proche des donnÃ©es (pour le sat, sachant que la date est connue et
                    # que qu'on a dÃ©jÃ  rÃ©duit la zone d'intÃ©rÃªt si nec)
                    try:
                        igrid = find_nearest(correcb[nomsat]['lon'], amalon[isc, ifo])
                    except IndexError:
                        logger.error('pb find_nearest i isc {0} ifo {1}'.format(isc, ifo), exc_info=True)
                        logger.error("correcb[nomsat]['lon'] {0}".format(correcb[nomsat]['lon']))
                        logger.error('amalon[isc, ifo] {0}'.format(amalon[isc, ifo]))
                        raise

                    try:
                        jgrid = find_nearest(correcb[nomsat]['lat'], amalat[isc, ifo])
                    except IndexError:
                        logger.error('pb find_nearest j isc {0} ifo {1}'.format(isc, ifo), exc_info=True)
                        logger.error("correcb[nomsat]['lat'] {0}".format(correcb[nomsat]['lat']))
                        logger.error('amalat[isc, ifo] {0}'.format(amalat[isc, ifo]))
                        raise

                    try:
                        ch_nadir[isc, ifo] = amch[isc, ifo] - correcb[nomsat]['correction'][jgrid, igrid, ifo]
                    except IndexError:
                        logger.error('pb correction ++ isc {0} ifo {1}'.format(isc, ifo), exc_info=True)
                        logger.error('shape de amch{0}'.format(amch.shape))
                        logger.error('shape of ch_nadir {0}'.format(ch_nadir.shape))
                        logger.error("shape of correcb[nomsat]['correction'] {0}".format(correcb[nomsat]['correction'].shape))
                        raise
                    else:
                        logger.debug('correction on isc {0} ifo {1} = {2}'.format(isc, ifo, correcb[nomsat]['correction'][jgrid, igrid, ifo]))

            moych = numpy.empty((n_fov), dtype='Float32')
            #++ numpy max min logger.info('extrema tb min raw {0} min correct {1} max raw {2} max correct {3}'.format(min(amch), min(ch_nadir), max(amch), max(ch_nadir)))
            # appel Ã Â  interpolswath pour ajuster les pixels sur une grille
            # rÃ©guliÃ¨re dans la fauchÃ©e
            # et sÃ©lection de la zone conservÃ©e
            nbgrid = 0
            cont = 0
            for i in range(0, nzon):
                logger.info('i {0}/nzon {1}'.format(i, nzon))
                tb = ch_nadir[i, :]
                try:
                    lon = amalon[i, :]
                except IndexError:
                    logger.error('pb IndexError i {0} nzon {1} shape de amalon {2}'.format(i, nzon, amalon.shape), exc_info=True)

                try:
                    lat = amalat[i, :]
                except IndexError:
                    logger.error('pb IndexError i {0} nzon {1} shape de amalat {2}'.format(i, nzon, amalat.shape), exc_info=True)

                ind = numpy.where(tb > tbmin)[0]
                nbon = len(ind)
                # test pour éviter pb d'interpolation de longitude (méridien 180) ++ je ne vois pas le rapport entre commentaire et commandes
                if nbon == n_fov:
                    cont = cont + 1
                    nbgrid1 = nbgrid
                    try:
                        (nbgrid, tbgrid, latgrid, longrid) = interpolswathb_py(tb, lat, lon, resol, swath)
                    except ValueError as error:
                        logger.error('pb interpolswath error {0}'.format(error), exc_info=True)
                        logger.error('pb interpolswath shape de tb {0}'.format(tb.shape))

                    fovgrid = numpy.array(range(1, len(tbgrid) + 1))
                    logger.info('nb points fov interpolés {0}'.format(len(tbgrid)))
                    # sélection des taches au sol situées dans la zone d'intérêt
                    mask_lat = (latgrid >= latmin) & (latgrid <= latmax)[0]
                    mask_lon = (longrid >= lonmin) & (longrid <= lonmax)[0]
                    mask_tb = (tbgrid >= tbmin) & (tbgrid <= tbmax)[0]
                    mask_lat_lon_tb = mask_lat & mask_lon & mask_tb
                    npt = sum(mask_lat_lon_tb)
                    logger.info('nb points zone d intérêt {0}'.format(npt))
                    if npt != 0:
                        zone = numpy.where(mask_lat_lon_tb)[0]
                        try:
                            chint
                        except NameError:
                            timeint = numpy.repeat(tt[i], npt)
                            chint = tbgrid[zone]
                            latint = latgrid[zone]
                            lonint = longrid[zone]
                            fovint = fovgrid[zone]
                        else:
                            timeint = numpy.append(timeint, numpy.repeat(tt[i], npt))
                            chint = numpy.append(chint, tbgrid[zone])
                            latint = numpy.append(latint, latgrid[zone])
                            lonint = numpy.append(lonint, longrid[zone])
                            fovint = numpy.append(fovint, fovgrid[zone])

                        nn = chint.shape[0]
                        logger.info('nb points dans la zone en fin de traitement de l''orbite {0}'.format(nn, format='(I10.10)'))

                        for i in range(0, nn):
                            try:
                                data
                            except NameError:
                                data = {'desc': numpy.empty(1, 'Int8'),
                                        'hour': numpy.empty(1, 'Float32'),
                                        'fov': numpy.empty(1, 'Int8'),
                                        'lon': numpy.empty(1, 'Float32'),
                                        'lat': numpy.empty(1, 'Float32'),
                                        'tb': numpy.empty(1, 'Float32')}
                                # ++ map copy/paste data = {'desc': numpy.empty(1, 'Int8'), 'hour': numpy.empty(1, 'Float32'), 'fov': numpy.empty(1, 'Int8'), 'lon': numpy.empty(1, 'Float32'), 'lat': numpy.empty(1, 'Float32'), 'tb': numpy.empty(1, 'Float32')}
                                data['desc'][-1] = desc
                                data['hour'][-1] = timeint[i]
                                data['fov'][-1] = fovint[i]
                                data['lon'][-1] = lonint[i]
                                data['lat'][-1] = latint[i]
                                data['tb'][-1] = chint[i]
                            else:
                                data['desc'] = numpy.append(data['desc'], desc)
                                data['hour'] = numpy.append(data['hour'], timeint[i])
                                data['fov'] = numpy.append(data['fov'], fovint[i])
                                data['lon'] = numpy.append(data['lon'], lonint[i])
                                data['lat'] = numpy.append(data['lat'], latint[i])
                                data['tb'] = numpy.append(data['tb'], chint[i])
                else:
                    logger.info('n_fov : {0} vs nbon {1}'.format(n_fov, nbon))
                    logger.info('indices of tb > tbmin ({0}) :{1}'.format(tbmin, ind))
        else:
            logger.info('www : aucun point dans la zone 1 {0} {1}'.format(geomin, geomax))

        desc = desc + 1  # incrÃ©mentation du numÃ©ro du fichier equivalent  l'orbite
    #
    # fin boucle sur les fichiers lus

    # write amsu_t2 file if data found
    try:
        data
    except NameError:
        logger.info('no data to write')
        return None 
    else:
        # write outputfile
        header1 = {'yyyymmdd': yyyymmdd,
                   'numch': numch,
                   'nbpix': nbgrid,
                   'resol': resol}
        header2 = {'lonmin': lonmin,
                   'lonmax': lonmax,
                   'latmin': latmin,
                   'latmax': latmax}

        amsu_t2 = {'header1': header1,
                   'header2': header2,
                   'data': data}
        testfilename = None
        try:
            fileout = mem_to_file_amsu_t2_py(amsu_t2, testfilename)
        except:
            logger.error('exception pb mem_to_file_amsu_t2 in extract_amsu_py',  exc_info=True)
            raise
        else:
            result = fileout

    return result


def extract_amsu_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['numch', 'yyyymmdd',  'resol', 'files_list',
                            'lonmin', 'lonmax',
                            'latmin', 'latmax', 'testname']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    numch = fromcli.numch
    yyyymmdd = dateutil.parser.parse(fromcli.yyyymmdd, ignoretz=True)
    lonmin = fromcli.lonmin
    lonmax = fromcli.lonmax
    latmin = fromcli.latmin
    latmax = fromcli.latmax
    files_list = fromcli.files_list
    resol = fromcli.resol
    testname = fromcli.testname

    #
    # !! check consistency of those lines with equivalent in project_init.pro++
    # !! when change ++
    yyyymmdd_min = dateutil.parser.parse('19991230', ignoretz=True)
    yyyymmdd_max = dateutil.parser.parse('20150102', ignoretz=True)

    if yyyymmdd_min <= yyyymmdd <= yyyymmdd_max:
        logging.info('yyyymmdd {0} is between {1} and {2}'.format(yyyymmdd, yyyymmdd_min, yyyymmdd_max))
    else:
        logging.error('yyyymmdd {0} must be between {1} and {2}'.format(yyyymmdd, yyyymmdd_min, yyyymmdd_max))
        parser.print_help()
        raise

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    #logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    handler = logging.FileHandler(log)
    logger.addHandler(handler)
    if is_verbose:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))

    logger.info('[Parameters]')
    logger.info('numch={0}'.format(numch))
    logger.info('yyyymmdd={0}'.format(yyyymmdd))
    logger.info('files_list={0}'.format(files_list))
    logger.info('lonmin={0}'.format(lonmin))
    logger.info('lonmax={0}'.format(lonmax))
    logger.info('latmin={0}'.format(latmin))
    logger.info('latmax={0}'.format(latmax))
    logger.info('resol={0}'.format(resol))
    logger.info('testname={0}'.format(testname))
    #++

    if testname == 'test1':
        from file_correcb_to_mem_py import file_correcb_to_mem_py
        nomsat = 'N15_AMSUB'
        nocanal = int(numch.split()[0][1])
        correcb_onesat = file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax)
        correcb = {nomsat: correcb_onesat}
        swath = numpy.ones(90, dtype='Float32')
    else:
        logger.error('no correction can be simulated')

    try:
        result = extract_amsu_py(numch, files_list, yyyymmdd, resol, lonmin, lonmax, latmin, latmax, correcb, swath)
    except:  # ++ exception
        logger.error('pb extract_amsu_py',  exc_info=True)
        # ++ exit bad status

# Run main, if called from the command line
if __name__ == '__main__':
    extract_amsu_py_main()
