function varargout=extract_nc(file,var,varargin)

%EXTRACT_NC Lecture de fichiers .nc

%+
%
% .. _extract_nc.m:
%
% ============
% extract_nc.m
% ============
%
% DESCRIPTION
% ===========
%
% Extraction d''un fichier .nc de variables
% selon les dimensions fournies sous la forme : 
% (nom de la dimension, maximum, minimum ) * nombre de dimension
%
% Retourne chaque dimension extraite suivant les minimum et maximum fournis
% dans l''ordre fourni puis la ou les variables demandees
%
% Retourne les variables demandees corrigees de 'scale_factor' et 'add_offset'
% avec NaN pour les valeurs d''erreur
%
% NB : 
%  - pour extraire une dimension entiere entrer NaN comme bornes min et max
%  - var doit etre un tableau de cellules
%
% EXAMPLES
% ========
%
% ::
%
%  [tps,lon,lat,extract1,extract2]=extract_nc('nomdufichier',{'data1' 'data2'},
%  ... 'tps',tps_min,tps_max,'lon',lonmin,lonmax,'lat',latmin,latmax);
% 
%
% SEE ALSO
% ========
%
% TODO
% ====
%
% EVOLUTIONS
% ==========
%
% $Id: extract_nc.m 326 2011-08-05 14:24:46Z pinsard $
%
% $URL$
%
%-

%usage='varargout=extract_nc(file,var,varargin)';
%if nargin~=nargin(mfilename)
%   disp(['Incorrect number of input arguments ' num2str(nargin)]);
%   error(usage);
%end
%
%if nargout ~= nargout(mfilename)
%   disp(['Incorrect number of output arguments ' num2str(nargout)]);
%   error(usage);
%end
%

  
%% File parameters
ncid=netcdf.open(file,'NC_NOWRITE');
% nombre de dimensions (dont unlimited), variables, attributs, dimensions unlimited 
[ndims,nvars,natts,unlimdimID]= netcdf.inq(ncid); 
% Dimensions
for i=1:ndims
  [dimfic(i).name, dimfic(i).len] = netcdf.inqDim(ncid,i-1);
  dimfic(i).id   = i-1;
end
% Variables
for i=1:nvars
  varfic(i).id=i-1;
  [varfic(i).name,varfic(i).type,varfic(i).dimids,varfic(i).natts]=netcdf.inqVar(ncid,i-1);
end


%% Inputs information about dimensions
nb_dim=(nargin-2)/3;  
start=zeros(1,length(dimfic))+NaN;
count=zeros(1,length(dimfic))+NaN;
for i=1:nb_dim
  % Read inputs
  % dimension name
  inp(i).name=varargin{(i-1)*3+1};       
  % min limit
  inp(i).min=varargin{(i-1)*3+2};  
  % max limit
  inp(i).max=varargin{(i-1)*3+3};
  
  % find id and len from dimensions
  indd=find(strcmp(inp(i).name,{dimfic.name})==1);
  if isempty(indd)
    sprintf('Error: dimension "%s" unknown !',inp(i).name)
    return
  end
  inp(i).id=dimfic(indd).id;
  inp(i).len=dimfic(indd).len;  

  % find id of variable linked to dimensions
  indv=find(strcmp(inp(i).name,{varfic.name})==1);
  if ~isempty(indv)
    out_dim = netcdf.getVar(ncid,varfic(indv).id);
  else
    % if dimension is not in variables
    out_dim=[1:inp(i).len]';
  end

  % Index for extraction
  if ~isnan(inp(i).min)
    indmin=find(out_dim>=inp(i).min);
  else        
    indmin=[1:length(out_dim)];
  end
  if ~isnan(inp(i).max)
    indmax=find(out_dim<=inp(i).max);
  else        
    indmax=[1:length(out_dim)];
  end
  ind=intersect(indmin,indmax);
  % start index and number of points to extract
  % index beginning = 0 not 1
  start(inp(i).id+1)=min(ind)-1;
  count(inp(i).id+1)=max(ind)-start(inp(i).id+1);
  % Stock dimension values
  varargout(i)={out_dim(ind)};
end

%% Variables
for k=1:length(var)

  indv=find(strcmp(var(k),{varfic.name})==1);
  if isempty(indv)
    sprintf('Error: variable "%s" unknown !',var(k))
    return
  end
  
  %NetCDF files use C-style row-major ordering for multidimensional
  %arrays, while MATLAB uses FORTRAN-style column-major ordering.
  %This means that the size of a MATLAB array must be flipped
  %relative to the defined dimension sizes of the netCDF data set.
  %For example, if the netCDF dataset has dimensions 3x4x5, then the
  %equivalent MATLAB array has size 5x4x3.
  indd=fliplr(varfic(indv).dimids+1);
  ext_data=netcdf.getVar(ncid,varfic(indv).id,fliplr(start(indd)),fliplr(count(indd)));
  if length(indd)>1
    ext_data=permute(ext_data,fliplr(1:length(indd)));
  end
  
  % Attribut names
  for i=1:varfic(indv).natts
    atts(i)={netcdf.inqAttName(ncid,varfic(indv).id,i-1)};
  end
  
  % unpacked = packed*scale_factor+add_offset
  inds=find(strcmp('scale_factor',atts));
  if ~isempty(inds)
    scale_factor=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(inds)));
  else
    scale_factor=1;
  end
  inda=find(strcmp('add_offset',atts));
  if ~isempty(inda)
    add_offset=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(inda)));
  else
    add_offset=0;
  end
  ext_data=ext_data*scale_factor+add_offset;
  
  % Attribut _FillValue and missing_value
  indfv=find(strcmp('_FillValue',atts));
  if ~isempty(indfv)
    fval=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(indfv)));
    ext_data(ext_data==fval)=NaN;
  end
  indmv=find(strcmp('missing_value',atts));
  if ~isempty(indmv)
    mval=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(indmv)));
    ext_data(ext_data==mval)=NaN;
  end
  
  % Attribut valid_min, valid_max, valid_range
  indmi=find(strcmp('valid_min',atts));
  if ~isempty(indmi)
    mmin=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(indmi)));
    ext_data(ext_data<mmin)=NaN;
  end
  indma=find(strcmp('valid_max',atts));
  if ~isempty(indma)
    mmax=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(indma)));
    ext_data(ext_data>mmax)=NaN;
  end
  indra=find(strcmp('valid_range',atts));
  if ~isempty(indra)
    rval=netcdf.getAtt(ncid,varfic(indv).id,cell2str(atts(indra)));
    ext_data(ext_data<rval(1) | ext_data>rval(2))=NaN;
  end
  
  % Output
  varargout(k+nb_dim)={ext_data};
  
end

netcdf.close(ncid)