#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

=========================
file_amsu_t2_to_mem_py.py
=========================

SYNOPSIS
========

.. code-block:: bash

   file_amsu_t2_to_mem_py --yyymmdd yyyymmdd --lonmin lonmin --lonmax lonmax --latmin latmin --latmax latmax --numch numch --testfilename testfilename

DESCRIPTION
===========

.. option:: --yyyymmdd <YYYYMMDD>
.. option:: --lonmin <lonmin>
.. option:: --lonmax <lonmax>
.. option:: --latmin <latmin>
.. option:: --latmax <latmax>
.. option:: --numch <channel>
.. option:: --testfilename <testfilename>


.. note::

   for files created with IDL programs
   split with separator criterium is possible when fov < 100 like in

   .. parsed-literal::

      8   5.7985 87   21.395   19.126 1  232.565

   but not when fov >=100 like in

   .. parsed-literal::

      8   5.7985100   18.948   19.419 1  251.377

   split with separator are is done when 11th character is blank.
   otherwise positions are used.

   This trick might be removed when all amsu_t2 files will be created by
   python program

Read an amsu_t2 file and store data and header in memory.

This type of file has been produced by :ref:`extract_amsu_py`

If **testfilename** is not empty, this filename will be written, otherwise
amsu_t2 terminology will be applied.

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest file_amsu_t2_to_mem_py.py

Realistic example with VARAMMA file AMSU-B channel 3
:file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat`

   mkdir -p ${PROJECT_ID}/AMSU//2011/04
   ln -s /homedata/eymard/varamma_d/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat ${PROJECT_ID}/AMSU//2011/04/
   numch='b3'
   yyyymmdd=20110403
   lonmin=-22
   lonmax=22
   latmin=-12
   latmax=42
   file_amsu_t2_to_mem_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --lonmin ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax}
   tlogd.sh file_amsu_t2_to_mem_py_main

SEE ALSO
========

:func:`extract_amsu_py`

:func:`mem_to_file_amsu_t2_py`

use :func:`geolocation_to_string_py`

TODO
====

blinder les arguments dans file_amsu_t2_to_mem_py :
si testfilename <> None, on n'a pas besoin de tous les autres. Si non, on a besoin de tout les autres.

lecture plus maligne

missing values for amsu_t2 dictionary init

renvoyer le nom du fichier lu (ie terminologie ou tesfilename)

ajouter une impression de controle pour doctest

EVOLUTIONS
==========

$Id$

$URL$

- pinsard 20140806T163358Z loholt1.ipsl.polytechnique.fr (Linux)

  * test with :file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat`
    fix reading because of fov :

    .. parsed-literal::

   20110403 b3 000  1.000
     -22.000   22.000  -12.000   42.000
    8   5.7985 84   21.961   19.054 1  261.888
    8   5.7985 85   21.772   19.078 1  261.888
    8   5.7985 86   21.584   19.102 1  245.070
    8   5.7985 87   21.395   19.126 1  232.565
   ...
    8   5.7985100   18.948   19.419 1  251.377

.. note:: here a memo to remember how python slice string

   cf. also https://docs.python.org/2/tutorial/introduction.html#strings

   ::

    +---+---+---+---+---+
    | H | e | l | p | A |
    +---+---+---+---+---+
    0   1   2   3   4   5
   -5  -4  -3  -2  -1

- fplod 20140806T121224Z callisto.locean-ipsl.upmc.fr (Linux)

  * usage of dictionary

- fplod 20140805T133753Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from file_amsu_t2_to_mem.pro

"""
__all__ = ["file_amsu_t2_to_mem_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

import dateutil.parser
from geolocation_to_string_py import geolocation_to_string_py
import numpy

import re


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """read AMSU_T2 file"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--version',
                      action='store_true',
                      dest='version',
                      help='show version number and exit')

    parser.add_option('--yyyymmdd',
                      help='date to be found',
                      dest='yyyymmdd',
                      metavar='<YYYYMMDD>',
                      action='store')

    parser.add_option('--lonmin',
                      help='min longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmin')

    parser.add_option('--lonmax',
                      help='max longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmax')

    parser.add_option('--latmin',
                      help='min latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmin')

    parser.add_option('--latmax',
                      help='max latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmax')

    parser.add_option('--numch',
                      help='name of the channel',
                      dest='numch',
                      metavar='<NUMCH>',
                      action='store')

    parser.add_option('--testfilename',
                      help='testfilename to be read++',
                      dest='testfilename',
                      metavar='<testfilename>',
                      action='store')

    return parser


def file_amsu_t2_to_mem_py(yyyymmdd, numch, lonmin, lonmax, latmin, latmax, testfilename):

    """
    amsu_t2 A5 file

    >>> numch = 'a5'
    >>> yyyymmdd = dateutil.parser.parse('20060813', ignoretz=True)
    >>> lonmin = -25.
    >>> lonmax = 25.
    >>> latmin = -5.
    >>> latmax = 20.
    >>> testfilename = None
    >>> result = file_amsu_t2_to_mem_py(yyyymmdd, numch, lonmin, lonmax, latmin, latmax, testfilename)
    >>> type(result)
    <type 'dict'>

    amsu_t2 B3 file cf. /homedata/eymard/varamma_d/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat

    .. warning::

       this file must be linked or copy in ${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat
       Too big to be in svn repository.

    >>> numch = 'b3'
    >>> yyyymmdd = dateutil.parser.parse('20110403', ignoretz=True)
    >>> lonmin = -22.
    >>> lonmax = 22.
    >>> latmin = -12.
    >>> latmax = 42.
    >>> testfilename = None
    >>> result = file_amsu_t2_to_mem_py(yyyymmdd, numch, lonmin, lonmax, latmin, latmax, testfilename)
    >>> type(result)
    <type 'dict'>

    """

    status = -1
    if testfilename is None:
        look = 'filename'
        scale = 1.
        geomin = geolocation_to_string_py(lonmin, latmin, look, scale)
        geomax = geolocation_to_string_py(lonmax, latmax, look, scale)
        # decompose yyyymmdd
        yyyy = yyyymmdd.strftime('%Y')
        mm = yyyymmdd.strftime('%m')
        dd = yyyymmdd.strftime('%d')
        project_id_env = os.environ['PROJECT_ID']
        fullfilename = '{0}/AMSU/{1}/{2}/{4}_{1}{2}{3}_{5}_{6}.dat'.format(project_id_env, yyyy, mm, dd, numch, geomin, geomax)
    else:
        fullfilename = testfilename

    # test if file exist
    if not os.path.isfile(fullfilename):
        logging.error('file {0} not found'.format(fullfilename))
        return status
    #
    # ++ missing values
    header1 = {'yyyymmdd': 0,
               'numch': 'a2',
               'nbpix': 0,
               'resol': 0}
    header2 = {'lonmin': 0.,
               'lonmax': 0.,
               'latmin': 0.,
               'latmax': 0.}

    nrows = sum(1 for line in open(fullfilename)) - 2
    logging.info('nb de lignes de {0} = {1}'.format(fullfilename, nrows))

    data = {'desc': numpy.zeros(nrows, 'Int8'),
            'hour': numpy.zeros(nrows, 'Float32'),
            'fov': numpy.zeros(nrows, 'Int8'),
            'lon': numpy.zeros(nrows, 'Float32'),
            'lat': numpy.zeros(nrows, 'Float32'),
            'landseamask': numpy.zeros(nrows, 'Int8'),
            'tb': numpy.zeros(nrows, 'Float32')}

    amsu_t2 = {'header1': header1,
               'header2': header2,
               'data': data}
    #
    # open file
    logging.info('ouverture pour lecture de {0}'.format(fullfilename))
    try:
        fid = open(fullfilename, "r")
    except IOError, err:
        logging.error("Error: {0} {1}\n".format(err.filename, err.strerror))
        return status

    amsu_t2_string = open(fullfilename).read().split("\n")
    separator = ' '
    #
    # read header lines
    linesqueeze = re.sub('  +', ' ', amsu_t2_string[0])
    items = linesqueeze.split(separator)
    amsu_t2['header1']['yyyymmdd'] = int(items[0])
    amsu_t2['header1']['numch'] = items[1]
    amsu_t2['header1']['nbpix'] = int(items[2])
    amsu_t2['header1']['resol'] = float(items[3])

    logging.info('header1 {0}'.format(amsu_t2['header1']))

    linesqueeze = re.sub('  +', ' ', amsu_t2_string[1])
    linestrip = linesqueeze.lstrip()
    items = linestrip.split(separator)
    amsu_t2['header2']['lonmin'] = float(items[0])
    amsu_t2['header2']['lonmax'] = float(items[1])
    amsu_t2['header2']['latmin'] = float(items[2])
    amsu_t2['header2']['latmax'] = float(items[3])

    logging.info('header2 {0}'.format(amsu_t2['header2']))

    # read data lines
    amsu_t2_stringdata = amsu_t2_string[2:-1]
    for irow, amsu_t2_stringdata_one in enumerate(amsu_t2_stringdata):

        linesqueeze = re.sub('  +', ' ', amsu_t2_stringdata_one)
        linestrip = linesqueeze.lstrip()
        # for files created with IDL programs
        # split with separator criterium is possible when fov < 100 like in
        # 8   5.7985 87   21.395   19.126 1  232.565
        # but not when fov >=100 like in
        # 8   5.7985100   18.948   19.419 1  251.377

        if amsu_t2_stringdata_one[11:12] == ' ':
            # ++ cas lon < 0
            # 55  14.0803 36  -21.547   41.983 0  245.338
            items = linestrip.split(separator)
            amsu_t2['data']['desc'][irow] = int(items[0])
            amsu_t2['data']['hour'][irow] = float(items[1])
            amsu_t2['data']['fov'][irow] = int(items[2])
            amsu_t2['data']['lon'][irow] = float(items[3])
            amsu_t2['data']['lat'][irow] = float(items[4])
            try:
                amsu_t2['data']['landseamask'][irow] = int(items[5])
            except ValueError:
                print('pb reading landseamask row {0} in line {1} split by separator'.format(irow, amsu_t2_stringdata_one))
            amsu_t2['data']['tb'][irow] = float(items[6])
        else:
            #
            # split with position
            # (cf. format idl (i2,1x,f8.4,i3,1x,f8.3,1x,f8.3,1x,I1,1x,f8.3))
            # 0         1         2         3         4
            # 0123456789012345678901234567890123456789012
            #  8   5.7985100   18.948   19.419 1  251.377
            # 12 12345678123 12345678 12345678 1 12345678
            # 12 123.5678123 1234.678 1234.678 1 1234.678

            amsu_t2['data']['desc'][irow] = int(amsu_t2_stringdata_one[0:2])
            amsu_t2['data']['hour'][irow] = float(amsu_t2_stringdata_one[4:11])
            amsu_t2['data']['fov'][irow] = int(amsu_t2_stringdata_one[11:14])
            try:
                amsu_t2['data']['lon'][irow] = float(amsu_t2_stringdata_one[16:23])
            except ValueError:
                print('pb reading lon row {0} in line {1} split by position'.format(irow, amsu_t2_stringdata_one))
                print('pb amsu_t2_stringdata_one[16:23]= AAA{0}AAA'.format(amsu_t2_stringdata_one[16:23]))
                print("amsu_t2['data']['desc'][irow] {0} amsu_t2['data']['hour'][irow] {1} amsu_t2['data']['fov'][irow] {2}".format(amsu_t2['data']['desc'][irow], amsu_t2['data']['hour'][irow], amsu_t2['data']['fov'][irow]))
                return None  # ++ break

            amsu_t2['data']['lat'][irow] = float(amsu_t2_stringdata_one[24:32])
            try:
                amsu_t2['data']['landseamask'][irow] = int(amsu_t2_stringdata_one[33:34])
            except ValueError:
                print('pb reading landseamask row {0} in line {1} split by position'.format(irow, amsu_t2_stringdata_one))
                print('pb amsu_t2_stringdata_one[33:34] = AAA{0}AAA'.format(amsu_t2_stringdata_one[33:34]))
                print("amsu_t2['data']['desc'][irow] {0} amsu_t2['data']['hour'][irow] {1} amsu_t2['data']['fov'][irow] {2} amsu_t2['data']['lon'][irow] {3} amsu_t2['data']['lat'][irow] {4}".format(amsu_t2['data']['desc'][irow], amsu_t2['data']['hour'][irow], amsu_t2['data']['fov'][irow], amsu_t2['data']['lon'][irow], amsu_t2['data']['lat'][irow]))
                return None  # ++ break
            amsu_t2['data']['tb'][irow] = float(amsu_t2_stringdata_one[36:])

    logging.info('data hour {0}'.format(amsu_t2['data']['hour']))

    # close file
    fid.close()
    #
    status = amsu_t2

    return status


def file_amsu_t2_to_mem_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['numch', 'yyyymmdd',
                            'lonmin', 'lonmax',
                            'latmin', 'latmax']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    numch = fromcli.numch
    yyyymmdd = dateutil.parser.parse(fromcli.yyyymmdd, ignoretz=True)
    lonmin = fromcli.lonmin
    lonmax = fromcli.lonmax
    latmin = fromcli.latmin
    latmax = fromcli.latmax
    testfilename = fromcli.testfilename

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))

    logger.info('[Parameters]')
    logger.info('numch={0}'.format(numch))
    logger.info('yyyymmdd={0}'.format(yyyymmdd))
    logger.info('testfilename={0}'.format(testfilename))
    logger.info('lonmin={0}'.format(lonmin))
    logger.info('lonmax={0}'.format(lonmax))
    logger.info('latmin={0}'.format(latmin))
    logger.info('latmax={0}'.format(latmax))

    result = file_amsu_t2_to_mem_py(yyyymmdd, numch, lonmin, lonmax, latmin, latmax, testfilename)
    if not result:
        logger.error('pb file_amsu_t2_to_mem_py', exc_info=True)
    else:
        logger.info('hour in file {0}'.format(result['data']['hour']))

# Run main, if called from the command line
if __name__ == '__main__':
    file_amsu_t2_to_mem_py_main()
