#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

=========================
file_correcb_to_mem_py.py
=========================

SYNOPSIS
========

.. code-block:: bash

   file_correcb_to_mem_py.py --nomsat nomsat --nocanal nocanal --yyyymmdd yyyymmdd --lonmin lonmin --lonmax lonmax --latmin latmin --latmax latmax

DESCRIPTION
===========

.. option:: --lonmin <lonmin>
.. option:: --lonmax <lonmax>
.. option:: --latmin <latmin>
.. option:: --latmax <latmax>
.. option:: --nomsat <nomsat>

   N18_MHS et N19_MHS, N15_AMSUB ou N16_AMSUB, M02_MHS

.. option:: --nocanal <nocanal>

   AMSUB/MHS: from 1 to  5

   SAPHIR: from 1 to 6

.. option:: --yyyymmdd <YYYYMMDD>

   the nearest DOY in correcb file will be used

lit les fichiers de correction AMSU-B generes par MLL
à partir de fichiers ++mensuels de résolution ERA-interim en lon, lat, pour les 90 points de fauchée

Original correction is a 4D array:

.. parsed-literal::

   dimensions:
        time = 24 ;
        fov = 90 ;
        lat = 93 ;
        lon = 480 ;

Correction is spatialy reduce to the box defined by lonmin,lonmax and latmin,latmax.

Correction is temporaly reduce to one day closest to **yyyymmdd** (among 24 DOY).

.. only:: man or text

   Figure is visible on PDF and HTML documents only.

.. only:: html or latex

  .. graphviz::

       digraph file_correcb_to_mem_py {

          correcb [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/CorrTb/corrTb_{nomsat}_L1C_B{nocanal}.nc"];
          lat [shape=tab,fontname=Courier,label="latitudes"];
          lon [shape=tab,fontname=Courier,label="longitudes];
          yyyymmdd_used [shape=tab,fontname=Courier,label="closest date"];
          correc [shape=tab,fontname=Courier,label="correction array common 3D (lat, lon, fov)"];

          file_correcb_to_mem_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/file_correcb_to_mem_py.py",
          label="${PROJECT}/src/file_correcb_to_mem_py.py"];

          {correcb} -> {file_correcb_to_mem} -> {lon lat yyyymmdd_used correc}
      }

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest ${PROJECT}/src/file_correcb_to_mem_py.py

.. warning::

   Reference for correcb files are in file:`loholt1.ipsl.polytechnique.fr:/homedata/mleduc/CorrTb/`.

Realistic example with :file:`${PROJECT_ID}/CorrTb/corrTb_N15_AMSUB_L1C_B3.nc`

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/CorrTb/
   ln -s /homedata/mleduc/CorrTb/corrTb_N15_AMSUB_L1C_B3.nc ${PROJECT_ID}/CorrTb/
   nomsat=N15_AMSUB
   nocanal=3
   yyyymmdd=20110403
   lonmin=-22
   lonmax=22
   latmin=-12
   latmax=42
   file_correcb_to_mem_py.py --nomsat ${nomsat} --nocanal ${nocanal} --yyyymmdd ${yyyymmdd} --lonmin  ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax}
   tlogd.sh file_correcb_to_mem_py_main

Realistic example with :file:`${PROJECT_ID}/CorrTb/corrTb_M02_MHS_L1C_B3.nc`

.. warning::

   Reference for correcb files are in file:`loholt1.ipsl.polytechnique.fr:/homedata/mleduc/CorrTb/`.

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/CorrTb/
   ln -s /homedata/mleduc/CorrTb/corrTb_M02_MHS_L1C_B3.nc ${PROJECT_ID}/CorrTb/
   nomsat=M02_MHS
   nocanal=3
   yyyymmdd=20060801
   lonmin=-25
   lonmax=25
   latmin=-5
   latmax=20
   file_correcb_to_mem_py.py --nomsat ${nomsat} --nocanal ${nocanal} --yyyymmdd ${yyyymmdd} --lonmin  ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax}
   tlogd.sh file_correcb_to_mem_py_main

Same in debug mode:

.. code-block:: bash

   python -m pdb ${PROJECT}/src/file_correcb_to_mem_py.py --nomsat ${nomsat} --nocanal ${nocanal} --yyyymmdd ${yyyymmdd} --lonmin  ${lonmin} --lonmax ${lonmax} --latmin ${latmin} --latmax ${latmax}

SEE ALSO
========

called by :func:`traite_amsuab_py`

returned values used by :func:`extract_amsu_py`

TODO
====

modulo +-90 ou +-180 pour lat lon +- marge

use classe cKDTree for georeduction cf. ex dans le mail de MLL

split reading and (time, space) reduction

write plot tool with "from mpl_toolkits.basemap import Basemap"

write get_correcb tool

EVOLUTIONS
==========

$Id$

$URL$

- pinsard 20141020T123328Z climtest0.ipsl.polytechnique.fr (Linux)

  * change usage of output to be able to store multiple satellites

- pinsard 20141014T132022Z climtest0.ipsl.polytechnique.fr (Linux)

  * fix lat and lon reduction

- pinsard 20141007T132000Z loholt1.ipsl.polytechnique.fr (Linux)

  * fix log
  * use raise

- pinsard 20140813T142144Z loholt1.ipsl.polytechnique.fr (Linux)

  * add lat and lon to result

- pinsard 20140811T084235Z loholt1.ipsl.polytechnique.fr (Linux)

  * fix spatial reduction
  * organize output in a dictionary

- pinsard 20140808T100606Z loholt1.ipsl.polytechnique.fr (Linux)

  * migration from file_correcb_to_mem.pro
    thanks to http://schubert.atmos.colostate.edu/~cslocum/netcdf_example.html
    and http://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array
    http://gis.stackexchange.com/questions/71630/subsetting-a-curvilinear-netcdf-file-roms-model-output-using-a-lon-lat-boundin

    use https://github.com/GoogleCloudPlatform/appengine-24hrsinsf-python/blob/master/geobox.py
    copied to external_python/others


"""

__all__ = ["file_correcb_to_mem_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

import netCDF4
import numpy
import dateutil.parser


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """Read AMSU-B correction files"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--version',
                      action='store_true',
                      dest='version',
                      help='show version number and exit')

    parser.add_option('--lonmin',
                      help='min longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmin')

    parser.add_option('--lonmax',
                      help='max longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmax')

    parser.add_option('--latmin',
                      help='min latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmin')

    parser.add_option('--latmax',
                      help='max latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmax')

    parser.add_option('--yyyymmdd',
                      help='date (yyyymmdd)',
                      default=None,
                      metavar='<DATE>',
                      dest='yyyymmdd',
                      action='store')

    parser.add_option('--nomsat',
                      help='name of the channel',
                      type='choice',
                      choices=['N18_MHS', 'N19_MHS', 'N15_AMSUB', 'N16_AMSUB', 'M02_MHS'],
                      dest='nomsat',
                      metavar='<NOMSAT>',
                      action='store')

    parser.add_option('--nocanal',
                      help="no channel (AMSUB/MHS: from 1 to 5; SAPHIR: from 1 to 6')",
                      type='int',
                      default=None,
                      metavar='<NOCANAL>',
                      dest='nocanal')

    return parser


def file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax):

    """

    test M02_MHS

    >>> nomsat = 'M02_MHS'
    >>> nocanal = 3
    >>> yyyymmdd = dateutil.parser.parse('20060313', ignoretz=True)
    >>> lonmin = -25
    >>> lonmax = 25
    >>> latmin = -5
    >>> latmax = 20
    >>> resultonesat = file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax)
    >>> result = {nomsat: resultonesat}
    >>> print('test {0}: date {1}'.format(result[nomsat]['nomsat'], result[nomsat]['yyyymmdd']))
    test M02_MHS: date 2006-03-17 00:00:00

    test N15_AMSUB

    >>> nomsat = 'N15_AMSUB'
    >>> nocanal = 3
    >>> yyyymmdd = dateutil.parser.parse('20110403', ignoretz=True)
    >>> lonmin = -22
    >>> lonmax = 22
    >>> latmin = -12
    >>> latmax = 42
    >>> resultonesat = file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax)
    >>> result = {nomsat: resultonesat}
    >>> print('test {0}: date {1}'.format(result[nomsat]['nomsat'], result[nomsat]['yyyymmdd']))
    test N15_AMSUB: date 2011-04-01 00:00:00
    >>> print('test {0}: min lat {1} min lon {2}'.format(result[nomsat]['nomsat'], min(result[nomsat]['lat']), min(result[nomsat]['lon'])))
    test N15_AMSUB: min lat -27.75 min lon -37.5
    >>> print('test {0}: max lat {1} max lon {2}'.format(result[nomsat]['nomsat'], max(result[nomsat]['lat']), max(result[nomsat]['lon'])))
    test N15_AMSUB: max lat 34.5 max lon 37.5

    """

    result = None

    logger = logging.getLogger()

    # build filename
    project_id_env = os.environ['PROJECT_ID']
    fullfilename = '{0}/CorrTb/corrTb_{1}_L1C_B{2}.nc'.format(project_id_env, nomsat, nocanal)
    # test if file exist
    if not os.path.isfile(fullfilename):
        msg = 'file {0} not found'.format(fullfilename)
        logger.error(msg)
        raise IOError(msg)

    # open file
    fid = netCDF4.Dataset(fullfilename)

    # copy the data to numpy arrays
    correcb_doy = fid.variables[u'jday'][:]
    correcb_lon = fid.variables[u'lon'][:]
    correcb_lat = fid.variables[u'lat'][:]
    # print(fid.variables[u'corrTb'].shape)  # dimensions (24, 93, 480, 90)

    correcb_corrTb = fid.variables[u'corrTb'][:]

    # close file
    fid.close()

    # nearest doy for temporal reduction
    try:
        doy = int(yyyymmdd.strftime('%j'))
        doy_idx = numpy.abs(correcb_doy - doy).argmin()
    except:
        logger.error('pb find nearest doy', exc_info=True)
        raise
    else:
        yyyy = yyyymmdd.strftime('%Y')
        yyyy_doy_str = '{0} {1}'.format(yyyy, int(correcb_doy[doy_idx]))
        del(yyyy)
        yyyymmdd = datetime.strptime(yyyy_doy_str, '%Y %j')
        del(yyyy_doy_str)
        logger.info('doy asked: {0} nearest found: {1} {2}'.format(doy, correcb_doy[doy_idx],  yyyymmdd))

    # spatial reduction: marge autour du domaine cible pour restriction des recherches dans l'orbite

    dim_doy, dim_lat, dim_lon, dim_fov = correcb_corrTb.shape
    marge = 16.
    # ++ migration xmin = (lonmin - marge + 180.)
    # ++ migration ymin = (latmin - marge + 90.)
    # ++ migration xmax = (lonmax + marge + 180.)
    # ++ migration ymax = (latmax + marge + 90.)
    try:
        lon_indices_ok = numpy.where(numpy.logical_and(correcb_lon < lonmax + marge, correcb_lon > lonmin - marge))[0]
    except:
        logger.error('pb find lon interval', exc_info=True)
        raise
    else:
        logger.info('lon interval asked: {0} - {1} ({2}), {3} + {4} ({5}); found: {6}'.format(lonmin,  marge, lonmin - marge, lonmax, marge, lonmax + marge, correcb_lon[lon_indices_ok]))
    lon_indices_not_ok = [ i for i in range(0, dim_lon) if i not in lon_indices_ok]

    try:
        lat_indices_ok = numpy.where(numpy.logical_and(correcb_lat < latmax + marge, correcb_lat > latmin - marge))[0]
    except:
        logger.error('pb find lat interval', exc_info=True)
        raise
    else:
        logger.info('lat interval asked: {0} - {1} ({2}), {3} + {4} ({5}); found: {6}'.format(latmin,  marge, latmin - marge, latmax, marge, latmax + marge, correcb_lat[lat_indices_ok]))
    lat_indices_not_ok = [ i for i in range(0, dim_lat) if i not in lat_indices_ok]

    axis_doy, axis_lat, axis_lon, axis_fov = (0, 1, 2, 3)

    correc_timereduced = correcb_corrTb[doy_idx, :, :, :]
    try:
        correc_timereduced_latreduced = numpy.delete(correc_timereduced, lat_indices_not_ok, axis=0)
    except ValueError as error:
        logger.error('pb latitude reduction ValueError : {0}'.format(error), exc_info=True)
        logger.error(' lat_indices {0}'.format(lat_indices))
        raise

    try:
        correc_timereduced_latreduced_lonreduced = numpy.delete(correc_timereduced_latreduced, lon_indices_not_ok, axis=1)
    except ValueError as error:
        logger.error('pb longitude reduction ValueError: {0}'.format(error), exc_info=True)
        logger.error(' lon_indices_not_ok {0}'.format(lon_indices_not_ok))
        raise

    result = {'nomsat': nomsat , 'yyyymmdd': yyyymmdd,
              'correction': correc_timereduced_latreduced_lonreduced,
              'lon': correcb_lon[lon_indices_ok],
              'lat': correcb_lat[lat_indices_ok]}

    return result


def file_correcb_to_mem_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['nomsat', 'nocanal', 'yyyymmdd', 'lonmin', 'lonmax', 'latmin', 'latmax']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    nomsat = fromcli.nomsat
    nocanal = fromcli.nocanal
    yyyymmdd = dateutil.parser.parse(fromcli.yyyymmdd, ignoretz=True)
    lonmin = fromcli.lonmin
    lonmax = fromcli.lonmax
    latmin = fromcli.latmin
    latmax = fromcli.latmax

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger(command)

    if is_verbose:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))

    logger.info('[Parameters]')
    logger.info('nomsat={0}'.format(nomsat))
    logger.info('nocanal={0}'.format(nocanal))
    logger.info('yyyymmdd={0}'.format(yyyymmdd))
    logger.info('lonmin={0}'.format(lonmin))
    logger.info('lonmax={0}'.format(lonmax))
    logger.info('latmin={0}'.format(latmin))
    logger.info('latmax={0}'.format(latmax))

    try:
        resultonesat = file_correcb_to_mem_py(nomsat, nocanal, yyyymmdd, lonmin, lonmax, latmin, latmax)
        result = {nomsat: resultonesat}
    except:
        logger.error('pb file_correcb_to_mem_py', exc_info=True)
        raise
    else:
        logger.info('correction  dimensions {0}'.format(result[nomsat]['correction'].shape))
        logger.info('date {0}'.format(result['yyyymmdd']))

# Run main, if called from the command line
if __name__ == '__main__':
    file_correcb_to_mem_py_main()
