#!/usr/bin/env python
# -- coding: utf-8 -*-

"""

===========================
geolocation_to_string_py.py
===========================

DESCRIPTION
===========

.. option:: --help
.. option:: --verbose

.. option:: lon_value <LON>

   longitude, W < 0, in degree

.. option:: lat_value <LAT>

   latitude, N > 0, in degree

.. option:: look

   'filename'

.. option:: scale <FLOAT>

   ++ puissance de 10.

.. only:: man

    Figure is visible on PDF and HTML documents only.

.. only:: html or latex

   .. graphviz::

      digraph geolocation_to_string_py {


          geolocation_to_string_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/geolocation_to_string_py.py",
          label="${PROJECT}/src/geolocation_to_string_py.py"]

          {lon lat look scale} -> geolocation_to_string_py -> string

      }


Build a string using W, E, S or N according to values of latitude and
longitude

if look ++

if scale ++

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest geolocation_to_string_py.py

A realistic example with integer values:

.. code-block:: bash

   lon_value=-11
   lat_value=10
   look='filename'
   scale=1
   geolocation_to_string_py.py --lon ${lon_value} --lat ${lat_value} --look ${look} --scale ${scale}

   lon_value=-11
   lat_value=10
   look='figure'
   scale=1
   geolocation_to_string_py.py --lon ${lon_value} --lat ${lat_value} --look ${look} --scale ${scale}

Same in debug mode:

.. code-block:: bash

   python -m pdb ${PROJECT}/src/geolocation_to_string_py.py --lon ${lon_value} --lat ${lat_value} --look ${look} --scale ${scale}

SEE ALSO
========

.. index:: ! python

equivalent functionality :
:func:`geolocation_to_string_idl` (IDL)
:ref:`geolocation_to_string_sh.sh` (shell script)
:ref:`geolocation_to_string.m` (octave/matlab)

TODO
====

pb unicode
    '11.00\xc2\xb0W,10.00\xc2\xb0N'
vs
    '11.00Â°W,10.00Â°N'
vs
    '11.00Â°W,10.00Â°N'

doc look et scale

doctest error (equivalent de failed octave)

traduction trouver les points cardinaux N or S and E or W

parameter convention ?

return values in case of error

complete digraph

log is prefixed by geolocation_to_string_py_main : should be geolocation_to_string_py

EVOLUTIONS
==========

$Id$

$URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/geolocation_to_string_py.py $


- fplod 20140805T124502Z callisto.locean-ipsl.upmc.fr (Linux)

  * add look and scale parameters

- fplod 20140730T090145Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from geolocation_to_string.m

"""

__all__ = ["geolocation_to_string_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """%prog: Build a string using W, E, S or N according to values of latitude and longitude"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')
    parser.add_option('--version',
                      action='store_true',
                      dest='version',
                      help='show version number and exit')
    parser.add_option('--lon',
                      help='longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lon_value')
    parser.add_option('--lat',
                      help='latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='lat_value')

    parser.add_option('--look',
                      help='for filename  or figure',
                      type='choice',
                      choices=['filename', 'figure'],
                      dest='look',
                      action='store')

    parser.add_option('--scale',
                      help='scale',
                      type='float',
                      default=None,
                      dest='scale')

    return parser


def geolocation_to_string_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['lon_value', 'lat_value', 'look', 'scale']
    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    lon_value = fromcli.lon_value
    lat_value = fromcli.lat_value
    look = fromcli.look
    scale = fromcli.scale

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    logging.basicConfig(filename=log, level=logging.INFO)

    logging.info('[Context]')
    logging.info('PID={0}'.format(os.getpid()))
    logging.info('command={0}'.format(command))
    import socket
    logging.info('hostname={0}'.format(socket.getfqdn()))
    logging.info('runtime={0}'.format(log_date))
    logging.info('log={0}'.format(log))

    logging.info('[Parameters]')
    logging.info('lon_value={0}'.format(lon_value))
    logging.info('lat_value={0}'.format(lat_value))
    logging.info('look={0}'.format(look))
    logging.info('scale={0}'.format(scale))

    status = geolocation_to_string_py(lon_value, lat_value, look, scale)
    print status


def geolocation_to_string_py(lon_value, lat_value, look, scale):

    """
    :param lon_value: longitude, W < 0
    :units lon_value: deg
    :type lon_value: float
    :raise lon_value: required

    :param lat_value: latitude, N > 0
    :units lat_value: deg
    :type lat_value: float
    :raise lat_value: required

    :param look: for filename or figure
    :type look: string
    :raise look: required

    :param scale: scale to be applied to lat and lon
    :type scale: float
    :raise scale: required

    :returns: a string or ++ if error
    :rtype: string

    >>> lon_value = -11.
    >>> lat_value = 10.
    >>> look = 'filename'
    >>> scale = 1.
    >>> geolocation_to_string_py(lon_value, lat_value, look, scale)
    '011w10n'

    >>> lon_value = -11.
    >>> lat_value = 10.
    >>> look = 'figure'
    >>> scale = 1.
    >>> geolocation_to_string_py(lon_value, lat_value, look, scale)
    '11.00°W,10.00°N'

    >>> lon_value = -11.
    >>> lat_value = None
    >>> look = 'figure'
    >>> scale = 1.
    >>> #++geolocation_to_string_py(lon_value, lat_value, look, scale)

    >>> lon_value = None
    >>> lat_value = 10.
    >>> look = 'figure'
    >>> scale = 1.
    >>> #++geolocation_to_string_py(lon_value, lat_value, look, scale)
    """

    result = -1

    logger = logging.getLogger()

    if (lon_value > 0.):
        if look == 'figure':
            west_east = 'E'
        elif look == 'filename':
            west_east = 'e'
    else:
        if look == 'figure':
            west_east = 'W'
        elif look == 'filename':
            west_east = 'w'
    if (lat_value > 0.):
        if look == 'figure':
            south_north = 'N'
        elif look == 'filename':
            south_north = 'n'
    else:
        if look == 'figure':
            south_north = 'S'
        elif look == 'filename':
            south_north = 's'
    #
    # conversion lon and lat absolute values to string
    if look == 'figure':
        lon_string = '{0:.2f}'.format(abs(lon_value))
        lat_string = '{0:.2f}'.format(abs(lat_value))
    elif look == 'filename':
        lon_string = '{0:03d}'.format(int(abs(lon_value)))
        lat_string = '{0:02d}'.format(int(abs(lat_value)))
    #
    if look == 'figure':
        sep = u','
        unit_string = "°"  # ++ unicode
    elif look == 'filename':
        sep = u''
        unit_string = ""
    #
    # build the result
    geolocation_string = '{0}{1}{2}{3}{4}{5}{6}'.format(lon_string, unit_string, west_east, sep, lat_string, unit_string, south_north)
    #
    del(sep)
    del(unit_string)
    del(lon_string)
    del(lat_string)
    del(west_east)
    del(south_north)

    return geolocation_string

# Run main, if called from the command line
if __name__ == '__main__':
    geolocation_to_string_py_main()
