#! /bin/sh
#+
#
# .. program:: get_msg_xrit.sh
#
# .. _get_msg_xrit.sh:
#
# ===============================================
# get_msg_xrit.sh -- get MSG xrit reference files
# ===============================================
#
# SYNOPSIS
# ========
#
# .. code-block:: bash
#
#    get_msg_xrit.sh [-f] -p product -b yyyymmdd -e yyyymmdd
#
# DESCRIPTION
# ===========
#
# ``get_msg_xrit.sh`` get MSG xrit reference files from
# **yyyymmddb** include to **yyyymmdde** excluded.
#
# .. option:: -f
# .. option:: -p product name IR108
# .. option:: -b beginning date <yyyymmdd>
# .. option:: -e end date <yyyymmdd>
#
# Log file is written on
# :file:`${PROJECT_LOG}/get_msg_xrit.log.{YYYYMMDDTHHMMSSZ}`
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph get_msg_xrit {
#           graph [
#             rankdir="TB",
#           ]
#
#           msg_xrit_ref_t1 [shape=diamond,
#           fontname=Courier,
#           label="loholt1.ipsl.polytechnique.fr:/bdd/msg/images/xrit/yyyy/mm/dd/H-000-MSG1__-MSG11________-prod____-CYCLE____-yyyymmddhhmn.gz"];
#
#           msg_xrit_t1 [shape=ellipse,
#           fontname=Courier,
#           label="${PROJECT_ID}/MSG/yyyy/mm/dd/H-000-MSG1__-MSG11________-prod____-CYCLE____-yyyymmddhhmn.gz"];
#
#           log [shape=ellipse,
#           fontname=Courier,
#           label="${PROJECT_LOG}/get_msg_xrit.log.YYYYMMDDTHHMMSSZ"];
#
#           get_msg_xrit [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/get_msg_xrit.sh",
#           label="${PROJECT}/src/get_msg_xrit.sh"];
#
#           {msg_xrit_ref_t1} -> {get_msg_xrit} -> {log msg_xrit_t1}
#
#          }
#
# EXAMPLES
# ========
#
# You don't have any MSG xrit reference data and you want IR108 during
# [20060801, 20060802[, you just have to run this tool :
#
# .. code-block:: bash
#
#    get_msg_xrit.sh -p IR108 -b 20060801 -e 20060802
#
# And look at log file with :
#
# .. code-block:: bash
#
#    tlogd.sh get_msg_xrit
#
# and of course on files in ${PROJECT_ID}.
#
# CAUTIONS
# ========
#
# how to know if there is enough place before ? 9.9GB/day !!
#
# SEE ALSO
# ========
#
# :ref:`data_msg` ++
#
# :ref:`varamma_profile.sh`
#
# Next step :
# :ref:`traite_msg-prod.sh` ++
#
# TODO
# ====
#
# next step : according to KR 20110428 :
# "Pour le format XRIT, deux programmes de lecture sont disponibles
# dans /bdd/msg/software/read_xrit."
#
# .. code-block:: bash
#
#    [pinsard@loholt1 xrit2raw]$ cd /bdd/msg/software/read_xrit
#    [pinsard@loholt1 read_xrit]$ ls -l
#
# .. parsed-literal::
#    :class: stdout
#
#    total 8
#    drwxr-xr-x 2 climserv climserv 4096 Sep 16  2009 readXRIT
#    drwxr-xr-x 4 climserv climserv 4096 Jul 22  2011 xrit2raw
#
# In xrit2raw, version 0.1.9 avec des patches.
# In http://www.icare.univ-lille1.fr/tools/, I discover
# http://www.icare.univ-lille1.fr/wiki/index.php/Xrit2raw where
# release 0.2.7 seems to be available
#
# .. code-block:: bash
#
#    wget http://www-loa.univ-lille1.fr/~ducos/public/xrit/xrit-0.2.7.tar.gz
#    tar xtvf xrit-0.2.7.tar.gz
#    cd xrit-0.2.7
#    mkdir -p ${PROJECT}/bin
#    export PATH=${PROJECT}/bin:${PATH}
#    make install INSTDIR=/home/pinsard/varamma/bin
#
#    file_msg_xrit_gz=${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.gz
#    file_msg_xrit_gz_rf=/tmp/$(basename ${file_msg_xrit_gz})
#    file_msg_xrit=${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000
#    cp ${file_msg_xrit_gz} ${file_msg_xrit_gz_rf}
#    gunzip -c ${file_msg_xrit_gz_rf} > ${file_msg_xrit}
#    rm ${file_msg_xrit_gz_rf}
#
#    xrit2raw -I ${file_msg_xrit}
#
#   For explanations about the following quantities, please refer to the
#   documentation from Eumetsat about SEVIRI, available there (at the time of
#   this writing):
#   http://www.eumetsat.int/idcplg?IdcService=GET_FILE&dDocName=PDF_TEN_05105_MSG_IMG_DATA&RevisionSelectionMethod=LatestReleased
#   the author of the software won't provide any assistance on this.
#
#   Nominal Longitude: -3.400000 (actual longitude of the satellite in the
#   space, counted positively towards the east from Greenwich)
#   Longitude of subsatellite point: 0.000000 (it's the actual term from
#   Eumetsat documentation, to make it clear it is the longitude of the central
#   pixel of the image after reprojection)
#   calibration coefficients : VIS006       (1)     slope:    0.0231281    offset:     -1.17953
#   calibration coefficients : VIS008       (2)     slope:    0.0297266    offset:     -1.51606
#   calibration coefficients : IR016        (3)     slope:    0.0236219    offset:     -1.20472
#   calibration coefficients : IR039        (4)     slope:   0.00365867    offset:    -0.186592
#   calibration coefficients : WV062        (5)     slope:   0.00831811    offset:    -0.424224
#   calibration coefficients : WV073        (6)     slope:     0.038622    offset:     -1.96972
#   calibration coefficients : IR087        (7)     slope:     0.126744    offset:     -6.46392
#   calibration coefficients : IR097        (8)     slope:     0.103961    offset:     -5.30202
#   calibration coefficients : IR108        (9)     slope:     0.205034    offset:     -10.4568
#   calibration coefficients : IR120        (10)    slope:     0.222311    offset:     -11.3379
#   calibration coefficients : IR134        (11)    slope:     0.157607    offset:     -8.03795
#   calibration coefficients : HRV          (12)    slope:    0.0319993    offset:     -1.63196
#
# génération du prologue : ++ on en fait quoi ?
#
# .. code-block:: bash
#
#    xrit2raw -P ${file_msg_xrit}_prolog ${file_msg_xrit}
#
# Conversion en format raw
#
# .. code-block:: bash
#
#    xrit2raw -F -999. -v -f ${file_msg_xrit}
#    xrit2raw: H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw: height: 3712 width: 3712, data type: float (real 32 bits) fill value: -999.000000
#
# 3712*3712=13778944 3712*3712*4=55115776 et taille du fichier raw 18079230 ... le compte n'est pas bon
#
# Si en fait !!! je regardais la taille du fichier original:
#
# .. code-block:: bash
#
#    ls -l ${file_msg_xrit} ${file_msg_xrit}.raw
#    -rw-r--r-- 1 pinsard users 18079230 Jul 30 09:51 /homedata/pinsard/varamma_d//MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000
#    -rw-r--r-- 1 pinsard users 55115776 Jul 30 10:15 /homedata/pinsard/varamma_d//MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw
#
# Pour voir la valeur de la case (1,1) :
#
# .. code-block:: bash
#
#    od -f --read-bytes 4 ${file_msg_xrit}.raw
#    0000000  -9.990000e+02
#    0000004
#
# Pour voir la 1re ligne/colonne :
#
# .. code-block:: bash
#
#    od -f --read-bytes $((4*3712)) ${file_msg_xrit}.raw
#    0000000  -9.990000e+02  -9.990000e+02  -9.990000e+02  -9.990000e+02
#    *
#    0035000
#
# Pour voir la 2e ligne/colonne :
#
# .. code-block:: bash
#
#    od -f --skip-bytes $((3712*(2-1))) --read-bytes $((4*3712)) ${file_msg_xrit}.raw
#
# Pour créer un fichier avec la 1re ligne:
#
# .. code-block:: bash
#
#    dd if=${file_msg_xrit}.raw of=${file_msg_xrit}.1l.raw
#    107648+0 records in
#    107648+0 records out
#    55115776 bytes (55 MB) copied, 1.68088 seconds, 32.8 MB/s
#
# On retrouve bien 3712*3712*4=55115776
#
# Autre piste de conversion /bdd/msg/software/read_xrit/readXRIT:
#
# Compilation + édition de lien
#
#    cd /bdd/msg/software/read_xrit/readXRIT
#    gcc -o ${PROJECT}/bin/readXRIT readXRIT1-4.c msglib2-21.c rad2tmp2-3.c -Wall -O3 -lm -lz -lbz2
#    readXRIT1-4.c: In function 'main':
#    readXRIT1-4.c:92: warning: 'maxlon' may be used uninitialized in this function
#    readXRIT1-4.c:92: warning: 'maxlat' may be used uninitialized in this function
#    readXRIT1-4.c:92: warning: 'minlon' may be used uninitialized in this function
#    readXRIT1-4.c:92: warning: 'minlat' may be used uninitialized in this function
#    msglib2-21.c: In function 'Convers':
#    msglib2-21.c:384: warning: pointer targets in passing argument 1 of 'BConvpro' differ in signedness
#    msglib2-21.c:384: warning: pointer targets in passing argument 2 of 'BConvpro' differ in signedness
#    msglib2-21.c: In function 'Converx':
#    msglib2-21.c:2096: warning: pointer targets in passing argument 1 of 'BConvpro' differ in signedness
#    msglib2-21.c:2096: warning: pointer targets in passing argument 2 of 'BConvpro' differ in signedness
#    msglib2-21.c:2098: warning: pointer targets in passing argument 1 of 'CConvpro' differ in signedness
#    msglib2-21.c:2098: warning: pointer targets in passing argument 2 of 'CConvpro' differ in signedness
#
# est-ce que les warning disparaissent avec une version plus récente ? pour
# l'instant je ne sais pas où elle se trouve.
#
# Je m'inspire de /bdd/msg/software/read_xrit/readXRIT/README pour faire
# un test:
#
# .. code-block:: bash
#
#    readXRIT ${file_msg_xrit_gz} ${file_msg_xrit}.readXRIT.raw LATLON 40 55 -5 5 RAW CAL NOROT

#    images des longitudes LONS absente
#
# J'essaie avec un critère ligne, colonne plutôt que lat,lon
#
# .. code-block:: bash
#
#    readXRIT ${file_msg_xrit_gz} ${file_msg_xrit}.readXRIT.raw LINECOL 1 3712 1 3712 RAW CAL NOROT
#    Zone indéterminée : LINECOL
#    Choix possibles : LATLON ou LIGCOL
#
# donc README pas à jour ++ à signaler
#
# Je réduis le domaine ...
#
# .. code-block:: bash
#
#    readXRIT ${file_msg_xrit_gz} ${file_msg_xrit}.readXRIT.raw LIGCOL 1 2 1 2 RAW CAL NOROT
#    images des longitudes LONS absentes
#
# hyperlouche !!
#
# Je soupçonne une embrouille sur la localisation des fichiers LATS et LONS:
#
#    ln -s /bdd/msg/software/read_xrit/readXRIT/LATS $(pwd)/
#    ln -s /bdd/msg/software/read_xrit/readXRIT/LONS $(pwd)/
#    readXRIT ${file_msg_xrit_gz} ${file_msg_xrit}.readXRIT.raw LIGCOL 1 2 1 2 RAW CAL NOROT
#
# je laisse de coté à cause de la phrase suivante dans /bdd/msg/software/read_xrit/readXRIT/readXRIT1-4.c
#
# .. parsed-literal::
#
#    ATTENTION :
#    Ces bornes permettent de s'assurer que tous les pixels compris dans cette boite
#    englobantes apparaissent sur l'image résultat. Il ne définissent en aucun cas des
#    limites ne pouvant être dépassées par la fenêtre. On peut très bien
#    avoir des pixels correspondant à une latitude de 21N même si on a
#    spécifié comme limite 20N.
#
# Dans http://www.icare.univ-lille1.fr/tools/, I discover Remap A reprojection tool of geolocated data between various grids.
# Faut essayer
#
# .. code-block:: bash
#
#    cd
#    wget http://www.icare.univ-lille1.fr/projets/ducos/public/remap_website/remap-0.4.6.tar.gz
#    tar xvzf remap-0.4.6.tar.gz
#    cd remap-0.4.6.tar.gz
#
# En lisant le fichier INSTALL, je me rends compte qu 1) beaucoup de
# prérequis en cascade (filedata, grib), 2) très vieux - 2006 -
# donc je laisse aussi de côté
#
# On peut voir des images de MSG avec le portail ICARE
# http://www.icare.univ-lille1.fr/msg/browse/index.php?date=20060801&all=all#
# On remarquera que le rectangle noir dans l'image 20060801T091500Z
# http://www.icare.univ-lille1.fr/msg/browse/openWin.php?img=/DATA/LIENS/SEVIRI/L1_B/2006/2006_08_01/SEVIRI_L1B_20060801.0915.12km.jpg
# idem à 20060801T101500Z, rose à 20060801T114500Z
#
# je ne sais pas le paramètre montré dans ces images "false color level 1" ...
#
# ...
#
# make it work outside ClimServ
#
# build/complete catalog
#
# handle -f option
#
# describe next step traite_msg-prod.sh
#
# See strange behaviour of date command :
#
# .. code-block:: bash
#
#    date -d '20060622T0645' +%Y%m%dT%H%M
#    20060622T0145
#
# while 20060622T0645 expected !!
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# $URL$
#
# - pinsard 20130729T094455Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * creation from get_msg.sh /bdd/AMMA/AOC-PARIS to sharp longitude range
#     longitude min = 25°W, 40°W needed to follow cyclones
#
#-
system=$(uname)
case "${system}" in
    AIX|IRIX64)
        echo "www : no specific posix checking"
        date_cmd=date
    ;;
    Darwin)
        set -o posix
        date_cmd=gdate
    ;;
    Linux)
        set -o posix
        date_cmd=date
    ;;
    *)
        set -o posix
    ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-f] -p product -b yyyymmdd -e yyyymmdd"
#
hostname=$(hostname)
#
yyyymmddb_min=20060420
yyyymmdde_max=20081103
#
# default
force=0
product=IR108
yyyymmddb=20060801
yyyymmdde=20060802
#
minargcount=6
if [ ${#} -lt ${minargcount} ]
then
    echo "eee : not enough arguments"
    echo "${usage}"
    exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
    case ${1} in
        -p)
            product=${2}
            shift
        ;;
        -b)
            # first date to get
            yyyymmddb=${2}
            shift
        ;;
        -e)
            # last date to get
            yyyymmdde=${2}
            shift
        ;;
        -f)
            force=1
        ;;
        -h)
            echo "${usage}"
            exit 0
        ;;
        *)
            # other choice
            echo "eee : unknown option ${1}"
            echo "${usage}"
            exit 1
        ;;
    esac
    # next flag
    shift
done
unset usage
#
# check parameters
# and define associated variables
#
case ${product} in
    IR108)
        prefix="IR108"
    ;;
    *)
       echo "${command} : eee : product ${product} argument invalid"
       exit 1
    ;;
esac
#
${date_cmd} -u -d "${yyyymmddb}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmddb}" < "${yyyymmddb_min}" ]]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} must be equal or greater than yyyymmddb_min ${yyyymmddb_min}"
    exit 1
fi
#
${date_cmd} -u -d "${yyyymmdde}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmdde}" > "${yyyymmdde_max}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be lower or equal to yyyymmdde_max ${yyyymmdde_max}"
    exit 1
fi
#
if [[ "${yyyymmdde}" < "${yyyymmddb}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
    exit 1
fi
#
if [[ "${yyyymmdde}" = "${yyyymmddb}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
    exit 1
fi
#
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
    echo "${command} : eee : \${PROJECT_LOG} not defined"
    exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
    echo "${command} : eee : ${PROJECT_LOG} not found"
    exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
    echo "${command} : eee : ${PROJECT_LOG} not reachable"
    exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
    echo "${command} : eee : ${PROJECT_LOG} not writable"
    exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "product=${product}" 1>> ${log}
echo "yyyymmddb=${yyyymmddb}" 1>> ${log}
echo "yyyymmdde=${yyyymmdde}" 1>> ${log}
echo "" 1>> ${log}
#
dirref_base=/bdd/msg/images/xrit/
#
# loop over all dates to build URL lists
list_url=""
current=$(${date_cmd} -u -d "${yyyymmddb}" +%s | awk '{printf "%d",$1}')
declare -i current
end=$(${date_cmd} -u -d "${yyyymmdde}" +%s | awk '{printf "%d",$1}')
declare -i end
declare -i delta=900
while [ ${current} -lt ${end} ]
do
    echo "iii : looking for files of $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%dT%H%M)" 1>> ${log}
    yyyymmdd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)
    yyyy=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y)
    mm=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%m)
    dd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%d)
    hhmn=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%H%M)
    dirref="${dirref_base}/${yyyy}/${mm}/${dd}"
    #
    # look for these reference files
    bfile_gz=H-000-MSG1__-MSG1________-${prefix}____-CYCLE____-${yyyymmdd}${hhmn}.gz
    file_gz=${dirref}/${bfile_gz}
    exist=0
    if [ ! -f ${file_gz} ]
    then
        echo "iii : ${file_gz} not found" >> ${log} 2>&1
    else
        echo "iii : ${file_gz} found" >> ${log} 2>&1
        exist=$(( ${exist} + 1))
    fi
    if [ ${exist} -eq 1 ]
    then
        list_url="${list_url} ${file_gz}"
    fi
    unset yyyymmdd
    unset yyyy
    unset mm
    unset dd
    unset hhmn
    unset dirref
    #
    # next time step
    current=$((${current} + ${delta}))
done
unset current
unset end
#
if [ "${list_url}" = "" ]
then
    echo "iii : no files founds" >> ${log} 2>&1
else
    #
    # if file is not already in ${PROJECT_ID}, get it
    for url in ${list_url}
    do
        burl=$( basename ${url})
        after_cycle=${burl#*CYCLE____-}
        url_yyyymmdd=${after_cycle%_hh}
        yyyy=${url_yyyymmdd:0:4}
        mm=${url_yyyymmdd:4:2}
        dd=${url_yyyymmdd:6:2}
        file=${PROJECT_ID}/MSG/${yyyy}/${mm}/${dd}/$(basename ${url})
        if [ ! -f ${file} ]
        then
            loc=$(dirname ${file})
            if [ ! -d ${loc} ]
            then
                mkdir -p ${loc}
                echo "iii : adding ${loc}" >> ${log}
            fi
            ln -s ${url} ${file}
            echo "iii : adding ${file}" >> ${log}
        else
            echo "iii : ${file} exists" >> ${log}
        fi
        unset yyyy
        unset mm
        unset dd
        unset burl
        unset after_cycle
        unset loc
    done
    unset file
    unset url
fi
unset list_url
#
unset command
unset log
unset hostname
unset usage
#
unset date_cmd
#
# end
#++set
exit 0
