function [hovdata] = hovmullerlonorlat(data,dim1,dim2,dim3,numdimtomean,centralvalue,extmeandim, nomvar)

%hovmullerlonorlat permet de réaliser un diagramme de hovmöller à partir
%d'une matrice 3D (temps, longitudes, latitudes). Celui-ci peut-être
%latitudes vs temps ou temps vs longitudes.

%
%+
%
% ====================
% hovmullerlonorlat.m
% ====================
%
% .. function:: hovmullerlonorlat(data,dim1,dim2,dim3,numdimtomean,centralvalue,extmeandim,nomvar)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * data : Matrice de données 3D (temps, longitudes, latitudes).
%   * dim1 : Variable de la dimension 1 de la matrice 'data', c'est-a-dire
%     le temps.
%   * dim2 : Variable de la dimension 2 de la matrice 'data', c'est-a-dire
%     les longitudes.
%   * dim3 : Variable de la dimension 3 de la matrice 'data', c'est-a-dire
%     les latitudes.
%   * numdimtomean : Numéro de la dimension spatiale sur laquelle il faudra
%     moyenner. (Ex : Pour un diagramme latitudes vs temps, il faut
%     moyenner sur les longitudes soit la dimension 2).
%   * centralvalue : Valeur centrale du moyennage sur la dimension
%     correspondante (en degré + si N/E ou - si S/W).
%   * extmeandim : Étendue du moyennage autour de la valeur centrale (en
%     degré).
%   * nomvar (caractères) : Nom et unité de la donnée étudiée ici
%     (Ex : 'cumuls EPSAT (mm)').
%
% - Données de sorties :
%
%   * hovdata : Matrice 2D comportant les données du diagramme de
%     hovmöller. La première dimension est le temps et la dimension
%     spatiale du diagramme.
%
% Cette fonction permet d'analyser les données MSG et de concaténer les
% intensités résultantes en sortie. L'analyse s'effectue sur une seule
% saison, sur tous les fichiers ou entre deux dates prédéfinies.
%
% EXAMPLES
% ========
%
% Voir la 'demo'.
%
% SEE ALSO
% ========
%
% TODO
% ====
%
% use datetick
%
% check if demo latitude correct
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/hovmullerlonorlat.m $
%
% - fplod 20110804T123916Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add demo latitude
%
% - jaclod 2011-07-27
%
%   * Révision de la documentation et des commentaires.
%   * Remplacement des arguments d'entrées 'minmeandim' et 'maxmeandim' par
%     'centralvalue' et 'extmeandim' et modification des titres.
%
% - jaclod 2011-07-21
%
%   * Modification de l'affichage des données temporelles sur l'axe
%     correspondant.
%   * Ajout de la documentation et des commentaires.
%
% - jaclod 2011-07-20
%
%   * Creation
%
%-
usage='[hovdata] = hovmullerlonorlat(data,dim1,dim2,dim3,numdimtomean,centralvalue,extmeandim, nomvar)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
if size(size(data),2) ~= 3;
    error('Les données doivent etre dans une matrice a 3 dimensions.')
end

if numdimtomean ~= 2 && numdimtomean ~= 3;
    error('Le moyennage ne se fait que sur l''une des dimensions spatiales de la matrice de données (temps, longitudes, latitudes).');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Création des bornes sur la dimension moyenne
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

minmeandim = centralvalue - extmeandim/2;
maxmeandim = centralvalue + extmeandim/2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extraction des données concernées
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% On cherche les indices entre lesquelles il faut prendre les données. On
% extrait ensuite les données correspondantes.
if numdimtomean == 2;

    %%%%% Recherche des indices %%%%%
    indicemin = find(dim2 >= minmeandim,1,'first');
    indicemax = find(dim2 <= maxmeandim,1,'last');

    %%%%% Extraction des données %%%%%
    extracteddata = data(1:size(data,1),indicemin:indicemax,1:size(data,3));

elseif numdimtomean == 3;

    %%%%% Recherche des indices %%%%%
    indicemin = find(dim3 >= minmeandim,1,'first');
    indicemax = find(dim3 <= maxmeandim,1,'last');

    %%%%% Extraction des données %%%%%
    extracteddata = data(1:size(data,1),1:size(data,2),indicemin:indicemax);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Moyennage sur la dimension spatiale sélectionnée
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if numdimtomean == 2;
    hovdata = squeeze(mean(extracteddata,2));
elseif numdimtomean == 3;
    hovdata = squeeze(mean(extracteddata,3));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dessin du diagramme de Hovmöller
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% Ouverture d'une figure
h=figure;

%%%%% Calculs pour l'affichage de la coordonnée temporelle %%%%%
% Nombre de 'tick' temporel maximum.
nbtickmax = 10;

% Nombre de pas de temps.
nbtimestep = size(data,1);

% On cherche le pas d'affichage des pas de temps pour ne pas dépasser la
% limite autorisée (on prend l'arrondi supérieur avec la fonction 'ceil').
tickstep = ceil(nbtimestep / nbtickmax);

% On garde les numéros dans temps à afficher.
numtime = 1 : tickstep : nbtimestep;

% On enregistre les futures 'ticklabel' sous forme de caractères.
time = datestr(dim1(numtime),'mmm dd');

if numdimtomean == 2;

    %%%%% Image du diagramme %%%%%
    % On veut un diagramme latitudes vs temps, il faut donc transposer la
    % matrice de données.
    imagesc(dim1,dim3,hovdata');
    set(gca,'ydir','normal');

    %%%%% Axes du diagramme %%%%%
    set(gca,'xtick',dim1(numtime));
    set(gca,'xticklabel',time);
    ylabel('Latitudes');

    %%%%% Titre du diagramme %%%%%
    title(['Diagramme de hovmöller des ' nomvar ' sur ' num2str(extmeandim) ' degres autour de ' num2str(centralvalue) ' en longitudes']);

elseif numdimtomean == 3;

    %%%%% Image du diagramme %%%%%
    % On veut un diagramme temps vs longitudes.
    imagesc(dim2,dim1,hovdata);
    set(gca,'ydir','normal');

    %%%%% Axes du diagramme %%%%%
    xlabel('Longitudes');
    set(gca,'ytick',dim1(numtime));
    set(gca,'yticklabel',time);

    %%%%% Titre du diagramme %%%%%
    title(['Diagramme de hovmöller des ' nomvar ' sur ' num2str(extmeandim) ' degres autour de ' num2str(centralvalue) ' en latitudes']);
end

%%%%% Ajout d'une barre de couleur %%%%%
colorbar

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

%!demo
%! % school example longitudes
%! clear all
%! close all
%! data(1,:,:) = reshape(1:100,10,10);
%! for i = 2:50;
%!     data(i,:,:) = data(i-1,:,:)+2;
%! end
%! dim1 = datenum(2011,07,27) : datenum(2011,07,27)+49;
%! dim2 = -4.5:4.5;
%! dim3 = 0.5:9.5;
%! numdimtomean = 2;
%! centralvalue = 0;
%! extmeandim = 2;
%! nomvar = 'valeurs tests pour la demonstration';
%! [hovdata] = hovmullerlonorlat(data, dim1, dim2, dim3, numdimtomean, centralvalue, extmeandim, nomvar);

%!demo
%! % school example latitude
%! clear all
%! close all
%! data(1,:,:) = reshape(1:100,10,10);
%! for i = 2:50;
%!     data(i,:,:) = data(i-1,:,:)+2;
%! end
%! dim1 = datenum(2011,07,27) : datenum(2011,07,27)+49;
%! dim2 = -4.5:4.5;
%! dim3 = 0.5:9.5;
%! numdimtomean = 3;
%! centralvalue = 0;
%! extmeandim = 3;
%! nomvar = 'valeurs tests pour la demonstration';
%! [hovdata] = hovmullerlonorlat(data, dim1, dim2, dim3, numdimtomean, centralvalue, extmeandim, nomvar);
