function [mat_intensity_precip] = int2precip(mat_intensity, type, Longitudes, Latitudes)

%int2precip est une fonction permettant de passer de données d'intensités
%en données de précipitations en mm sur le domaine 0-20W 0-20N.

%
%+
%
% ============
% int2precip.m
% ============
%
% .. function:: int2precip(mat_intensity, type, Longitudes, Latitudes)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * mat_intensity : Matrice contenant les intensités sur la zone 0-20W et
%     0-20N, de dimension longitudes, latitudes si elle est 2D ou temps,
%     longitudes, latitudes si elle est 3D.
%   * type : Type de formules a utiliser, 'global' pour utiliser la même
%     sur l'ensemble du domaine ou 'régional' pour utiliser les formules
%     sur les 4 sous-régions étudiées.
%   * Longitudes (si type = 'regional') : Variable de longitudes des
%     données.
%   * Latitudes (si type = 'regional') : Variable de latitudes des
%     données.
%
% - Données de sorties :
%
%   * mat_intensity_precip : Matrice de données de précipitations dont les
%     dimensions sont celle de la matrice d'intensités initiales.
%
% Cette fonction permet de passer de données d'intensités en données de 
% précipitations en mm sur le domaine 0-20W 0-20N.
%
% EXAMPLES
% ========
%
% Voir la 'demo'.
%
% SEE ALSO
% ========
%
% :ref:`data_msg`
%
% :ref:`varamma_startup.m`
%
% :func:`MSGbuildfullfilename`
% :func:`MSGread`
% :func:`intensityMSG_day`
% :func:`intensityMSG_concat`
%
% :func:`nonprecipperiods`
% :func:`missingdaytonan`
%
% TODO
% ====
%
% use :func:`extractedmatrixdata`
% use :func:`spatialresolution`
%
% more explanation on demo
%
% demo with simulation
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/int2precip.m $
%
% - jaclod 2011-07-29
%
%   * Creation.
%
%-
usage='[mat_intensity_precip] = int2precip(mat_intensity, type, Longitudes, Latitudes)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% Application des formules pour passer des intensités en précipitations
% (formules basées sur la validation croisée entre les intensités obtenues 
% par l'analyse des données MSG et des données EPSAT sur une zone de -20 à
% 0 en longitudes et de 0 à 20 en latitudes. Dans chaque cas, la formule
% moyenne est obtenue à partir des proches résultats pour un domaine donnée
% des régressions linéaires, cf. MSGanalysiscrossvalidation).
switch type
    
    case 'global'
        % Dans ce cas, on souhaite appliquer la même formule à l'ensemble
        % du domaine (0-20W et 0-20N). Nous ne gardons que le facteur
        % d'échelle, pour garder les précipitations nulles, sur la formule 
        % moyenne suivante :
        % précipitations (mm) = 0.0327 * intensités + 1.02.
        mat_intensity_precip = 0.0327 * mat_intensity;
        
    case 'regional'
        % Dans ce cas, on souhaite appliquer la formule sur différents
        % domaines étudiés (Il s'agit des régions 10-20W 10-20N, 10-20W
        % 0-10N, 0-20W 5-10N et enfin 0-20W 0-5N).
        
        % Initialisation de la matrice de données de précipitations.
        mat_intensity_precip = zeros(size(mat_intensity));
        
        % On applique le facteur d'échelle de la formule moyenne 
        % correspondante à chaque région.
        % Region 10-20W 10-20N :
        
        % Récupération des indices des bornes de la région.
        indlonmin = find(Longitudes >= -20,1,'first');
        indlonmax = find(Longitudes <= -10,1,'last');
        indlatmin = find(Latitudes >= 10,1,'first');
        indlatmax = find(Latitudes <= 20,1,'last');
        
        % Application, pour cette région, du facteur de la formule
        % moyenne :
        % Précipitations (mm) = 0.028035 * intensités + 0.66.
        if size(size(mat_intensity),2)==3;
            
            % Matrice 3D avec une première dimension temporelle.
            mat_intensity_precip(:,indlonmin:indlonmax,indlatmin:indlatmax) = 0.028035 * mat_intensity(:,indlonmin:indlonmax,indlatmin:indlatmax);
            
        elseif size(size(mat_intensity),2)==2;
            
            % Matrice 2D.
            mat_intensity_precip(indlonmin:indlonmax,indlatmin:indlatmax) = 0.028035 * mat_intensity(indlonmin:indlonmax,indlatmin:indlatmax);
            
        else
            error('Les données d''intensités doivent être dans une matrice 2D ou 3D avec une première dimension temporelle.')
        
        end
        
        % Région 0-10W 10-20N :
        
        % Récupération des indices des bornes de la région.
        indlonmin = find(Longitudes >= -10,1,'first');
        indlonmax = find(Longitudes <= 0,1,'last');
        indlatmin = find(Latitudes >= 10,1,'first');
        indlatmax = find(Latitudes <= 20,1,'last');
        
        % Application, pour cette région, du facteur de la formule
        % moyenne :
        % Précipitations (mm) = 0.028126 * intensités + 0.55.
        % (On remarque que cette formule, ne change pas beaucoup par
        % rapport à la précédente qui est a la même latitude. La constante
        % change probablement à cause des précipitations sous-estimées avec
        % notre analyse sur l'océan).
        if size(size(mat_intensity),2)==3;
            
            % Matrice 3D avec une première dimension temporelle.
            mat_intensity_precip(:,indlonmin:indlonmax,indlatmin:indlatmax) = 0.028126 * mat_intensity(:,indlonmin:indlonmax,indlatmin:indlatmax);
            
        elseif size(size(mat_intensity),2)==2;
            
            % Matrice 2D.
            mat_intensity_precip(indlonmin:indlonmax,indlatmin:indlatmax) = 0.028126 * mat_intensity(indlonmin:indlonmax,indlatmin:indlatmax);
            
        else
            % Erreur deja donnée.
        end
        
        % Region 0-20W 5-10N :
        
        % Récupération des indices des bornes de la région.
        indlonmin = find(Longitudes >= -20,1,'first');
        indlonmax = find(Longitudes <= 0,1,'last');
        indlatmin = find(Latitudes >= 5,1,'first');
        indlatmax = find(Latitudes <= 10,1,'last');
        
        % Application, pour cette région, du facteur de la formule
        % moyenne :
        % Précipitations (mm) = 0.0395 * intensités + 2.
        if size(size(mat_intensity),2)==3;
            
            % Matrice 3D avec une première dimension temporelle.
            mat_intensity_precip(:,indlonmin:indlonmax,indlatmin:indlatmax) = 0.0395 * mat_intensity(:,indlonmin:indlonmax,indlatmin:indlatmax);
            
        elseif size(size(mat_intensity),2)==2;
            
            % Matrice 2D.
            mat_intensity_precip(indlonmin:indlonmax,indlatmin:indlatmax) = 0.0395 * mat_intensity(indlonmin:indlonmax,indlatmin:indlatmax);
            
        else
            % Erreur déjà donnée.
        end
        
        % Région 0-20W 0-5N :
        
        % Récupération des indices des bornes de la région.
        indlonmin = find(Longitudes >= -20,1,'first');
        indlonmax = find(Longitudes <= 0,1,'last');
        indlatmin = find(Latitudes >= 0,1,'first');
        indlatmax = find(Latitudes <= 5,1,'last');
        
        % Application, pour cette région, du facteur de la formule
        % moyenne :
        % Précipitations (mm) = 0.055 * intensités + 0.56.
        if size(size(mat_intensity),2)==3;
            
            % Matrice 3D avec une première dimension temporelle.
            mat_intensity_precip(:,indlonmin:indlonmax,indlatmin:indlatmax) = 0.055 * mat_intensity(:,indlonmin:indlonmax,indlatmin:indlatmax);
            
        elseif size(size(mat_intensity),2)==2;
            
            % Matrice 2D.
            mat_intensity_precip(indlonmin:indlonmax,indlatmin:indlatmax) = 0.055 * mat_intensity(indlonmin:indlonmax,indlatmin:indlatmax);
            
        else
            % Erreur déjà donnée.
        end
        
    otherwise
        error('Le type d''application de formules ne peut qu''être ''global'' ou ''regional''.')
    
end

end

%!demo
%! [ij_intensity_concat, time_concat, Longitudes, Latitudes] = intensityMSG_concat('extracted', 0, 0, 0, 0, 0, 0, 2006, 08, 01, 08, 07);
%! [ij_intensity_concat_precip] = int2precip(ij_intensity_concat, 'global');
