function [ij_intensity_concat, time_concat, Longitudes, Latitudes] = intensityMSG_concat(ncfilesset, lonmin, lonmax, latmin, latmax, longresol, latresol, Annee, Mois_start, Jour_start, Mois_stop, Jour_stop)

%INTENSITYMSG_CONCAT permet d'analyser les données de température de
%brillance par MSG et de concaténer les intensités résultantes en sortie.
%L'analyse s'effectue sur une seule saison, sur tous les fichiers ou entre
%deux dates prédéfinies.

%
%+
%
% ======================
% intensityMSG_concat.m
% ======================
%
% .. function:: intensityMSG_concat(ncfilesset, lonmin, lonmax, latmin, latmax, longresol, latresol, Annee, Mois_start, Jour_start, Mois_stop, Jour_stop)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * ncfilesset (en caractères) : Type de fichier netCDF que l'on cherche,
%     'normal' ou 'extracted'.
%   * lonmin : Borne inférieure des longitudes du domaine étudié.
%   * lonmax : Borne supérieure des longitudes du domaine étudié.
%   * latmin : Borne inférieure des latitudes du domaine étudié.
%   * latmax : Borne supérieure des latitudes du domaine étudié.
%   * longresol : Nouvelle résolution souhaitée (nombre de points) pour les
%     longitudes (0 pour garder la résolution d'origine).
%   * latresol : Nouvelle résolution souhaitée (nombre de points) pour les
%     latitudes (0 pour garder la résolution d'origine).
%   * Annee : Année de la saison d'étude.
%   * Mois_start (Facultatif) : Mois du premier jour d'étude pour la
%     sélection des fichiers.
%   * Jour_start (Facultatif) : Jour du premier jour d'étude pour la
%     sélection des fichiers.
%   * Mois_stop (Facultatif) : Mois du dernier jour d'étude pour la
%     sélection des fichiers.
%   * Jour_stop (Facultatif) : Jour du dernier jour d'étude pour la
%     sélection des fichiers.
%
% - Données de sorties :
%
%   * ij_intensity_concat : Matrice de données (temps, longitudes,
%     latitudes) contenant les intensités concaténées des différents jours
%     d'étude.
%   * time_concat : Variable de temps, en jour julien, pour identifier les
%     jours étudiés.
%   * Longitudes : Variable de longitudes dans le domaine sélectionné.
%   * Latitudes : Variable de latitudes dans le domaine sélectionné.
%
% Cette fonction permet d'analyser les données de température de brillance
% par MSG et de concaténer les intensités résultantes en sortie. L'analyse
% s'effectue sur une seule saison, sur tous les fichiers ou entre deux
% dates prédéfinies.
%
% EXAMPLES
% ========
%
% Voir la 'demo'.
%
% SEE ALSO
% ========
%
% :ref:`data_msg`
%
% :ref:`varamma_startup.m`
%
% :func:`MSGbuildfullfilename`
% :func:`extractedmatrixdata`
% :func:`spatialresolution`
% :func:`MSGread`
% :func:`intensityMSG_day`
%
% :func:`int2precip`
% :func:`nonprecipperiods`
% :func:`MSGanalysiscrossvalidation`
% :func:`missingdaytonan`
%
% :func:`stagejasonjuillet`
%
% TODO
% ====
%
% improve optional arguments
%
% coding rule and description of demo
%
% demo normal
%
% demo simul
%
% use :func:`simul_msg`
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/intensityMSG_concat.m $
%
% - fplod 20110805T093216Z cratos (Linux)
%
%   * remove varamma_startup call

% - fplod 20110804T150823Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * octave and matlab 7.4 (zeus) do not like [argout1,~, argout3]=myfunc() so
%     replace ~ by temporary variable
%
% - jaclod 2011-07-27
%
%   * Revision de la documentation et des commentaires.
%
% - jaclod 2011-07-22
%
%   * Compatibilité avec des fichiers 'extracted'.
%
% - jaclod 2011-07-20
%
%   * Passage sur cratos, fonctionne!
%   * Modifications de la documentation et des commentaires.
%
% - jaclod 2011-07-19
%
%   * Ajout de boucles pour la lecture des fichiers avec ou sans sélection.
%
% - jaclod 2011-07-13
%
%   * Creation.
%
%-

usage='[ij_intensity_concat, time_concat, Longitudes, Latitudes] = intensityMSG_concat(ncfilesset, lonmin, lonmax, latmin, latmax, longresol, latresol, Annee, Mois_start, Jour_start, Mois_stop, Jour_stop)';
%
if nargin < nargin(mfilename) - 4
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
%%%%%%%%%%%%%%%%%
% Initialisation
%%%%%%%%%%%%%%%%%

% Appel de la variable global 'PROJECT_ID'.
global PROJECT_ID

% Sauvegarde de la variable 'Annee' sous forme de caractères.
annee = num2str(Annee);

%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Boucles sur les fichiers
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Les boucles se réalisent de différentes façons selon la sélection ou non
% de jours.

% Si le nombre d'arguments est supérieur à 8 c'est qu'une sélection est
% demandée. Sinon, on analyse tous les fichiers présent dans le répertoire
% MSG.
if nargin > 8;

    % Création d'un vecteur comportant le nombre maximum de jours que l'on
    % a dans chaque mois.
    maxjourmois = [31 29 31 30 31 30 31 31 30 31 30 31];

    %%%%%% Boucle sur les mois %%%%%%
    for Mois = Mois_start : Mois_stop;

        % Récupération de la variable 'Mois' sous forme de caractères.
        if Mois < 10;
            mois = ['0' num2str(Mois)];
        else
            mois = num2str(Mois);
        end

        % S'il s'agit du premier mois, le jour de départ sera celui entre
        % en argument, sinon ce sera le premier du mois.
        if Mois == Mois_start;
            Jour_ini = Jour_start;
        else
            Jour_ini = 1;
        end

        % S'il s'agit du dernier mois, le jour de fin sera celui entre
        % en argument, sinon ce sera le dernier jour possible du mois.
        if Mois == Mois_stop;
            Jour_fin = Jour_stop;
        else
            Jour_fin = maxjourmois(Mois);
        end

        %%%%%% Boucle sur les jours %%%%%%
        for Jour = Jour_ini : Jour_fin;

            % Récupération de la variable 'Jour' sous forme de caractères.
            if Jour < 10;
                jour = ['0' num2str(Jour)];
            else
                jour = num2str(Jour);
            end

            % Création du chemin d'accès complet au fichier correspondant.
            [fullfilename] = MSGbuildfullfilename(ncfilesset, annee, mois, jour);

            %%%%%% Analyse du fichier %%%%%%

            % Vérification que le fichier du jour existe bien. Si ce n'est
            % pas le cas un message d'avertissement est donné. Sinon, on
            % peut faire l'analyse.
            % Cette vérification diffère selon le type de fichier en raison
            % d'un changement de répertoire et de nom.

            switch ncfilesset

                case 'normal'
                    repertoire = [PROJECT_ID 'MSG/' annee '/' mois '/'];

                case 'extracted'
                    repertoire = [PROJECT_ID 'MSG/extracted/' annee '/' mois '/'];

                otherwise
                    error('La variable ncfilesset doit être ''normal'' ou ''extracted''.');

            end

            path(path,repertoire);

            switch ncfilesset

                case 'normal'
                    if exist(['msg-tb108_' annee '-' mois '-' jour '_15min.nc'],'file') == 2;

                        % Message confirmant l'analyse du fichier.
                        disp(['iii : Analyse du fichier msg-tb108_' annee '-' mois '-' jour '_15min.nc.']);

                        % Lecture du fichier netCDF, récupération des
                        % données et des variables.
                        [TbMSG, Temps, Longitudes, Latitudes] = MSGread(ncfilesset, fullfilename);

                        % Extraction des données.
                        [TbMSG, Temps, Longitudes, Latitudes] = extractedmatrixdata(TbMSG, Temps, Longitudes, Latitudes, Temps(1), Temps(size(Temps,1)), lonmin, lonmax, latmin, latmax);

                        %%%%%% Changement de résolution %%%%%%
                        % Si longresol et/ou latresol ont pour valeur 0, alors on
                        % récupère la valeur de la résolution initiale pour ne pas
                        % faire de changement.
                        if longresol == 0;
                            longresol = size(TbMSG,2);
                        end

                        if latresol == 0;
                            latresol = size(TbMSG,3);
                        end

                        % Si la résolution demandée est différente de celle
                        % initiale, alors on interpole les données avec la
                        % fonction spatialresolution.
                        if longresol ~= size(TbMSG,2) || latresol ~= size(TbMSG,3);
                            [Longitudes, Latitudes, TbMSG] = spatialresolution(Longitudes, Latitudes, TbMSG, longresol, latresol);
                        end

                        %%%%%% Calcul des intensités %%%%%%
                        [ij_intensity, unused_Tps_SeuilInf, unused_Tb_SeuilSup, unused_Tb_LimNoyau] = intensityMSG_day(Temps, TbMSG, 3, 235, 190);
                        clear unused_Tps_SeuilInf
                        clear unused_Tb_SeuilSup
                        clear unused_Tb_LimNoyau

                        %%%%%% Concaténation %%%%%%
                        % Sauvegarde des intensités (la dimension 3 est ici
                        % le temps).
                        if Mois == Mois_start && Jour == Jour_start;
                            ij_intensity_concat = ij_intensity;
                        else
                            ij_intensity_concat = cat(3,ij_intensity_concat,ij_intensity);
                        end

                        % Sauvegarde de l'information sur la date. On la
                        % sauve en jour julien, il faut donc convertir par
                        % rapport au temps fourni, en heure depuis le 1er
                        % janvier 1960.
                        Tempsjourjulien = Temps(1) / 24 + datenum(1960,01,01);
                        if Mois == Mois_start && Jour == Jour_start;
                            time_concat = Tempsjourjulien;
                        else
                            time_concat = cat(2,time_concat,Tempsjourjulien);
                        end

                    else
                        % Message d'avertissement en cas de non présence du
                        % fichier.
                        disp(['www : le fichier msg-tb108_' annee '-' mois '-' jour '_15min.nc n''est pas present.']);
                    end

                case 'extracted'
                    if exist(['extracted-msg-tb108_' annee '-' mois '-' jour '_15min.nc'],'file') == 2;

                        % Message confirmant l'analyse du fichier.
                        disp(['iii : Analyse du fichier extracted-msg-tb108_' annee '-' mois '-' jour '_15min.nc.']);

                        % Lecture du fichier netCDF, récupération des
                        % données et des variables.
                        [TbMSG, Temps, Longitudes, Latitudes] = MSGread(ncfilesset, fullfilename);

                        % Pas d'extraction des données.

                        %%%%%% Changement de résolution %%%%%%
                        % Si longresol et/ou latresol ont pour valeur 0,
                        % alors on récupère la valeur de la résolution
                        % initiale pour ne pas faire de changement.
                        if longresol == 0;
                            longresol = size(TbMSG,2);
                        end

                        if latresol == 0;
                            latresol = size(TbMSG,3);
                        end

                        % Si la résolution demandée est différente de celle
                        % initiale, alors on interpole les données avec la
                        % fonction spatialresolution.
                        if longresol ~= size(TbMSG,2) || latresol ~= size(TbMSG,3);
                            [Longitudes, Latitudes, TbMSG] = spatialresolution(Longitudes, Latitudes, TbMSG, longresol, latresol);
                        end

                        %%%%%% Calcul des intensités %%%%%%
                        [ij_intensity, unused_Tps_SeuilInf, unused_Tb_SeuilSup, unused_Tb_LimNoyau] = intensityMSG_day(Temps, TbMSG, 3, 235, 190);
                        clear unused_Tps_SeuilInf
                        clear unused_Tb_SeuilSup
                        clear unused_Tb_LimNoyau

                        %%%%%% Concaténation %%%%%%
                        % Sauvegarde des intensités (la dimension 3 est ici
                        % le temps).
                        if Mois == Mois_start && Jour == Jour_start;
                            ij_intensity_concat = ij_intensity;
                        else
                            ij_intensity_concat = cat(3,ij_intensity_concat,ij_intensity);
                        end

                        % Sauvegarde de l'information sur la date. On la
                        % sauve en jour julien, il faut donc convertir par
                        % rapport au temps fourni, en heure depuis le 1er
                        % janvier 1960.
                        Tempsjourjulien = Temps(1) / 24 + datenum(1960,01,01);
                        if Mois == Mois_start && Jour == Jour_start;
                            time_concat = Tempsjourjulien;
                        else
                            time_concat = cat(2,time_concat,Tempsjourjulien);
                        end

                    else
                        % Message d'avertissement en cas de non présence du
                        % fichier.
                        disp(['www : le fichier extracted-msg-tb108_' annee '-' mois '-' jour '_15min.nc n''est pas present.']);
                    end

                otherwise
                    % Cas impossible. Erreur déjà donnée.
            end

        end

    end

else

    %%%%% Recherche des mois de données %%%%%
    % Répertoire correspondant aux fichiers MSG de l'année en question.
    % différents selon le type de fichier.
    switch ncfilesset

        case 'normal'
            repertoireMSGsaison = [PROJECT_ID 'MSG/' annee];

        case 'extracted'
            repertoireMSGsaison = [PROJECT_ID 'MSG/extracted/' annee];

        otherwise
    end

    path(path,repertoireMSGsaison);

    % On cherche les dossier dans ce dernier répertoire correspondant aux
    % mois dont nous disposons des données.
    listdir = dir(repertoireMSGsaison);

    % Nombre de dossier. Attention les deux premiers résultats sont '.' et
    % '..' et ne compte donc pas comme dossier de fichiers.
    nbdir = numel(listdir)-2;

    %%%%% Boucle sur les mois %%%%%
    for numdir = 1 : nbdir;

        % Récupération du numéro du mois sous forme de caractères.
        mois = listdir(numdir+2).name;

        % Répertoire du mois correspondant.
        repertoireMSGmois = [repertoireMSGsaison '/' mois];

        path(path,repertoireMSGmois);

        %%%%% Recherche des fichiers netCDF %%%%%
        ext = '*.nc';
        chemin = fullfile(repertoireMSGmois,ext);
        listncfiles = dir(chemin);

        nbfiles = numel(listncfiles);

        %%%%% Boucle sur les fichiers, les jours %%%%%
        for numfiles = 1 : nbfiles;

            % Récupération du nom du fichier et de son chemin d'accès
            % complet.
            fullfilename = [repertoireMSGmois '/' listncfiles(numfiles).name];

            % Message confirmant l'analyse du fichier.
            disp(['iii : Analyse du fichier ' listncfiles(numfiles).name '.']);

            % Lecture du fichier netCDF, récupération des données et des
            % variables.
            [TbMSG, Temps, Longitudes, Latitudes] = MSGread(ncfilesset, fullfilename);

            switch ncfilesset

                case 'normal'
                    % Extraction des données.
                    [TbMSG, Temps, Longitudes, Latitudes] = extractedmatrixdata(TbMSG, Temps, Longitudes, Latitudes, Temps(1), Temps(size(Temps,1)), lonmin, lonmax, latmin, latmax);

                case 'extracted'
                    % Pas d'extraction.

                otherwise
            end

            %%%%%% Changement de résolution %%%%%%
            % Si longresol et/ou latresol ont pour valeur 0, alors on
            % récupère la valeur de la résolution initiale pour ne pas
            % faire de changement.
            if longresol == 0;
                longresol = size(TbMSG,2);
            end

            if latresol == 0;
                latresol = size(TbMSG,3);
            end

            % Si la résolution demandée est différente de celle
            % initiale, alors on interpole les données avec la fonction
            % spatialresolution.
            if longresol ~= size(TbMSG,2) || latresol ~= size(TbMSG,3);
                [Longitudes, Latitudes, TbMSG] = spatialresolution(Longitudes, Latitudes, TbMSG, longresol, latresol);
            end

            %%%%%% Calcul des intensités %%%%%%
            [ij_intensity, unused_Tps_SeuilInf, unused_Tb_SeuilSup, unused_Tb_LimNoyau] = intensityMSG_day(Temps, TbMSG, 3, 235, 190);
            clear unused_Tps_SeuilInf
            clear unused_Tb_SeuilSup
            clear unused_Tb_LimNoyau

            %%%%%% Concaténation %%%%%%

            % Sauvegarde des intensités (la dimension 3 est ici le temps).
            if numdir==1 && numfiles==1;
                ij_intensity_concat = ij_intensity;
            else
                ij_intensity_concat = cat(3,ij_intensity_concat,ij_intensity);
            end

            % Sauvegarde de l'information sur la date. On la sauve en jour
            % julien, il faut donc convertir par rapport au temps fourni,
            % en heure depuis le 1er janvier 1960.
            Tempsjourjulien = Temps(1) / 24 + datenum(1960,01,01);
            if numdir==1 && numfiles==1;
                time_concat = Tempsjourjulien;
            else
                time_concat = cat(2,time_concat,Tempsjourjulien);
            end

        end

    end

end

% On passe le temps en première dimension pour plus de concordance avec les
% autres variables.
ij_intensity_concat = permute(ij_intensity_concat,[3 1 2]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%

end

%!demo
%! clear all
%! close all
%! varamma_startup;
%! more off
%! [ij_intensity_concat, time_concat, Longitudes, Latitudes] = intensityMSG_concat('normal',-20, 0, 0, 20, 200, 200, 2006, 07, 26, 08, 09);
