#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

====================
interpolswathb_py.py
====================

SYNOPSIS
========

.. code-block:: bash

   interpolswathb_py.py --testname testname

DESCRIPTION
===========

.. warning::

   this CLI exists only to consolidate the function

.. option:: --testname

méthode d'affectation des tb amsu sur grille de résolution
fine pour éviter les pbs de trous entre les pixels
version AMSUB
tb du canal - dimension fovb (90 points pour amsua)
latlu
lonlu mêmes dimensions que tb

principe: on lit la fauchée, puis on calcule fova, distances de puis
le centre de chaque pixel (en principe symétrique)
on crée grid, de dimension ngrid pixels, depuis le centre
jusqu'au bout du dernier pixel de chaque coté, avec une
résolution fixée au diamètre du pixel central ou la moitié (resol
vaut 1 ou 2)

on interpole les longitudes et latitudes en correspondance
on affecte à chaque pixel nouveau la Tb originelle la plus proche
en sortie on a une fauchée synthétique amsua régulière

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest interpolswathb_py.py

Unrealistic example with testname '''test1''':

.. code-block:: bash

   testname='test1'
   interpolswathb_py.py --testname ${testname}
   tlogd.sh interpolswathb_py_main

Same in debug mode

.. code-block:: bash

   python -m pdb ${PROJECT}/src/interpolswathb_py.py --testname ${testname}

++ Realistic example

SEE ALSO
========

:func:`pixelsize_py` to produce  ***swathb***

TODO
====

missing values

plot lat lon test1

science validation

error handling

use basemap for earth radius
cf. http://matplotlib.org/basemap/api/basemap_api.html?highlight=constant
variable rsphere

EVOLUTIONS
==========

$Id$

$URL$

- pinsard 20141015T085620Z climtest0.ipsl.polytechnique.fr (Linux)

  * fix grid construction
  * fix interpolation order of args !!
  * minimize logging

- pinsard 20141006T103421Z climtest0.ipsl.polytechnique.fr (Linux)

  * fix initial values of tb in test1 and doctest

- fplod 20140919T160005Z callisto.locean-ipsl.upmc.fr (Linux)

  * may be fix bug on inter argument order (<> IDL order !)

- fplod 20140812T084135Z callisto.locean-ipsl.upmc.fr (Linux)

   creation from interpolswathb.pro

"""

__all__ = ["interpolswathb_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

import numpy


from find_nearest import find_nearest

try:
    from mpl_toolkits.basemap import Basemap
except ImportError:
    print('Sorry, python module Basemap is needed')
    exit(-1)

def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """interpole swath AMSU-B"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--testname',
                      help='name of dataset values for test',
                      dest='testname',
                      metavar='<testname>',
                      action='store')

    return parser


def interpolswathb_py(tb, latlu, lonlu, resol, swathb):

    """

    >>> from pixelsize_py import pixelsize_py
    >>> (swatha, swathb, tracka, trackb) = pixelsize_py()
    >>> lonmin = -22.
    >>> lonmax = 22.
    >>> latmin = -12.
    >>> latmax = 42.
    >>> resol = 1.
    >>> lonlu = numpy.linspace(lonmin, lonmax, swathb.shape[0])
    >>> latlu = numpy.linspace(latmin, latmax, swathb.shape[0])
    >>> tb = numpy.linspace(150.,300., swathb.shape[0])
    >>> nbgrid, tbint, latgrid, longrid = interpolswathb_py(tb, latlu, lonlu, resol, swathb)

    """

    result = None

    logger = logging.getLogger()

    na = len(swathb)
    nnadir = na / 2
    pixa = swathb[nnadir:]

    fova = numpy.zeros(na, dtype='Float32')
    fova[nnadir] = pixa[0] / 2.
    fova[nnadir - 1] = -1 * (pixa[0] / 2.)
    rterre = 6400.
    deg = numpy.pi / 180.
    lat = latlu * deg
    lon = lonlu * deg
    mnadir = nnadir - 1
    ifov = numpy.arange(0, na, dtype='int8')
    for i in range(nnadir + 1, na):
        dist = rterre * numpy.arccos(numpy.sin(lat[i - 1]) * numpy.sin(lat[i]) + numpy.cos(lat[i - 1]) * numpy.cos(lat[i]) * numpy.cos(lon[i] - lon[i - 1]))
        fova[i] = fova[i - 1] + dist
    for i in range(nnadir - 2, -1, -1):
        dist = rterre * numpy.arccos(numpy.sin(lat[i]) * numpy.sin(lat[i + 1]) + numpy.cos(lat[i]) * numpy.cos(lat[i + 1]) * numpy.cos(lon[i] - lon[i + 1]))
        # logger.info('i {0} dist {1}'.format(i, dist))
        fova[i] = fova[i + 1] - dist

    #ploterr, ifov, fova, swathb  # ++ migration plot

    distmax = (min(fova) * -1) + max(fova) + max(pixa)
    dmax = distmax / 2.
    delta = numpy.fix(pixa[0] / resol)
    ndemigrid = int(numpy.fix(dmax / delta) + 1)
    if (dmax / delta - numpy.fix(dmax / delta) <= 0.5):
        ndemigrid = ndemigrid - 1
    nbgrid = int(ndemigrid * 2)

    grid2 = numpy.arange(0, ndemigrid, dtype='int8') * delta + fova[nnadir]
    grid1 = numpy.zeros(ndemigrid)
    for i in range(0, ndemigrid):
        grid1[i] = -1 * grid2[ndemigrid - 1 - i]

    #alternative équivalente ++ on compare ?
    #for i = ndemigrid - 1, 0, -1 do grid1[i]=-1 * (ndemigrid - i - 1) * delta - fova[nnadir]

    grid = numpy.concatenate((grid1, grid2))

    tbint = numpy.zeros(nbgrid, dtype=tb.dtype)
    for i in range(0, nbgrid):
        ind = find_nearest(fova, grid[i])
        tbint[i] = tb[ind]


    # interpolation des positions
    try:
        latgrid = numpy.interp(grid, fova, latlu)
    except ValueError:
        logger.error('pb numpy.interp lat shape de latlu {0}'.format(latlu.shape))
        logger.error('pb numpy.interp lat shape de fova {0}'.format(fova.shape))
        logger.error('pb numpy.interp lat shape de grid {0}'.format(grid.shape))

    # pour la longitude, traitement spécifique des longitudes proches de -180
    if (numpy.mean(lonlu) >= -150. and (numpy.mean(lonlu) < 160)):
        longrid = numpy.interp(grid, fova, lonlu)
    else:
        ind = numpy.where(lonlu < 0.)[0]
        neg = len(indg)
        if neg == 0:
            longrid = numpy.interp(grid, fova, lonlu)
        else:
            # interpolation dans repere 0 - 360
            lonlu[ind] = 360. + lonlu[ind]
            longrid = numpy.interp(grid, fova, lonlu)
            # reconversion en -180 - 180
            indg = numpy.where(longrid > 180.)[0]
            negrid = len(indg)
            if negrid != 0:
                longrid[indg] = longrid[indg] - 360.

    return (nbgrid, tbint, latgrid, longrid)


def interpolswathb_plot():

    return

def interpolswathb_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['testname']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    testname = fromcli.testname

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('testname={0}'.format(testname))

    if testname == 'test1':
        from pixelsize_py import pixelsize_py
        (swatha, swathb, tracka, trackb) = pixelsize_py()
        lonmin = -22.
        lonmax = 22.
        latmin = -12.
        latmax = 42.
        resol = 1.
        lonlu = numpy.empty(shape=swathb.shape[0])
        lonlu[:] = [-1478839, -1473110, -1467737, -1462678, -1457895, -1453360,
        -1449048, -1444934, -1441000, -1437230, -1433609, -1430123,
        -1426762, -1423515, -1420372, -1417326, -1414368, -1411492,
        -1408693, -1405963, -1403297, -1400693, -1398143, -1395644,
        -1393193, -1390785, -1388418, -1386089, -1383793, -1381530,
        -1379295, -1377086, -1374901, -1372737, -1370593, -1368466,
        -1366355, -1364256, -1362168, -1360090, -1358019, -1355953,
        -1353892, -1351832, -1349772, -1347710, -1345645, -1343575,
        -1341497, -1339410, -1337313, -1335202, -1333077, -1330935,
        -1328773, -1326590, -1324384, -1322152, -1319892, -1317601,
        -1315276, -1312914, -1310513, -1308069, -1305579, -1303038,
        -1300444, -1297791, -1295075, -1292291, -1289434, -1286498,
        -1283476, -1280362, -1277148, -1273825, -1270383, -1266814,
        -1263104, -1259241, -1255210, -1250995, -1246577, -1241934,
        -1237042, -1231871, -1226388, -1220554, -1214320, -1207631]
        lonlu = lonlu / 1.e4
        latlu = numpy.empty(shape=swathb.shape[0])
        latlu[:] = [455530, 457182, 458696, 460094, 461388, 462591, 463716, 464769,
        465759, 466693, 467576, 468412, 469207, 469964, 470686, 471376,
        472037, 472670, 473280, 473866, 474431, 474978, 475505, 476016,
        476511, 476992, 477459, 477913, 478355, 478786, 479207, 479618,
        480020, 480414, 480799, 481176, 481548, 481912, 482270, 482623,
        482970, 483311, 483649, 483982, 484310, 484635, 484956, 485274,
        485589, 485901, 486211, 486518, 486822, 487126, 487427, 487726,
        488024, 488320, 488616, 488911, 489205, 489498, 489790, 490082,
        490374, 490666, 490957, 491248, 491540, 491831, 492122, 492413,
        492705, 492996, 493287, 493578, 493868, 494158, 494447, 494734,
        495019, 495301, 495579, 495852, 496120, 496378, 496626, 496861,
        497078, 497273]
        latlu = latlu / 1.e4
        tb = numpy.linspace(150., 300., swathb.shape[0])
        (nbgrid, tbint, latgrid, longrid) = interpolswathb_py(tb, latlu, lonlu, resol, swathb)
    else:
        logger.error('not valid testname', exc_info=True)

# Run main, if called from the command line
if __name__ == '__main__':
    interpolswathb_py_main()
