#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

=========================
mem_to_file_amsu_t2_py.py
=========================

SYNOPSIS
========

.. code-block:: bash

   mem_to_file_amsu_t2_py.py --testname testname --testfilename testfilename

DESCRIPTION
===========

.. warning::

   this CLI exists only to consolidate :func:`file_amsu_t2_to_mem_py`

   :func:`mem_to_file_amsu_t2_py`  has to be used inside a main program
   for realistic amsu_t2 values

Write info and data of **amsu_t2** structure to an amsu_t2 file

If **testfilename** is not empty, this filename will be written, otherwise
amsu_t2 terminology will be applied.

If **testname**, test1 set values will be written : no realistic silly values.

If **testname** is set to test2, :file:`${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat` will be read and write to **testfilename**.

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest mem_to_file_amsu_t2_py.py

Unrealistic example with dataset test1:

.. code-block:: bash

   testname=test1
   testfilename='./mem_to_file_amsu_t2_py_${testname}.svg'
   mem_to_file_amsu_t2_py.py --testname test1 --testfilename ${testfilename}
   tlogd.sh mem_to_file_amsu_t2_py_main
   more ${testfilename}

Realistic example with dataset test2 ie :file:`${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat`:

.. code-block:: bash

   testname=test2
   testfilename='./mem_to_file_amsu_t2_py_${testname}.svg'
   mem_to_file_amsu_t2_py.py --testname test2 --testfilename ${testfilename}
   tlogd.sh mem_to_file_amsu_t2_py_main
   more ${testfilename}

.. note::

   If :file:`${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat` has been written with IDL
   syntaxic differences will be detect by :commmand:`diff` even if no semantic differences occure.

   .. code-block:: bash

      diff ${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat ./b

The same in debug

.. code-block:: bash

   python -m pdb ${PROJECT}/src/mem_to_file_amsu_t2_py.py --testname ${testname} --testfilename ${testfilename}

SEE ALSO
========

:func:`extract_amsu_py` where this function should be called

:func:`file_amsu_t2_to_mem_py` reverse function (read)

:func:`plot_amsu_t2` for plot

use :func:`geolocation_to_string_py`

TODO
====

semantic difference thanks to
http://stackoverflow.com/questions/9253493/comparing-keys-and-values-in-2-dictionaries
or https://groups.google.com/forum/#!topic/comp.lang.python/HAZFTHcQ1pI

crier sur valeur manquante de tb : ajouter test

crier sur les dates louches : ajouter test

EVOLUTIONS
==========

$Id$

$URL$

- pinsard 20140827T143326Z loholt1.ipsl.polytechnique.fr (Linux)

  * fix terminology
  * suppress landseamask

- pinsard 20140814T094709Z loholt1.ipsl.polytechnique.fr (Linux)

  * clean examples; cite :func:`plot_amsu_t2`

- pinsard 20140807T102843Z loholt1.ipsl.polytechnique.fr (Linux)

  * add test2 with :file:`${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat`
  * avoid overwritting

- fplod 20140806T135929Z callisto.locean-ipsl.upmc.fr (Linux)

  * usage of dictonary

- fplod 20140805T121524Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from mem_to_file_amsu_t2.pro

"""

__all__ = ["mem_to_file_amsu_t2_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

from geolocation_to_string_py import geolocation_to_string_py

import numpy
import dateutil


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """write AMSU_T2 file"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--testfilename',
                      help='testfilename to be written',
                      dest='testfilename',
                      metavar='<testfilename>',
                      action='store')

    parser.add_option('--testname',
                      help='name of dataset values for test',
                      dest='testname',
                      metavar='<testname>',
                      action='store')

    return parser


def mem_to_file_amsu_t2_py(amsu_t2, testfilename):

    """
    >>> header1 = {'yyyymmdd': 99999999, 'numch': 'b3', 'nbpix': 0, 'resol': 0}
    >>> header2 = {'lonmin': 0., 'lonmax': 0., 'latmin': 0., 'latmax': 0.}
    >>> nrows = 2
    >>> data = {'desc': numpy.zeros(nrows, 'Int8'),
    ...         'hour': numpy.zeros(nrows, 'Float32'),
    ...         'fov': numpy.zeros(nrows, 'Int8'),
    ...         'lon': numpy.zeros(nrows, 'Float32'),
    ...         'lat': numpy.zeros(nrows, 'Float32'),
    ...         'tb': numpy.zeros(nrows, 'Float32')}
    >>> amsu_t2 = {'header1': header1, 'header2': header2, 'data': data}
    >>> print(amsu_t2)
    {'header2': {'latmax': 0.0, 'lonmin': 0.0, 'lonmax': 0.0, 'latmin': 0.0}, 'header1': {'resol': 0, 'numch': 'b3', 'nbpix': 0, 'yyyymmdd': 99999999}, 'data': {'fov': array([0, 0], dtype=int8), 'hour': array([ 0.,  0.], dtype=float32), 'lon': array([ 0.,  0.], dtype=float32), 'lat': array([ 0.,  0.], dtype=float32), 'desc': array([0, 0], dtype=int8), 'tb': array([ 0.,  0.], dtype=float32)}}
    >>> testfilename = './ginette'
    >>> result =  mem_to_file_amsu_t2_py(amsu_t2, testfilename)

    :returns: name of the output file if ok or -1 if error
    :rtype: integer
    """

    status = -1

    if testfilename is None:
        look = 'filename'
        scale = 1.
        geomin = geolocation_to_string_py(amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmin'], look, scale)
        geomax = geolocation_to_string_py(amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmax'], look, scale)
        project_id_env = os.environ['PROJECT_ID']
        yyyymmdd = dateutil.parser.parse('{0}'.format(amsu_t2['header1']['yyyymmdd']), ignoretz=True)
        yyyy = yyyymmdd.strftime('%Y')
        mm = yyyymmdd.strftime('%m')
        dd = yyyymmdd.strftime('%d')
        fullfilename = '{0}/AMSU/{1}/{2}/{3}_{1}{2}{4}_{5}_{6}.dat'.format(project_id_env, yyyy, mm, amsu_t2['header1']['numch'], dd, geomin, geomax)

    else:
        fullfilename = testfilename

    # avoid overwritting

    if os.path.exists(fullfilename):
        logging.error('{0} already exists'.format(fullfilename))
        return status

    # build the directory if needed
    dirout = os.path.dirname(fullfilename)
    if not os.path.exists(dirout):
        if dirout != '':
            os.makedirs(dirout)

    #
    logging.info('ouverture pour Ã©criture de {0}'.format(fullfilename))
    try:
        fid = open(fullfilename, "w")
    except IOError, err:
        logging.error("Error: {0} {1}\n".format(err.filename, err.strerror))
        return status
    #
    # write header lines
    header1_string = '{0:08d} {1:2s} {2:03d} {3:6.3f}\n'.format(amsu_t2['header1']['yyyymmdd'],
                                                                amsu_t2['header1']['numch'],
                                                                amsu_t2['header1']['nbpix'],
                                                                amsu_t2['header1']['resol'])
    fid.write(header1_string)
    header2_string = '{0:8.3f} {1:8.3f} {2:8.3f} {3:8.3f}\n'.format(amsu_t2['header2']['lonmin'],
                                                                    amsu_t2['header2']['lonmax'],
                                                                    amsu_t2['header2']['latmin'],
                                                                    amsu_t2['header2']['latmax'])
    fid.write(header2_string)
    #
    # get the number of data
    nrows = len(amsu_t2['data']['hour'])
    # write data lines
    for irow in range(0, nrows):
        #++ debugdata_string = '{0:02d} {1:8.4f} {2:03d} {3:8.3f} {4:8.3f} {5:01d} {6:8.3f}\n'.format(amsu_t2['data']['desc'][irow], amsu_t2['data']['hour'][irow], amsu_t2['data']['fov'][irow], amsu_t2['data']['lon'][irow], amsu_t2['data']['lat'][irow], amsu_t2['data']['tb'][irow])
        try:
            data_string = '{0:02d} {1:8.4f} {2:03d} {3:8.3f} {4:8.3f} {5:01d} {6:8.3f}\n'.format(int(amsu_t2['data']['desc'][irow]),
                                                                                                 float(amsu_t2['data']['hour'][irow]),
                                                                                                 int(amsu_t2['data']['fov'][irow]),
                                                                                                 float(amsu_t2['data']['lon'][irow]),
                                                                                                 float(amsu_t2['data']['lat'][irow]),
                                                                                                 float(amsu_t2['data']['tb'][irow]))
        except ValueError:
            print('pb writing data_string for row {0}'.format(irow))
            print("type amsu_t2['data']['desc'][irow] {0}".format(type(amsu_t2['data']['desc'][irow])))
            print("amsu_t2['data']['desc'][irow] {0:02d}".format(int(amsu_t2['data']['desc'][irow])))
            print("type amsu_t2['data']['hour'][irow] {0}".format(type(amsu_t2['data']['hour'][irow])))
            print("amsu_t2['data']['hour'][irow] {0:8.4f}".format(float(amsu_t2['data']['hour'][irow])))
            print("type amsu_t2['data']['fov'][irow] {0}".format(type(amsu_t2['data']['fov'][irow])))
            print("amsu_t2['data']['fov'][irow] {0:03d}".format(int(amsu_t2['data']['fov'][irow])))
            print("type amsu_t2['data']['lon'][irow] {0}".format(type(amsu_t2['data']['lon'][irow])))
            print("amsu_t2['data']['lon'][irow] {0:8.3f}".format(float(amsu_t2['data']['lon'][irow])))
            print("type amsu_t2['data']['lat'][irow] {0}".format(type(amsu_t2['data']['lat'][irow])))
            print("amsu_t2['data']['lat'][irow] {0:8.3f}".format(float(amsu_t2['data']['lat'][irow])))
            print("type amsu_t2['data']['tb'][irow] {0}".format(type(amsu_t2['data']['tb'][irow])))
            print("amsu_t2['data']['tb'][irow] {0:8.3f}".format(float(amsu_t2['data']['tb'][irow])))
            print("amsu_t2['data']['desc'][irow] {0} amsu_t2['data']['hour'][irow] {1} amsu_t2['data']['fov'][irow] {2} amsu_t2['data']['lon'][irow] {3} ams u_t2['data']['lat'][irow] {4} amsu_t2['data']['tb'][irow] {5}".format(amsu_t2['data']['desc'][irow], amsu_t2['data']['hour'][irow], amsu_t2['data']['fov'][irow], amsu_t2['data']['lon'][irow], amsu_t2['data']['lat'][irow], amsu_t2['data']['tb'][irow]))
            return None  # ++ break
        else:
            fid.write(data_string)
    #
    # close file
    fid.close()
    #
    result = fullfilename
    #
    return result


def mem_to_file_amsu_t2_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['testname']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    testfilename = fromcli.testfilename
    testname = fromcli.testname

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('testname={0}'.format(testname))
    logger.info('testfilename={0}'.format(testfilename))

    if testname == 'test1':
        header1 = {'yyyymmdd': 99999999, 'numch': 'b3', 'nbpix': 0, 'resol': 0}
        header2 = {'lonmin': 0., 'lonmax': 0., 'latmin': 0., 'latmax': 0.}
        nrows = 2
        data = {'desc': numpy.zeros(nrows, 'Int8'),
                'hour': numpy.zeros(nrows, 'Float32'),
                'fov': numpy.zeros(nrows, 'Int8'),
                'lon': numpy.zeros(nrows, 'Float32'),
                'lat': numpy.zeros(nrows, 'Float32'),
                'tb': numpy.zeros(nrows, 'Float32')}
        amsu_t2 = {'header1': header1, 'header2': header2, 'data': data}
    elif testname == 'test2':
        from file_amsu_t2_to_mem_py import file_amsu_t2_to_mem_py
        project_id_env = os.environ['PROJECT_ID']
        testfilename_read = '{0}/AMSU/2011/04/b3_20110403_022w12s_022e42n.dat'.format(project_id_env)
        result = file_amsu_t2_to_mem_py(None, None, None, None, None, None, testfilename_read)
        if result:
            amsu_t2 = result
    else:
        logger.error('no dataset to be written')

    result = mem_to_file_amsu_t2_py(amsu_t2, testfilename)

    if result is None:
        logger.error('pb mem_to_file_amsu_t2_py')

    if testname == 'test2':
        # read the file just written
        result_after_write = file_amsu_t2_to_mem_py(None, None, None, None, None, None, testfilename)

        #++result_after_write['header1']['yyyymmdd'] = 1999999 # debug car pas de difference détectée
        try:
            semantic_diff = (amsu_t2 == result_after_write)
        except ValueError:
            logging.warning('by pass ValueError: The truth value of an array with more than one element is ambiguous. Use a.any() or a.all()')
            logging.warning('need to be more precise in diff statement diff like = dict(set(d1.items()) - set(d2.items()))')
            semantic_diff = None   # ++ can not tell now !!
        if not semantic_diff:
            logging.warning('can not tell now if semantic diff between read and write !!')  # ++

# Run main, if called from the command line
if __name__ == '__main__':
    mem_to_file_amsu_t2_py_main()
