#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

===============
plot_amsu_t1.py
===============

SYNOPSIS
========

.. code-block:: bash

   plot_amsu_t1.py --testname testname --testfilename testfilename

DESCRIPTION
===========

.. warning::

   this CLI exists only to consolidate all amsu_t1 funtions

   :func:`read_amsua1c_py` has to be used inside a main program
   for realistic amsu_t1 values

Plot info and data of **amsu_t1** structure to an SVG++ file

If **testfilename** is not empty, this filename will be written, otherwise
amsu_t1_plot++ terminology will be applied.

If **testname** is set to simul1 set values will be written : no realistic silly values 1 scan, 1 fov.

If **testname** is set to simul2 set values will be written : no realistic silly values 1 scan, 3 fov.

If **testname** is set to simul3 set values will be written : no realistic silly values 1 scan, 3 fov and
one missing value.

If **testname** is set to test2, :file:`${PROJECT_ID}/AMSU/AMSUBN15/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C` will be plot and figure save in **testfilename**.

If **testname** is set to test2, :file:`${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C` will be plot and figure save in **testfilename**.

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest plot_amsu_t1.py

Unrealistic example with dataset simul1:

.. code-block:: bash

   testname=simul1
   testfilename='./plot_amsu_t1_${testname}.svg'
   plot_amsu_t1.py --testname ${testname} --testfilename ${testfilename}
   tlogd.sh plot_amsu_t1_main
   xdg-open ${testfilename}

Realistic example with dataset test2 ie :file:`${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C`

.. code-block:: bash

   testname=test2
   testfilename='./plot_amsu_t1_${testname}.svg'
   plot_amsu_t1.py --testname ${testname} --testfilename ${testfilename}
   tlogd.sh plot_amsu_t1_main
   xdg-open ${testfilename}

Realistic example with dataset test3 ie :file:`${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C`

.. code-block:: bash

   testname=test3
   testfilename='./plot_amsu_t1_${testname}.svg'
   plot_amsu_t1.py --testname ${testname} --testfilename ${testfilename}
   tlogd.sh plot_amsu_t1_main
   xdg-open ${testfilename}

The same in debug

.. code-block:: bash

   python -m pdb ${PROJECT}/src/plot_amsu_t1.py --testname ${testname} --testfilename ${testfilename}

SEE ALSO
========

use :func:`read_amsua1c_py` or :func:`simul_amsu_t1`

use :func:`geolocation_to_string_py`++

TODO
====

missing values
cf. http://ccplot.org/api/ example calipso hdf

voir aussi http://code.google.com/p/pyresample/ pour implmentation de kd-tree
sur le pb de swath (class SwathDefinition(CoordinateDefinition))

point out btemps missing values on lat and long plot

EVOLUTIONS
==========

$Id: plot_amsu_t1.py$

$URL$

- pinsard 20141003T142910Z loholt1.ipsl.polytechnique.fr (Linux)

  * draw lines on latmin, latmin, lonmin, lonmax if set
  * fix lat lon scale factor

- pinsard 20141002T153710Z climtest0.ipsl.polytechnique.fr (Linux)

  * nicer xticks (integer, number =f(nb of fov), independant of missing values)
  * start parametrization

- pinsard 20141001T160453Z climtest0.ipsl.polytechnique.fr (Linux)

  * progress on plots 2d thanks to http://nbviewer.ipython.org/github/jrjohansson/scientific-python-lectures/blob/master/Lecture-4-Matplotlib.ipynb

- pinsard 20140930 climtest0.ipsl.polytechnique.fr (Linux)

  * creation from plot_amsu_t2.py (but this one was not yet ok !)
    thanks to http://pythonbits.blogspot.fr/2011/09/i-have-problem-with-basemap-plotting.html,
    https://www.packtpub.com/books/content/plotting-geographical-data-using-basemap for plotting circle and text
    http://matplotlib.org/basemap/users/examples.html contour1.py (mais pas encore ok ++)

"""

__all__ = ["plot_amsu_t1"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

from geolocation_to_string_py import geolocation_to_string_py
from simul_amsu_t1 import simul_amsu_t1

import numpy
import dateutil
import matplotlib.pyplot as plt
from matplotlib.ticker import FormatStrFormatter

from read_amsua1c_py import read_amsua1c_py

try:
    from mpl_toolkits.basemap import Basemap
except ImportError:
    print('Sorry, python module Basemap is needed')
    exit(-1)


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """plot AMSU_T1 dataset"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--testfilename',
                      help='testfilename to be written',
                      dest='testfilename',
                      metavar='<testfilename>',
                      action='store')

    parser.add_option('--testname',
                      help='name of dataset values for test',
                      dest='testname',
                      metavar='<testname>',
                      action='store')

    return parser


def plot_amsu_t1_scan_fov(amsu_t1, testfilename,
                          channel_selection=None,
                          scan_selection=None,
                          fov_selection=None,
                          lonmin=None,
                          latmin=None,
                          lonmax=None,
                          latmax=None):

    """
    Unrealistic data 1 channel 1 scan 1 fov on Dakar

    >>> index_simulation = 1
    >>> amsu_t1 = simul_amsu_t1(index_simulation)
    >>> testfilename = './ginette.svg'
    >>> result =  plot_amsu_t1_scan_fov(amsu_t1, testfilename)

    # ++...        'lon': numpy.linspace(-25., +25., endpoint=True, num=nrows),
    # ++...        'lat': numpy.linspace(-12., +45., endpoint=True, num=nrows),

    >>> index_simulation = 3
    >>> amsu_t1 = simul_amsu_t1(index_simulation)
    >>> testfilename = './ginette.svg'
    >>> result =  plot_amsu_t1_scan_fov(amsu_t1, testfilename)

    >>> index_simulation = 3
    >>> amsu_t1 = simul_amsu_t1(index_simulation)
    >>> testfilename = './ginette.svg'
    >>> result =  plot_amsu_t1_scan_fov(amsu_t1, testfilename, lonmin=-25, latmin=-5, lonmax=25, latmax=20)

    """

    # scan selection
    if scan_selection is None:
        ind_scan = range(amsu_t1['dataset']['amb1c_btemps'].shape[0])  # ++ pas ok
    else:
        ind_scan = scan_selection

    # fov selection
    if fov_selection is None:
        ind_fov = range(amsu_t1['dataset']['amb1c_btemps'].shape[1])
    else:
        ind_fov = fov_selection
    #++ debug print('{0}'.format(ind_fov))

    if channel_selection is None:
        ich = 0
        #++ich = range(amsu_t1['dataset']['amb1c_btemps'].shape[2])
    else:
        ich = channel_selection

    if (amsu_t1['dataset']['amb1c_btemps'][ind_scan, ind_fov, ich] == -999999).all():
        msg = "only missing values in amsu_t1['dataset']['amb1c_btemps'][{0}, {1}, {2}]".format(ind_scan, ind_fov, ich)
        logging.info(msg)
        raise ValueError(msg)

    plt.rcParams.update({'font.size': 12, 'font.family': 'STIXGeneral', 'mathtext.fontset': 'stix'})
    # ++ ou matplotlib.rcParams.update({'font.size': 12, 'text.usetex': True})

    fig, axes = plt.subplots(nrows=1, ncols=3)

    axes[0].set_ylabel('btemps (K*1.e2)')

    # following line : thanks to
    # http://matplotlib.1069221.n5.nabble.com/color-missing-value-with-contourf-td511.html
    amb1c_btemps_ma = numpy.ma.masked_array(amsu_t1['dataset']['amb1c_btemps'], mask=(amsu_t1['dataset']['amb1c_btemps'] == -999999))

    #print('amb1c_btemps_ma {0}'.format(amb1c_btemps_ma))
    #print('amb1c_btemps_ma[ind_scan, ind_fov, ich] {0}'.format(amb1c_btemps_ma[ind_scan, ind_fov, ich]))

    axes[0].plot(ind_fov, amb1c_btemps_ma[ind_scan, ind_fov, ich], linestyle='none', marker='+', markersize=16, markerfacecolor="red")  # ++ scale factor

    axes[1].set_ylabel('lat (deg*100)')
    # pas la bonne formule amb1c_latlon_ma_notvm = numpy.ma.masked_array(amsu_t1['dataset']['amb1c_latlon'][:,:,0], mask=(amsu_t1['dataset']['amb1c_btemps'] == -999999))
    # toutes les lat
    #++ ok mais pas trs utile
    axes[1].plot(ind_fov, amsu_t1['dataset']['amb1c_latlon'][ind_scan, ind_fov, 0], linestyle='none', marker='+', markersize=16, markerfacecolor="red")  # ++ scale factor

    for limit in (latmin, latmax):
        if limit is None:
            pass
        else:
            axes[1].plot(ind_fov, numpy.ones(len(ind_fov)) * limit * 1.e4, color='g')

    for limit in (lonmin, lonmax):
        if limit is None:
            pass
        else:
            axes[2].plot(ind_fov, numpy.ones(len(ind_fov)) * limit * 1.e4, color='g')
    #++ amb1c_latlon_ma_notvm pas ok axes[1].plot(ind_fov, amb1c_latlon_ma_notvm, linestyle='none', marker='+', markersize=16, markerfacecolor="red")  # ++ scale factor

    axes[2].set_ylabel('lon (deg*1.e4)')
    axes[2].plot(ind_fov, amsu_t1['dataset']['amb1c_latlon'][ind_scan, ind_fov, 1], linestyle='none', marker='+', markersize=16, markerfacecolor="red")  # ++ scale factor
    for ax in axes:
        ax.set_xlabel('ifov')
        ax.xaxis.set_major_formatter(FormatStrFormatter('%0.0f'))
        ax.set_title('scan : {ind_scan}, channel : {ich}'.format(ind_scan=ind_scan, ich=ich))
    if len(ind_fov) < 10:
        for ax in axes:
            ax.xaxis.set_ticks(ind_fov)
    else:
        for ax in axes:
            ax.xaxis.set_ticks(ind_fov[::10])

    fig.tight_layout()

    plt.show()

    # ++ un jour peut-etre j'arriverai  faire des contours
    # cf. http://nbviewer.ipython.org/github/jrjohansson/scientific-python-lectures/blob/master/Lecture-4-Matplotlib.ipynb

    #++X, Y = numpy.meshgrid(ind_scan, ind_fov)
    #++Z = amsu_t1['dataset']['amb1c_btemps'].T # ++ channel ++ vm ++ scale factor
    #++fig, ax = plt.subplots()
    #++p = ax.pcolor(X/(2*pi), Y/(2*pi), Z, cmap=cm.RdBu, vmin=abs(Z).min(), vmax=abs(Z).max())
    #++cb = fig.colorbar(p, ax=ax)


def plot_amsu_t1(amsu_t1, testfilename):

    """
    Unrealistic data 1 channel 1 scan 1 fov on Dakar

    >>> index_simulation = 1
    >>> amsu_t1 = simul_amsu_t1(index_simulation)
    >>> testfilename = './ginette.svg'
    >>> result =  plot_amsu_t1(amsu_t1, testfilename)

    # ++...        'lon': numpy.linspace(-25., +25., endpoint=True, num=nrows),
    # ++...        'lat': numpy.linspace(-12., +45., endpoint=True, num=nrows),

    Unrealistic data 1 channel 1 scan 3 fov centered on Dakar

    >>> index_simulation = 2
    >>> amsu_t1 = simul_amsu_t1(index_simulation)
    >>> testfilename = './ginette.svg'
    >>> result =  plot_amsu_t1(amsu_t1, testfilename)

    :returns: name of the output file if ok or -1 if error
    :rtype: integer
    """

    status = -1

    logger = logging.getLogger()

    lonmin = amsu_t1['dataset']['amb1c_latlon'][:, :, 1].min() / 1.E4
    latmin = amsu_t1['dataset']['amb1c_latlon'][:, :, 0].min() / 1.E4
    lonmax = amsu_t1['dataset']['amb1c_latlon'][:, :, 1].max() / 1.E4
    latmax = amsu_t1['dataset']['amb1c_latlon'][:, :, 0].max() / 1.E4

    logger.info('lonmin {0}'.format(lonmin))
    logger.info('lonmax {0}'.format(lonmax))
    logger.info('latmin {0}'.format(latmin))
    logger.info('latmax {0}'.format(latmax))

    # same with missing values #++ to be verified
    mask_vm = amsu_t1['dataset']['amb1c_btemps'] <= -99999
    ind_vm = numpy.where(mask_vm)[0]
    # ind_vm  comparer avec amb1c_btemps_ma
    # following line : thanks to
    # http://matplotlib.1069221.n5.nabble.com/color-missing-value-with-contourf-td511.html
    amb1c_btemps_ma = numpy.ma.masked_array(amsu_t1['dataset']['amb1c_btemps'], mask=(amsu_t1['dataset']['amb1c_btemps'] == -99999))

    lonmin_mv = amsu_t1['dataset']['amb1c_latlon'][ind_vm].min() / 1.E4
    latmin_mv = amsu_t1['dataset']['amb1c_latlon'][ind_vm].min() / 1.E4
    lonmax_mv = amsu_t1['dataset']['amb1c_latlon'][ind_vm].max() / 1.E4
    latmax_mv = amsu_t1['dataset']['amb1c_latlon'][ind_vm].max() / 1.E4

    if testfilename is None:
        look = 'filename'
        scale = 1.
        geomin = geolocation_to_string_py(lonmin, latmin, look, scale)
        geomax = geolocation_to_string_py(lonmax, latmax, look, scale)
        project_id_env = os.environ['PROJECT_ID']
        # ++ le temps dans les donnes simules
        # ++yyyymmdd = dateutil.parser.parse('{0}'.format(amsu_t1['header1']['yyyymmdd']), ignoretz=True)
        # ++yyyy = yyyymmdd.strftime('%Y')
        # ++mm = yyyymmdd.strftime('%m')
        # ++dd = yyyymmdd.strftime('%d')
        # ++fullfilename = '{0}/AMSU/{1}/{2}/{3}/{4}_{1}{2}{3}_{5}_{6}.dat'.format(project_id_env, yyyy, mm, amsu_t1['header1']['numch'], yyyy, mm, dd, geomin, geomax)
        fullfilename = 'ginette'
    else:
        fullfilename = testfilename

    #
    # Dakar Coordinates: 1441#34#N 1726#48#W
    cities = {'Dakar': {'lat': 14.692778, 'lon': -17.446667}}

    m = Basemap(projection='ortho', lon_0=cities['Dakar']['lon'], lat_0=cities['Dakar']['lat'])
    m.drawcoastlines()
    # draw parallels and meridians.
    # label parallels on right and top
    # meridians on bottom and left
    parallels = numpy.arange(-81, 81, 20.)
    # labels = [left,right,top,bottom]
    m.drawparallels(parallels, labels=[False, True, True, False])
    meridians = numpy.arange(10., 351., 20.)
    m.drawmeridians(meridians, labels=[True, False, False, True])
    # si on veut m.drawcountries()

    # map city coordinates to map coordinates
    x, y = m(cities['Dakar']['lon'], cities['Dakar']['lat'])
    plt.plot(x, y, 'ro')
    #print('basemap dakar lon {0} lat{1}'.format(x, y))
    plt.text(x + 250000, y - 150000, 'Dakar', bbox=dict(facecolor='yellow', alpha=0.5))

    # ou
    lower_left = (lonmin, latmin)
    lower_right = (lonmax, latmin)
    upper_right = (lonmax, latmax)
    upper_left = (lonmin, latmax)
    logger.info('lower_left {0}'.format(lower_left))
    logger.info('lower_right {0}'.format(lower_right))
    logger.info('upper_right {0}'.format(upper_right))
    logger.info('upper_left {0}'.format(upper_left))
    xs = [lower_left[0], upper_left[0], upper_right[0], lower_right[0], lower_left[0]]
    ys = [lower_left[1], upper_left[1], upper_right[1], lower_right[1], lower_left[1]]
    x, y = m(xs, ys)

    for corner, xc, yc in zip(('lower_left ({0},{1})'.format(lonmin, latmin),
                               'upper_left ({0},{1})'.format(lonmax, latmin),
                               'upper_right ({0},{1})'.format(lonmax, latmax),
                               'lower_right ({0},{1})'.format(lonmin, latmax)), x, y):
        # draw the corner name in a green (shaded) box
        plt.plot(xc, yc, 'bo')
        plt.text(xc + 250000, yc - 150000, corner, bbox=dict(facecolor='green', alpha=0.5))

    # draw a rectangle with these corner
    m.plot(xs, ys, latlon=True)

    # data

    # scan selection
    scan_min = 0  # ++ parametre
    scan_max = 0  # ++ parametre
    #++ind_scan = numpy.where(numpy.logical_and(scan_min <= amsu_t1['dataset']['scan'], amsu_t1['dataset']['scan'] <= scan_max))[0]
    # if no scan selection
    #ind_scan = range(amsu_t1['dataset']['amb1c_btemps'].shape[0])
    ind_scan = (0)  # ++ parametre pour voir la 1re scan
    # ++ ind_scan = range(scan_min, scan_max + 1)

    # fov selection
    fov_min = 0  # ++ parametre
    fov_max = 0  # ++ parametre
    # if no fov selection
    ind_fov = range(amsu_t1['dataset']['amb1c_btemps'].shape[1])
    # ind_fov = (0) #++ parametre pour voir la 1re fov
    # ++ ind_fov = range(fov_min, fov_max + 1)] # ++ essayer

    # seulement les coordonnes
    #++m.plot(amsu_t1['dataset']['amb1c_latlon'][:, :, 1] / 1.E4, amsu_t1['dataset']['amb1c_latlon'][:, :, 0], latlon=True, color='b')  # ++ je ne les vois pas
    x, y = m(amsu_t1['dataset']['amb1c_latlon'][:, :, 1] / 1.E4, amsu_t1['dataset']['amb1c_latlon'][:, :, 0] / 1.E4)
    # debug print('vrai lon {0} lat{1}'.format(amsu_t1['dataset']['amb1c_latlon'][:, :, 1] / 1.E4, amsu_t1['dataset']['amb1c_latlon'][:, :, 0] / 1.E4))
    # debug print('basemap vrai lon {0} lat{1}'.format(x, y))
    # debug print('basemap vrai tuple lon {0} lat{1}'.format(tuple(x), tuple(y)))
    # debug print('basemap vrai tuple[0] lon {0} lat{1}'.format(tuple(x[0]), tuple(y[0])))
    plt.plot(tuple(x[0]), tuple(y[0]), '-', linewidth=2, color='r')
    # debug x, y = m((-21, -17, -12), (9, 14, 21)) # ++  forage
    # debug print('basemap force lon {0} lat{1}'.format(x, y))
    # debug plt.plot(x, y, '-', linewidth=2, color='g')

    # seulement les coordonnes in selection
    # #++OK avec 0, 0 m.plot(amsu_t1['dataset']['amb1c_latlon'][ind_scan, ind_fov, 1], amsu_t1['dataset']['amb1c_latlon'][ind_scan, ind_fov, 0], latlon=True, color='y')

    # ++ on ne peut pas tout prendre sous peine de *** MemoryError:
    lons_1d = numpy.unique(amsu_t1['dataset']['amb1c_latlon'][ind_scan, ind_fov, 1]) / 1.E4
    lats_1d = numpy.unique(amsu_t1['dataset']['amb1c_latlon'][ind_scan, ind_fov, 0]) / 1.E4
    nlats = lats_1d.shape[0]
    nlons = lons_1d.shape[0]

    # ++ use meshgrid instead of sequence below
    lons = numpy.array([lons_1d, ] * nlats)
    # nb : lons[1] == lons[0] must be TRUE,
    # lons.shape must be nlats, nlons ++ assert
    lats = numpy.array([lats_1d, ] * nlons).transpose()
    # nb : lats[0] must be constant
    # and lats.shape must be nlats, nlons ++ assert

    x, y = m(lons, lats)

    ich = 0  # ++ parametre
    # les tb
    #print('temperature {0}'.format(amsu_t1['dataset']['amb1c_btemps'][:, :, ich ]))
    #print('temperature tuple {0}'.format(tuple(amsu_t1['dataset']['amb1c_btemps'][:, :, ich ])))
    #print('temperature[0] {0}'.format(amsu_t1['dataset']['amb1c_btemps'][:, :, ich ][0]))
    #print('temperature[0] tuple {0}'.format(tuple(amsu_t1['dataset']['amb1c_btemps'][:, :, ich ][0])))
    #++m.contour(amsu_t1['dataset']['amb1c_latlon'][:, :, 1]/1.E4, amsu_t1['dataset']['amb1c_latlon'][:, :, 0]/1.E4, amsu_t1['dataset']['amb1c_btemps'][:, :, ich ]/ 1.E2, latlon=True, linewidths=1.5)
    Z = tuple(amsu_t1['dataset']['amb1c_btemps'][:, :, ich][0])  # ++
    Z = amsu_t1['dataset']['amb1c_btemps'][:, :, ich]  # ++
    Z = 0.75*(numpy.sin(2.*lats)**8*numpy.cos(4.*lons))  # forcage ++ cf. contour1.py de basempa galerie BIDON
    m.contour(x, y, Z, linewidths=1.5)  # CA MARCHE PAS ++

    plt.show()

    result = True

    return result


def plot_amsu_t1_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['testname']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    testfilename = fromcli.testfilename
    testname = fromcli.testname

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('testname={0}'.format(testname))
    logger.info('testfilename={0}'.format(testfilename))

    if testname == 'simul1':
        index_simulation = 1
        amsu_t1 = simul_amsu_t1(index_simulation)
        logger.info(amsu_t1)

    elif testname == 'simul2':
        index_simulation = 2
        amsu_t1 = simul_amsu_t1(index_simulation)
        logger.info(amsu_t1)

    elif testname == 'simul3':
        index_simulation = 3
        amsu_t1 = simul_amsu_t1(index_simulation)
        logger.info(amsu_t1)
    elif testname == 'test2':
        project_id_env = os.environ['PROJECT_ID']
        testfilename_read = '{0}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C'.format(project_id_env)
        nomcanal = 'b'
        try:
            (header, dataset) = read_amsua1c_py(testfilename_read, nomcanal)
        except:
            logger.error('pb read_amsua1c_py', exc_info=True)
        else:
            amsu_t1 = {'header': header, 'dataset': dataset}
    elif testname == 'test3':
        project_id_env = os.environ['PROJECT_ID']
        testfilename_read = '{0}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C'.format(project_id_env)
        nomcanal = 'b'
        try:
            (header, dataset) = read_amsua1c_py(testfilename_read, nomcanal)
        except:
            logger.error('pb read_amsua1c_py', exc_info=True)
        else:
            amsu_t1 = {'header': header, 'dataset': dataset}

    else:
        msg = 'testname {testname} unknown'.format(testname=testname)
        logger.error(msg)
        raise ValueError(msg)

    if (amsu_t1['dataset']['amb1c_btemps'] == -999999).all():
        msg = "only missing values in whole amsu_t1['dataset']['amb1c_btemps']"
        logging.info(msg)
        raise ValueError(msg)

    try:
        #++result = plot_amsu_t1_scan_fov(amsu_t1, testfilename, channel_selection=0, ind_scan=0)
        result = plot_amsu_t1_scan_fov(amsu_t1, testfilename,
                                       channel_selection=0,
                                       scan_selection=0)
        #++ example pour midfov : fov_selection=[44]) #++ midfov value to be checked
        #++ basemap result = plot_amsu_t1(amsu_t1, testfilename)
    except:
        logger.error('pb simul_amsu_t1')
        raise
    else:
        logger.info('plot_amsu_t1 reach end')

    if result is None:
        logger.error('pb plot_amsu_t1')

# Run main, if called from the command line
if __name__ == '__main__':
    plot_amsu_t1_main()
