#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

==================
plot_amsu_t2.py
==================

SYNOPSIS
========

.. code-block:: bash

   plot_amsu_t2.py --testname testname --testfilename testfilename

DESCRIPTION
===========

.. warning::

   this CLI exists only to consolidate all amsu_t2 funtions

   :func:`mem_to_file_amsu_t2_py`  has to be used inside a main program
   for realistic amsu_t2 values

Plot info and data of **amsu_t2** structure to an SVG++ file

If **testfilename** is not empty, this filename will be written, otherwise
amsu_t2_plot++ terminology will be applied.

If **testname**, test1 set values will be written : no realistic silly values.

If **testname** is set to test2, :file:`${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat` will be plot and figure save in **testfilename**.

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest plot_amsu_t2.py

Unrealistic example with dataset test1:

.. code-block:: bash

   testname=test1
   testfilename='./plot_amsu_t2_${testname}.svg'
   plot_amsu_t2.py --testname ${testname} --testfilename ${testfilename}
   tlogd.sh plot_amsu_t2_main
   xdg-open ${testfilename}

Realistic example with dataset test2 ie :file:`${PROJECT_ID}/AMSU//2011/04/b3_20110403_022w12s_022e42n.dat`:

.. code-block:: bash

   testname=test2
   testfilename='./plot_amsu_t2_${testname}.svg'
   plot_amsu_t2.py --testname ${testname} --testfilename ${testfilename}
   tlogd.sh plot_amsu_t2_main
   xdg-open ${testfilename}

The same in debug

.. code-block:: bash

   python -m pdb ${PROJECT}/src/plot_amsu_t2.py --testname ${testname} --testfilename ${testfilename}

SEE ALSO
========

:func:`extract_amsu_py` where this function can be called ++

use :func:`geolocation_to_string_py`

TODO
====

test si lat;lon non null

pas forcement de fichier image sauv : un parametre

finish writing

cf. plot_amsu.pro qui aurait pu servir de point de dpart

EVOLUTIONS
==========

$Id: plot_amsu_t2.py$

$URL$

  * creation from mem_to_file_amsu_t2_py.py
    thanks to http://pythonbits.blogspot.fr/2011/09/i-have-problem-with-basemap-plotting.html,
    https://www.packtpub.com/books/content/plotting-geographical-data-using-basemap for plotting circle and text
"""

__all__ = ["plot_amsu_t2"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

from geolocation_to_string_py import geolocation_to_string_py

import numpy
import dateutil
import matplotlib.pyplot as plt

try:
    from mpl_toolkits.basemap import Basemap
except ImportError:
    print('Sorry, python module Basemap is needed')
    exit(-1)


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """plot AMSU_T2 dataset"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--testfilename',
                      help='testfilename to be written',
                      dest='testfilename',
                      metavar='<testfilename>',
                      action='store')

    parser.add_option('--testname',
                      help='name of dataset values for test',
                      dest='testname',
                      metavar='<testname>',
                      action='store')

    return parser


def plot_amsu_t2(amsu_t2, testfilename):

    """
    >>> header1 = {'yyyymmdd': 99999999, 'numch': 'b3', 'nbpix': 0, 'resol': 0}
    >>> header2 = {'lonmin': 0., 'lonmax': 0., 'latmin': 0., 'latmax': 0.}
    >>> nrows = 20
    >>> data = {'desc': numpy.zeros(nrows, 'Int8'),
    ...    'hour': numpy.zeros(nrows, 'Float32'),
    ...        'fov': numpy.zeros(nrows, 'Int8'),
    ...        'lon': numpy.zeros(nrows, 'Float32'),
    ...        'lat': numpy.zeros(nrows, 'Float32'),
    ...        'landseamask': numpy.zeros(nrows, 'Int8'),
    ...        'tb': numpy.zeros(nrows, 'Float32')}
    >>> amsu_t2 = {'header1': header1, 'header2': header2, 'data': data}
    >>> print(amsu_t2)
    {'header2': {'latmax': 0.0, 'lonmin': 0.0, 'lonmax': 0.0, 'latmin': 0.0}, 'header1': {'resol': 0, 'numch': 'b3', 'nbpix': 0, 'yyyymmdd': 99999999}, 'data': {'fov': array([0, 0], dtype=int8), 'hour': array([ 0.,  0.], dtype=float32), 'lon': array([ 0.,  0.], dtype=float32), 'lat': array([ 0.,  0.], dtype=float32), 'desc': array([0, 0], dtype=int8), 'tb': array([ 0.,  0.], dtype=float32), 'landseamask': array([0, 0], dtype=int8)}}
    >>> testfilename = './ginette.svg'
    >>> result =  plot_amsu_t2(amsu_t2, testfilename)

    # ++...        'lon': numpy.linspace(-25., +25., endpoint=True, num=nrows),
    # ++...        'lat': numpy.linspace(-12., +45., endpoint=True, num=nrows),
    :returns: name of the output file if ok or -1 if error
    :rtype: integer
    """

    status = -1

    if testfilename is None:
        look = 'filename'
        scale = 1.
        geomin = geolocation_to_string_py(amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmin'], look, scale)
        geomax = geolocation_to_string_py(amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmax'], look, scale)
        project_id_env = os.environ['PROJECT_ID']
        yyyymmdd = dateutil.parser.parse('{0}'.format(amsu_t2['header1']['yyyymmdd']), ignoretz=True)
        yyyy = yyyymmdd.strftime('%Y')
        mm = yyyymmdd.strftime('%m')
        dd = yyyymmdd.strftime('%d')
        fullfilename = '{0}/AMSU/{1}/{2}/{3}/{4}_{1}{2}{3}_{5}_{6}.dat'.format(project_id_env, yyyy, mm, amsu_t2['header1']['numch'], yyyy, mm, dd, geomin, geomax)
    else:
        fullfilename = testfilename

    #
    # Dakar Coordinates: 1441#34#N 1726#48#W
    cities = {'Dakar': {'lat': 14.692778, 'lon': -17.446667}}

    m.drawcoastlines()
    # draw parallels and meridians.
    # label parallels on right and top
    # meridians on bottom and left
    parallels = numpy.arange(-81, 81, 20.)
    # labels = [left,right,top,bottom]
    m.drawparallels(parallels, labels=[False, True, True, False])
    meridians = numpy.arange(10., 351., 20.)
    m.drawmeridians(meridians, labels=[True, False, False, True])
    # si on veut m.drawcountries()

    # map city coordinates to map coordinates
    x, y = m(cities['Dakar']['lon'], cities['Dakar']['lat'])
    plt.plot(x, y, 'ro')
    plt.text(x + 250000, y - 150000, 'Dakar', bbox=dict(facecolor='yellow', alpha=0.5))

    # ou
    lower_left = (amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmin'])
    lower_right = (amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmin'])
    upper_right = (amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmax'])
    upper_left = (amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmax'])
    xs = [lower_left[0], upper_left[0], upper_right[0], lower_right[0]]
    ys = [lower_left[1], upper_left[1], upper_right[1], lower_right[1]]
    x, y = m(xs, ys)

    for corner, xc, yc in zip(('lower_left ({0},{1})'.format(amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmin']),
                               'upper_left ({0},{1})'.format(amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmin']),
                               'upper_right ({0},{1})'.format(amsu_t2['header2']['lonmax'], amsu_t2['header2']['latmax']),
                               'lower_right ({0},{1})'.format(amsu_t2['header2']['lonmin'], amsu_t2['header2']['latmax'])), x, y):
        # draw the corner name in a green (shaded) box
        plt.plot(xc, yc, 'bo')
        plt.text(xc + 250000, yc - 150000, corner, bbox=dict(facecolor='green', alpha=0.5))

    # draw a rectangle with these corner
    m.plot(xs, ys, latlon=True)

    # data

    # temporal selection
    hour_min = 5.  # ++ parametre
    hour_max = 7.  # ++ parametre
    ind_temporal = numpy.where(numpy.logical_and(hour_min <= amsu_t2['data']['hour'], amsu_t2['data']['hour'] <= hour_max))[0]

    # fov selection
    fov_min = 1  # ++ parametre
    fov_max = 1  # ++ parametre
    ind_fov = numpy.where(numpy.logical_and(fov_min <= amsu_t2['data']['fov'], amsu_t2['data']['fov'] <= fov_max))[0]

    # ++ quand k'aurai tous les parametres de selection ind_selection = numpy.logical_and(ind_temporal, ind_fov)
    ind_selection = numpy.logical_and(ind_temporal, ind_fov)

    # seulement les coordonnes
    m.plot(amsu_t2['data']['lon'], amsu_t2['data']['lat'], latlon=True, color='b')

    # seulement les coordonnes in selection
    m.plot(amsu_t2['data']['lon'][ind_selection], amsu_t2['data']['lat'][ind_selection], latlon=True, color='y')
    #++X, Y = numpy.meshgrid(amsu_t2['data']['lon'], amsu_t2['data']['lat'])
    #++ x, y = m()
    #++m.pcolormesh(x, y, X)

    # les orbites
    #++m.contour(amsu_t2['data']['lon'], amsu_t2['data']['lat'], amsu_t2['data']['desc'], latlon=True)

    # les temps
    # ++m.contour(amsu_t2['data']['lon'], amsu_t2['data']['lat'], amsu_t2['data']['hour'], latlon=True)

    # les tb
    #++m.contour(amsu_t2['data']['lon'], amsu_t2['data']['lat'], amsu_t2['data']['tb'], latlon=True)
    plt.show()

    result = True

    return result


def plot_amsu_t2_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['testname']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    testfilename = fromcli.testfilename
    testname = fromcli.testname

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('testname={0}'.format(testname))
    logger.info('testfilename={0}'.format(testfilename))

    if testname == 'test1':
        header1 = {'yyyymmdd': 99999999, 'numch': 'b3', 'nbpix': 0, 'resol': 0}
        header2 = {'lonmin': 0., 'lonmax': 0., 'latmin': 0., 'latmax': 0.}
        nrows = 2
        data = {'desc': numpy.zeros(nrows, 'Int8'),
                'hour': numpy.zeros(nrows, 'Float32'),
                'fov': numpy.zeros(nrows, 'Int8'),
                'lon': numpy.zeros(nrows, 'Float32'),
                'lat': numpy.zeros(nrows, 'Float32'),
                'landseamask': numpy.zeros(nrows, 'Int8'),
                'tb': numpy.zeros(nrows, 'Float32')}
        amsu_t2 = {'header1': header1, 'header2': header2, 'data': data}
    elif testname == 'test2':
        from file_amsu_t2_to_mem_py import file_amsu_t2_to_mem_py
        project_id_env = os.environ['PROJECT_ID']
        testfilename_read = '{0}/AMSU/2011/04/b3_20110403_022w12s_022e42n.dat'.format(project_id_env)
        result = file_amsu_t2_to_mem_py(None, None, None, None, None, None, testfilename_read)
        if result:
            amsu_t2 = result
    else:
        logger.error('no dataset to be plot')

    result = plot_amsu_t2(amsu_t2, testfilename)

    if result is None:
        logger.error('pb plot_amsu_t2')

# Run main, if called from the command line
if __name__ == '__main__':
    plot_amsu_t2_main()
