function [result] = plot_cumulEPSAT(cumuldata, Annee, Mois, Jour, Longitudes, Latitudes)

%PLOT_CUMULEPSAT dessine les précipitations cumulées sur une journée d'après les données EPSAT

%+
%
% =================
% plot_cumulEPSAT.m
% =================
%
% .. function:: plot_cumulEPSAT(cumuldata, Annee, Mois, Jour, Longitudes, Latitudes)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * Annee (en caractères) : Année (AAAA) du jour
%     associée aux données cumuldata
%   * Mois (en caractères) : Numéro du mois (MM)
%     aux données cumuldata
%   * Jour (en caractères) : Numéro du jour dans le mois
%     associé aux données cumuldata
%   * Longitudes : Variable de longitudes
%     associées aux données cumuldata
%   * Latitudes : Variable de latitudes
%     associées aux données cumuldata
%   * cumuldata : Données de précipitations estimées sur une journée AAAAMMJJ
%     dans le domaine fournies par Latitudes et Longitudes.
%     dimension 1 : lon,  dimension 2 : lat
%
% Dessin des précipitations cumulées sur une période et un domaine
% géographique donné en argument
%
% Figure is saved  in :file:`./plot_cumulEPSAT.ps`
%
% EXAMPLES
% ========
%
% To plot cumul of precipitation from simulated EPSAT data:
%
% see demo 1
%
% To plot cumul of precipitation from a real EPSAT file :
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :func:`EPSATread`
% :func:`simul_epsat`
%
% :ref:`world_map`
%
% :func:`cumulEPSAT_day`
%
% TODO
% ====
%
% trouver l'équivalent de 'alphadata'
%
% strange continents contour on the plot !!
%
% get rid of test equality between float using ==
%
% cumulEPSAT_day should output metadata (date and geolocation)
%
% !! cumul == 0 forcé à NaN !!! mais pourquoi ?
%
% Annee, Mois Jour should be arrays or better only one datestr_value array or usage of Temps
%
% coding rules : use permute of \'
%
% check for implementation of LaTeX interpreter on octave > 3.4
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/plot_cumulEPSAT.m $
%
% - fplod 20110805T083304Z cratos (Linux)
%
%   * fix demo with real EPSAT normal file
%
% - jaclod 2011-07-28
%
%   * Pour le imagesc, repassage en transposée sinon les longitudes
%     (dimension 1) sont en y sur le dessin.
%   * Fonctionne sur le Matlab de cratos.
%
% - pinsard 2011-06-08T14:42:32Z loholt1.ipsl.polytechnique.fr (Linux)
%
%   * remove unused lonmin, lonmax, latmin, latmax
%
% - fplod 20110520T101102Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * remove diacriticals from title and labels
%     to avoid "warning: ft_render: skipping missing glyph for character"
%     running with octave 3.4
%
% - fplod 20110311T092857Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * save figure
%   * revision of description
%   * add demo with :func:`simul_epsat`
%   * suppress fullscreen directive (known problem using octave)
%
% - fplod 20110310T081425Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * fix rst header
%   * minimal modif to be used on cratos with octave 3.2.4 :
%     * world map cmgco_traitdecote.mat (need varamma_startup)
%     * alphadata unknown
%
% - jaclod 2011-03-02
%
%   * Ajout de la doc et de la démonstration
%
% - jaclod 2011-02-25
%
%   * Creation
%
%-
global application
%
result=-1;
%
usage='[result] = plot_cumulEPSAT(cumuldata, Annee, Mois, Jour, Longitudes, Latitudes)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% Chargement des fichiers pour tracer les côtes.
load('cmgco_traitdecote.mat','Xgco','Ygco');

% Les données dont la valeur est 0 passent à "NaN" pour ne pas être
% affichées dans l'image.
%++cumuldata(cumuldata == 0) = NaN;

% On trace en utilisant la transposée pour que la longitude (dimension 1)
% soit en x.
my_image = imagesc(Longitudes,Latitudes,cumuldata');
switch application
  case {'matlab'}
     set(my_image,'alphadata',~isnan(cumuldata'));
  case {'octave'}
     disp(['www : no alphadata available using ', application]);
  otherwise
     error('eee : unknown application running');
end

% Modification des axes.
set(gca,'ydir','normal')

hold on
colorbar;

% Ajout des côtes.
coast = plot(Xgco,Ygco,'k.');

% Ajout de titres.
xlabel('Longitudes')
ylabel('Latitudes')
title(['Precipitations cumulees en mm pour le ' ([Annee '-' Mois '-' Jour])]);

% Exécution de la fonction permettant d'afficher les coordonnées en format
% °N/S et °E/W.
%status_ticklatlon=ticklatlon();

% Ajout d'une grille.
grid on;

% save figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./', mfilename '.' printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;

result=0;

end

%!demo
%! % To plot cumul of precipitation from simulated EPSAT data:
%!
%! clear all
%! close all
%! varamma_startup
%! more off
%! index_simulation = uint8(2);
%! [lon_value, lat_value, datejul_value, datestr_value, epsat_value]=simul_epsat(index_simulation);
%! % First datestr_value is used here as the only date of cumulEPSAT_day
%! Annee = datestr_value(1,1:4);
%! Mois = datestr_value(1,5:6);
%! Jour = datestr_value(1,7:8);
%! [cumuldata] = cumulEPSAT_day(epsat_value);
%! result = plot_cumulEPSAT(cumuldata, Annee, Mois, Jour, lon_value, lat_value);
%! whos

%!demo
%! % To plot cumul of precipitation for EPSAT file
%! % :file:`${PROJECT_ID}/EPSAT/2006/08/surf-rr_epsat-sg_multi-sat_010d_30min_20060801_v3.1-02c.nc`::
%! clear all
%! close all
%! varamma_startup;
%! more off
%! ncfilesset = 'normal';
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour);
%! [data, Temps, Longitudes, Latitudes] = EPSATread('normal', fullfilename);
%! mindim1 = min(Temps);
%! maxdim1 = max(Temps);
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! [data, Temps, Longitudes, Latitudes] = extractedmatrixdata(data, Temps, Longitudes, Latitudes, mindim1, maxdim1, lonmin, lonmax, latmin, latmax);
%! [cumuldata] = cumulEPSAT_day(data);
%! result = plot_cumulEPSAT(cumuldata, Annee, Mois, Jour, Longitudes, Latitudes);
