#!/usr/bin/env python
# -*- coding: utf-8 -*-

__docformat__ = 'reStructuredText'
__autodoc__ = ['plot_msg_xrit_raw']

import sys

import datetime
import matplotlib.pyplot as plt
import numpy
from numpy import ma


def plot_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn, missing_value):

    """

    =================
    plot_msg_xrit_raw
    =================

    DESCRIPTION
    ===========

    Plot data from MSG XRIT

    Parameters
    ----------

    values :
        values

    yyyy : yyyy
        year

    mm : mm
        month

    dd : dd
        day

    hh : hh
        hour

    mn : mn
        minute

    missing_value : missing value
        will be replace by NaN

    Returns
    -------

    result : int
       1 if ok

    Raises
    ------

    Missing parameters

    EXAMPLES
    ========

    From scratch to plot some simulated data in
    :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.plot_msg_xrit_raw.raw`

    >>> from simul_msg_xrit_raw import simul_msg_xrit_raw
    >>> index_simulation = 1
    >>> (product, values, missing_value) = simul_msg_xrit_raw(index_simulation)
    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> result = plot_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn, missing_value)
    iii : plot IR108 values 20060801T000000Z

    From real file :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw`

    >>> from read_msg_xrit_raw import read_msg_xrit_raw
    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> product = 'IR108'
    >>> values = read_msg_xrit_raw(product, yyyy, mm, dd, hh, mn)
    iii : opening for reading /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw
    >>> missing_value = -999.0
    >>> result = plot_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn, missing_value)
    iii : plot IR108 values 20060801T000000Z

    SEE ALSO
    ========

    :ref:`msg_data`

    :func:`simul_msg_xrit_raw`
    :func:`read_msg_xrit_raw`

    TODO
    ====

    coding rules

    save image

    lat, lon vs indices

    EVOLUTIONS
    ==========

    $Id$

    $URL$

    - pinsard 20130731T090914Z loholt1.ipsl.polytechnique.fr (Linux)

      * creation
        thanks to http://stackoverflow.com/questions/11776663/ploting-a-2d-array-with-matplotlib
        and http://physics.nmt.edu/~raymond/software/python_notes/paper003.html
    """

    result = -1
    #
    usage = 'result = plot_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn, missing_value)'
    #
    # build title
    # ++ replace 'plot_msg_xrit_raw' by the name of this function
    yyyymmddhhmn = datetime.datetime(yyyy, mm, dd, hh, mn, 00)
    label = yyyymmddhhmn.strftime('%Y%m%dT%H%M00Z')
    title = 'plot %s values %s' % (product, label)

    # replace missing values by NaN
    mask = ma.masked_values(values, missing_value)
    fill_value = numpy.nan
    filled_array = ma.filled(mask, fill_value)
    Z = numpy.ma.array(filled_array, mask=numpy.isnan(filled_array))

    X, Y = numpy.meshgrid(range(Z.shape[0]+1), range(Z.shape[1]+1))
    im = plt.pcolormesh(X, Y, Z,  cmap='hot')
    plt.colorbar(im, orientation='horizontal')
    plt.xlabel('X index')
    plt.xlim(xmin=0, xmax=X.shape[1] - 1)
    plt.ylabel('Y index')
    plt.ylim(ymin=0, ymax=Y.shape[1] - 1)
    plt.title(title)
    plt.show()

    print('iii : plot %s values %s' % (product, label))
    #
    result = 0
    #

# Run main, if called from the command line
if __name__ == '__main__':
    import doctest
    doctest.testmod()
