#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

===================
read_amsua1c_py.py
==================

SYNOPSIS
========

.. code-block:: bash

   read_amsua1c_py --filename filename --nomcanal nomcanal

DESCRIPTION
===========

.. option:: --filename <filename>
.. option:: --nomcanal <nomcanal>

Reads AMSU-B Level 1C Data from AAPP-formatted input
Ralf Bennartz 2 August 1999

.. only:: html or latex

    .. graphviz::

       digraph read_amsua1c_py{

          amsu_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/AMSU/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];

          read_amsua1c_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/extract_amsu_py.py",
          label="${PROJECT}/src/extract_amsu_py.py"];

          {amsu_t1} -> {read_amsua1c_py} -> {impressiondecontrole status};
      }

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest read_amsua1c_py.py

To read ReadRawFile test file (AMSU-A file):

.. code-block:: bash

   filename="${PROJECT}/src/external_python/ReadRawFile/test/NSS.AMAX.NK.D10010.S0020.E0212.B6062324.GC.L1C"
   nomcanal="a"
   read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}
   tlogd.sh read_amsua1c_py_main

Realistic example with one VARAMMA file AMSU-B channel
:file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C`

.. warning::

       this file must be linked or copy in ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C

       Too big to be in svn repository.

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/
   ln -s /homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/
   filename="${PROJECT_ID}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C"
   nomcanal="b"
   read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}
   tlogd.sh read_amsua1c_py_main

Realistic example with another VARAMMA file AMSU-B channel
:file:`loholt1.ipsl.polytechnique.fr:/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C`

.. warning::

       this file must be linked or copy in ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C

       Too big to be in svn repository.

.. code-block:: bash

   mkdir -p ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/
   ln -s /homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C ${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2008_08_13
   filename="${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C"
   nomcanal="b"
   read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}
   tlogd.sh read_amsua1c_py_main

Same in debug mode

.. code-block:: bash

   python -m pdb ${PROJECT}/src/read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}

 comparer avec la sortie de TestReadRawFile.sh/TestReadRawFile.py:

.. code-block:: bash

   TestReadRawFile.sh
   ++

To read an AMSU-A NOAA15 file:

.. code-block:: bash

   filename="${PROJECT_ID}/AMSU/AMSUAN15/L1C/2006/2006_08_01/NSS.AMAX.NK.D06213.S0110.E0255.B4271112.GC.L1C"
   nomcanal="a"
   read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}
   tlogd.sh read_amsua1c_py_main

To read an AMSU B file:

.. code-block:: bash

   filename=${PROJECT_ID}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C
   nomcanal="b"
   read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}
   tlogd.sh read_amsua1c_py_main

To read an METOP file:

.. code-block:: bash

   filename="${PROJECT_ID}/AMSU/AMSUAM02/L1C/2007/2007_08_01/NSS.AMAX.M2.D07213.S0115.E0259.B0405455.SV.L1C"
   nomcanal="m"
   read_amsua1c_py.py --filename ${filename} --nomcanal ${nomcanal}
   tlogd.sh read_amsua1c_py_main

SEE ALSO
========

:ref:`data_amsu`

:func:`simu_amsu_t1` for simulation of this module

called by :func:`extract_amsu_py`

++ use :file:`ReadRawfile.py`
:file:`amsua_1c_header`
:file:`amsua_1c_record`
:file:`amsub-mhs_1c_header`
:file:`amsub-mhs_1c_record`

TODO
====

pour l'instant pas de log si mauvaise valeur de nomcanal

test with metop file (not available on callisto)

use exception instead of return value (raffinement)

EVOLUTIONS
==========

$Id$

- pinsard 20140930T081439Z climtest0.ipsl.polytechnique.fr (Linux)

  * add test with :file:`/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C`

- pinsard 20140929T145809Z loholt1.ipsl.polytechnique.fr (Linux)

  * add instructoon to test an AMSU-B file

- pinsard 20140807T141633Z loholt1.ipsl.polytechnique.fr (Linux)

  * add test with :file:`/homedata/eymard/varamma_d/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C`

- fplod 20140804T150527Z callisto.locean-ipsl.upmc.fr (Linux)

  * add option nomcanal
  * add an AMSU B example

- fplod 20140804T085315Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from read_amsua1c.pro

"""

__all__ = ["read_amsua1c_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

from ReadRawFile import ReadRawFile


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """read AMSU file"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--filename',
                      help='filename to be read',
                      dest='filename',
                      metavar='<filename>',
                      action='store')

    parser.add_option('--nomcanal',
                      help='name of the channel',
                      type='choice',
                      choices=['a', 'b', 'm'],
                      dest='nomcanal',
                      metavar='<nomcanal>',
                      action='store')

    return parser


def read_amsua1c_py(filename, nomcanal):

    """
    To read ReadRawFile test file:

    >>> project = os.environ['PROJECT']
    >>> filename = '{0}/src/external_python/ReadRawFile/test/NSS.AMAX.NK.D10010.S0020.E0212.B6062324.GC.L1C'.format(project)
    >>> nomcanal = 'a'
    >>> (header, dataset) = read_amsua1c_py(filename, nomcanal)
    >>> print('test ReadRawFile : NUMERO DU SATELLITE {0}'.format(header['ama1c_h_satid']))
    test ReadRawFile : NUMERO DU SATELLITE 15
    >>> print('test ReadRawFile : INSTRUMENT {0}'.format(header['ama1c_h_instrument']))
    test ReadRawFile : INSTRUMENT 10

    To read an AMSU-A NOAA15 file:

    >>> project_id_env = os.environ['PROJECT_ID']
    >>> filename = '{0}/AMSU/AMSUAN15/L1C/2006/2006_08_01/NSS.AMAX.NK.D06213.S0110.E0255.B4271112.GC.L1C'.format(project_id_env)
    >>> nomcanal = 'a'
    >>> (header, dataset) = read_amsua1c_py(filename, nomcanal)
    >>> print('test AMSU-A : NUMERO DU SATELLITE {0}'.format(header['ama1c_h_satid']))
    test AMSU-A : NUMERO DU SATELLITE 15
    >>> print('test AMSU-A : INSTRUMENT {0}'.format(header['ama1c_h_instrument']))
    test AMSU-A : INSTRUMENT 10

    To read an AMSU-B file:

    >>> project_id_env = os.environ['PROJECT_ID']
    >>> filename='{0}/AMSU/AMSUBN15/L1C/2011/2011_04_03/NSS.AMBX.NK.D11093.S0056.E0234.B6700304.GC.L1C'.format(project_id_env)
    >>> nomcanal = 'b'
    >>> (header, dataset) = read_amsua1c_py(filename, nomcanal)
    >>> print('test AMSU-B : NUMERO DU SATELLITE {0}'.format(header['amb1c_h_satid']))
    test AMSU-B : NUMERO DU SATELLITE 15
    >>> print('test AMSU-B : INSTRUMENT {0}'.format(header['amb1c_h_instrument']))
    test AMSU-B : INSTRUMENT 11
    >>> print("min dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].min()))
    min dataset['amb1c_btemps'] -999999
    >>> print("max dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].max()))
    max dataset['amb1c_btemps'] -999999

    To read an other AMSU-B file:

    >>> project_id_env = os.environ['PROJECT_ID']
    >>> filename='{0}/AMSU/AMSUBN15/L1C/2006/2006_08_13/NSS.AMBX.NK.D06225.S0126.E0310.B4288283.GC.L1C'.format(project_id_env)
    >>> nomcanal = 'b'
    >>> (header, dataset) = read_amsua1c_py(filename, nomcanal)
    >>> print('test AMSU-B : NUMERO DU SATELLITE {0}'.format(header['amb1c_h_satid']))
    test AMSU-B : NUMERO DU SATELLITE 15
    >>> print('test AMSU-B : INSTRUMENT {0}'.format(header['amb1c_h_instrument']))
    test AMSU-B : INSTRUMENT 11
    >>> print("min dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].min()))
    min dataset['amb1c_btemps'] -999999
    >>> print("max dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].max()))
    max dataset['amb1c_btemps'] 29921

    To read an METOP file:

    >>> project_id_env = os.environ['PROJECT_ID']
    >>> filename = '{0}/AMSU/AMSUAM02/L1C/2007/2007_08_01/NSS.AMAX.M2.D07213.S0115.E0259.B0405455.SV.L1C'.format(project_id_env)
    >>> nomcanal = 'm'
    >>> (header, dataset) = read_amsua1c_py(filename, nomcanal)
    >>> print('test METOP : NUMERO DU SATELLITE {0}'.format(header['amb1c_h_satid']))
    test METOP : NUMERO DU SATELLITE 2
    >>> print('test METOP : INSTRUMENT {0}'.format(header['amb1c_h_instrument']))
    test METOP : INSTRUMENT 10
    >>> print("min dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].min()))
    min temp metop ++ louche -1799745
    >>> print("max dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].max()))
    max temp metop ++ louche 10730407

    """


    logger = logging.getLogger()

    status = (None, None)

    # test if file exist
    #++if not os.path.exists(filename):
    #++    logging.error('file {0} not found'.format(filename))
    #++    return status

    # test if nomcanal correct
    project = os.environ['PROJECT']
    if nomcanal == 'a':
        hdr_file = '{0}/src/external_python/ReadRawFile/data/amsua_1c_header'.format(project)
        rec_file = '{0}/src/external_python/ReadRawFile/data/amsua_1c_record'.format(project)
    elif nomcanal == 'b':
        hdr_file = '{0}/src/external_python/ReadRawFile/data/amsub-mhs_1c_header'.format(project)
        rec_file = '{0}/src/external_python/ReadRawFile/data/amsub-mhs_1c_record'.format(project)
    elif nomcanal == 'm':
        hdr_file = '{0}/src/external_python/ReadRawFile/data/amsub-mhs_1c_header'.format(project)
        rec_file = '{0}/src/external_python/ReadRawFile/data/amsub-mhs_1c_record'.format(project)
    else:
        logger.error('nomcanal {0} pas ok'.format(nomcanal))
        return status
    #
    # Define and read in header structure
    # ++ migration define_amsua_header_struct
    # ++ migration READU, lu, ama_head
    # ++ migration byteorder,ama_head,/Lswap
    #
    # ++ migration Define and read in data structure
    # ++ migration define_amsua_struct,one_scana, flag1
    flag1 = 1  # ++ bidon migration
    if flag1 == 0:
        # ++ migration       & goto, pb_filename
        logger.info('debranchement goto ++ migration')
    #READU,1,ama1cdt
    # read each scanline separately
    # ++ migration for i in range(0, ama_head.h_scnlin-1):
    # ++ migration     if eof(lu) == 0:
    # ++ migration         goto, pb_filename
    # ++ migration     READU, lu, one_scana
    # ++ migration     #byteorder,one_scana,/Lswap
    # ++ migration     ama_scan[i] = one_scana

    bigend = False
    header, dataset = ReadRawFile(filename, hdr_file, rec_file, bigend)

    return (header, dataset)


def read_amsua1c_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['filename', 'nomcanal']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    filename = fromcli.filename
    nomcanal = fromcli.nomcanal

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('filename={0}'.format(filename))
    logger.info('nomcanal={0}'.format(nomcanal))

    (header, dataset) = read_amsua1c_py(filename, nomcanal)
    if header is None or dataset is None:
        logger.error('pb read_amsua1c_py with filename {0}'.format(filename))
    else:
        if nomcanal == 'a':
            logger.info('NUMERO DU SATELLITE {0}'.format(header['ama1c_h_satid']))
            logger.info('INSTRUMENT {0}'.format(header['ama1c_h_instrument']))
            logger.info("min dataset['amb1c_btemps'] {0}".format(dataset['ama1c_btemps'].min()))
            logger.info("max dataset['amb1c_btemps'] {0}".format(dataset['ama1c_btemps'].max()))
        if nomcanal == 'b':
            logger.info('NUMERO DU SATELLITE {0}'.format(header['amb1c_h_satid']))
            logger.info('INSTRUMENT {0}'.format(header['amb1c_h_instrument']))
            #logger.info('dataset {0}'.format(dataset))  # !! might be bug ++
            logger.info("min dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].min()))
            logger.info("max dataset['amb1c_btemps'] {0}".format(dataset['amb1c_btemps'].max()))
            logger.info("min dataset['amb1c_latlon'][:, :, 1] (amalong) {0}".format(dataset['amb1c_latlon'][:, :, 1].min()))
            logger.info("max dataset['amb1c_latlon'][:, :, 1] (amalong) {0}".format(dataset['amb1c_latlon'][:, :, 1].max()))
            logger.info("min dataset['amb1c_latlon'][:, :, 1] (amalat) {0}".format(dataset['amb1c_latlon'][:, :, 0].min()))
            logger.info("max dataset['amb1c_latlon'][:, :, 1] (amalat) {0}".format(dataset['amb1c_latlon'][:, :, 0].max()))

# Run main, if called from the command line
if __name__ == '__main__':
    read_amsua1c_py_main()
