#!/usr/bin/env python
# -*- coding: utf-8 -*-

__docformat__ = 'reStructuredText'
__autodoc__ = ['read_msg_xrit_raw']

import os
import sys
import datetime
import numpy


def read_msg_xrit_raw(product, yyyy, mm, dd, hh, mn):

    """

    ====================
    read_msg_xrit_raw.py
    ====================

    DESCRIPTION
    ===========

    read 3712x3712 data in a MSG raw file
    :file:`${PROJECT_ID}/MSG/yyyy/mm/dd/H-000-MSG1__-MSG11________-{product}____-CYCLE____-{yyyymmddhhmn}.raw`

    Parameters
    ----------

    None

    Returns
    -------

    values : numpy.ndarray
       values of product 3712x3712

    Raises
    ------

    EXAMPLES
    ========

    To read :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw`, write a copy in
    :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_raw.raw`
    and compare those
    two files to help detecting problems (either on files or in programs
    involved)::

    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> product = 'IR108'
    >>> values = read_msg_xrit_raw(product, yyyy, mm, dd, hh, mn)
    iii : opening for reading /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw
    >>> values.strides
    (14848, 4)
    >>> values.shape
    (3712, 3712)
    >>> values.ndim
    2
    >>> print values[0,0]
    -999.0

    >>> from write_msg_xrit_raw import write_msg_xrit_raw
    >>> result = write_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn)
    iii : opening for writing /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_raw.raw
    >>> import filecmp
    >>> project_id_env = os.environ['PROJECT_ID']
    >>> # build msg_xrit_raw filenames to be compared
    >>> original = project_id_env + '/MSG/2006/08/01/' + 'H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw'
    >>> written = project_id_env + '/MSG/2006/08/01/' + 'H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_raw.raw'
    >>> filecmp.cmp(original,written)
    True

    SEE ALSO
    ========

    :ref:`data_msg`

    :func:`simul_msg_xrit_raw`
    :func:`write_msg_xrit_raw`
    :func:`plot_msg_xrit_raw`

    TODO
    ====

    cleaning

    EVOLUTIONS
    ==========

    $Id$

    $URL$

    - pinsard 20130730T104937Z loholt1.ipsl.polytechnique.fr (Linux)

      * creation draft

    """

    usage = 'values = read_msg_xrit_raw(product,yyyy,mm,dd,hh,mn)'

    project_id_env = os.environ['PROJECT_ID']

    # build msg_xrit_raw filename to be read
    yyyymmddhhmn = datetime.datetime(yyyy, mm, dd, hh, mn, 00)
    yyyy = yyyymmddhhmn.strftime('%Y')
    mm = yyyymmddhhmn.strftime('%m')
    dd = yyyymmddhhmn.strftime('%d')
    hh = yyyymmddhhmn.strftime('%H')
    mn = yyyymmddhhmn.strftime('%M')
    template_dir = "MSG/{yyyy}/{mm}/{dd}/"
    relativedir = template_dir.format(yyyy=yyyy, mm=mm, dd=dd)
    template_filename = "H-000-MSG1__-MSG1________-{product}____-CYCLE____-{yyyy}{mm}{dd}{hh}{mn}.raw"
    fullfilename = project_id_env + relativedir + template_filename.format(product=product, yyyy=yyyy, mm=mm, dd=dd, hh=hh, mn=mn)
    print('iii : opening for reading %s' % (fullfilename))

    # opening file
    try:
        fid = open(fullfilename, "rb")
    except IOError, err:
        sys.stderr.write("Error: %r. %s.\n" % (err.filename, err.strerror))
        sys.exit(1)

    # read file, store in a 2D numpy array
    n = 3712

    values1D = numpy.fromfile(fid, dtype=numpy.float32, count=n * n)
    values = values1D.reshape(n, n)

    fid.close()

    return values

# Run main, if called from the command line
if __name__ == '__main__':

    """

    if read_msg_xrit_raw.py is launched from the prompt, test is done.

    """

    import doctest
    doctest.testmod()
