#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

======================
search_amsufiles_py.py
======================

SYNOPSIS
========

DESCRIPTION
===========

.. option:: --help
.. option:: --verbose
.. option:: --numch <NUMCH>

   name of the channel

.. option:: --yyyymmdd <YYYYMMDD>

find files containing AMSU filenames on the given date in ${PROJECT_ID}
:file:`${PROJECT_ID}/AMSU/AMSU${prefix}*/L1C/${yyyy}/${yyyy}_${mm}_${dd}/*.L1C`
or :file:`${PROJECT_ID}/MSG/AMSU${prefix}*/L1C/${yyyy}/${yyyy}_${mm}_${dd}/*.L1C`.

where prefix is the upper case of the first letter of **numch**

AMSUBN17 files will be excluded.

This is the python equivalent of this shell sequence(except for AMSUBN17 exclusion):

.. code-block:: sh

   prefix=$(printf  "%1c" ${numch} | tr [:lower:] [:upper:])
   ls ${PROJECT_ID}/AMSU/AMSU${prefix}*/L1C/${yyyy}/${yyyy}_${mm}_${dd}/*.L1C
   ls ${PROJECT_ID}/MSG/AMSU${prefix}*/L1C/${yyyy}/${yyyy}_${mm}_${dd}/*.L1C

.. only:: man

    Figure is visible on PDF and HTML documents only.

.. only:: html or latex

   .. graphviz::

      digraph search_amsufiles_py {


          search_amsufiles_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/search_amsufiles_py.py",
          label="${PROJECT}/src/search_amsufiles_py.py"]

          {numch yyyymmdd yyyymmdd_min yyyymmdd_max } -> search_amsufiles_py -> listfiles

      }

SEE ALSO
========

:ref:`extract_amsua.pro` to be replace by extract_amsub.py

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest ${PROJECT}/src/search_amsufiles_py.py

Using AMSU-A channel 5:

.. code-block:: sh

   numch='a5'
   yyyymmdd=20060801
   yyyymmdd_min=19991230
   yyyymmdd_max=20150102
   search_amsufiles_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --yyyymmdd_min ${yyyymmdd_min} --yyyymmdd_max ${yyyymmdd_max}
   tlogd.sh search_amsufiles_py_main

Using AMSU-B channel 5:

.. code-block:: sh

   numch='b5'
   yyyymmdd=20060801
   yyyymmdd_min=19991230  # ++
   yyyymmdd_max=20150102  # ++
   search_amsufiles_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --yyyymmdd_min ${yyyymmdd_min} --yyyymmdd_max ${yyyymmdd_max}
   tlogd.sh search_amsufiles_py_main

Same in debug mode:

.. code-block:: sh

   python -m pdb ${PROJECT}/src/search_amsufiles_py.py --numch ${numch} --yyyymmdd ${yyyymmdd} --yyyymmdd_min ${yyyymmdd_min} --yyyymmdd_max ${yyyymmdd_max}


TODO
====

check param

yyyymmdd_min et yyyymmdd_max= f(satellite) ?

tri chrono dans les list_file ou après la production du fichier .dat
par extract_amsua.pro avec sort -n : on a besoin de connaitre la
convention de terminologie utilisée.

complete digraph

EVOLUTIONS
==========

$Id$

$URL$

- pinsard 20141007T091033Z climtest0.ipsl.polytechnique.fr (Linux)

  * AMSUBN17 exclusion
  * fix log

- fplod 20140730T112703Z callisto.locean-ipsl.upmc.fr (Linux)

  * creation from search_amsufiles.pro

"""

__all__ = ["search_amsufiles_py"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

import dateutil.parser
import glob


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """%prog: find files containing AMSU filenames on the given date in ${PROJECT_ID}"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')
    parser.add_option('--version',
                      action='store_true',
                      dest='version',
                      help='show version number and exit')

    parser.add_option('--numch',
                      help='name of the channel',
                      dest='numch',
                      metavar='<NUMCH>',
                      action='store')

    parser.add_option('--yyyymmdd',
                      help='date to be found',
                      dest='yyyymmdd',
                      metavar='<YYYYMMDD>',
                      action='store')

    parser.add_option('--yyyymmdd_min',
                      help='lower date of possible data',
                      dest='yyyymmdd_min',
                      metavar='<YYYYMMDD>',
                      action='store')

    parser.add_option('--yyyymmdd_max',
                      help='upper date of possible data',
                      dest='yyyymmdd_max',
                      metavar='<YYYYMMDD>',
                      action='store')

    return parser


def search_amsufiles_py_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['numch', 'yyyymmdd', 'yyyymmdd_min', 'yyyymmdd_max']
    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    numch = fromcli.numch
    yyyymmdd = dateutil.parser.parse(fromcli.yyyymmdd, ignoretz=True)
    yyyymmdd_min = dateutil.parser.parse(fromcli.yyyymmdd_min, ignoretz=True)
    yyyymmdd_max = dateutil.parser.parse(fromcli.yyyymmdd_max, ignoretz=True)

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()

    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))

    logger.info('[Parameters]')
    logger.info('numch={0}'.format(numch))
    logger.info('yyyymmdd={0}'.format(yyyymmdd))
    logger.info('yyyymmdd_min={0}'.format(yyyymmdd_min))
    logger.info('yyyymmdd_max={0}'.format(yyyymmdd_max))

    status = search_amsufiles_py(numch, yyyymmdd, yyyymmdd_min, yyyymmdd_max, is_verbose)


def build_directory(yyyymmdd, nomcanal, template_dir, is_verbose):
    """
    build relative directory according to a given date and a given dataset

    >>> nomcanal = "A"
    >>> yyyymmdd = dateutil.parser.parse('20060801', ignoretz=True)
    >>> template_dir = "AMSU/AMSU{nomcanal}*/L1C/{yyyy}/{yyyy}_{mm}_{dd}"
    >>> build_directory(yyyymmdd, nomcanal, template_dir, False)
    'AMSU/AMSUA*/L1C/2006/2006_08_01'

    >>> nomcanal = "B"
    >>> yyyymmdd = dateutil.parser.parse('20060801', ignoretz=True)
    >>> template_dir = "AMSU/AMSU{nomcanal}*/L1C/{yyyy}/{yyyy}_{mm}_{dd}"
    >>> build_directory(yyyymmdd, nomcanal, template_dir, False)
    'AMSU/AMSUB*/L1C/2006/2006_08_01'
    """

    logger = logging.getLogger()

    # decompose yyyymmdd
    yyyy = yyyymmdd.strftime('%Y')
    mm = yyyymmdd.strftime('%m')
    dd = yyyymmdd.strftime('%d')

    relativedir = template_dir.format(nomcanal=nomcanal, yyyy=yyyy, mm=mm, dd=dd)

    if is_verbose:
        logger.debug('relativedir {0}'.format(relativedir))

    return relativedir


def search_amsufiles_py(numch, yyyymmdd, yyyymmdd_min, yyyymmdd_max, is_verbose):

    """
    >>> numch = 'a5'
    >>> yyyymmdd = dateutil.parser.parse('20060801', ignoretz=True)
    >>> yyyymmdd_min = dateutil.parser.parse('19991230', ignoretz=True)
    >>> yyyymmdd_max = dateutil.parser.parse('20150102', ignoretz=True)
    >>> files_list = search_amsufiles_py(numch, yyyymmdd, yyyymmdd_min, yyyymmdd_max, False)
    >>> print ('nb de fichiers {0}'.format(len(files_list)))
    nb de fichiers 56

    >>> numch = 'b5'
    >>> yyyymmdd = dateutil.parser.parse('20060801', ignoretz=True)
    >>> yyyymmdd_min = dateutil.parser.parse('19991230', ignoretz=True)
    >>> yyyymmdd_max = dateutil.parser.parse('20150102', ignoretz=True)
    >>> files_list = search_amsufiles_py(numch, yyyymmdd, yyyymmdd_min, yyyymmdd_max, False)
    >>> print ('nb de fichiers {0}'.format(len(files_list)))
    nb de fichiers 28
    """

    result = -1
    logger = logging.getLogger()

    if yyyymmdd_min <= yyyymmdd <= yyyymmdd_max:
        logger.info('yyyymmdd {0} is between {1} and {2}'.format(yyyymmdd, yyyymmdd_min, yyyymmdd_max))
    else:
        logger.error('yyyymmdd {0} must be between {1} and {2}'.format(yyyymmdd, yyyymmdd_min, yyyymmdd_max))
        return False

    project_id_env = os.environ['PROJECT_ID']

    # build the files expression to be searched
    nomcanal_upper = numch.split()[0][0].capitalize()
    #

    filetype_dict = {'AMSU': {'template_dir': "AMSU/AMSU{nomcanal}*/L1C/{yyyy}/{yyyy}_{mm}_{dd}",
                              'template_file': "*.L1C"},
                     'MSG':  {'template_dir': "MSG/AMSU{nomcanal}*/L1C/{yyyy}/{yyyy}_{mm}_{dd}",
                              'template_file': "*.L1C"}}

    files_expr = []

    relativedir = build_directory(yyyymmdd, nomcanal_upper, filetype_dict['AMSU']['template_dir'], is_verbose)
    files_expr.append('{0}/{1}/*.L1C'.format(project_id_env, relativedir))
    relativedir = build_directory(yyyymmdd, nomcanal_upper, filetype_dict['MSG']['template_dir'], is_verbose)
    files_expr.append('{0}/{1}/*.L1C'.format(project_id_env, relativedir))

    # search files
    files_list = []
    for files_expr_one in files_expr:
        files_list = files_list + glob.glob(files_expr_one)

    # exclusion
    pattern = '{0}/AMSU/AMSUBN17'.format(project_id_env)
    files_list_filtered = [file for file in files_list if not file.startswith(pattern)]

    if files_list_filtered:
        logger.info('nb de fichiers {0}'.format(len(files_list_filtered)))
    else:
        logger.info('pas de fichiers')
    #
    if is_verbose:
        logger.debug('files_list {0}'.format(files_list_filtered))

    result = files_list_filtered

    return result

# Run main, if called from the command line
if __name__ == '__main__':
    search_amsufiles_py_main()
