#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

================
simul_amsu_t1.py
================

SYNOPSIS
========

.. code-block:: bash

   simul_amsu_t1.py --index_simulation index_simulation

DESCRIPTION
===========

.. warning::

   this CLI exists only to consolidate all amsu_t1 funtions


If **index_simulation** is set to 1, values will be written : no realistic silly values 1 scan, 1 fov.

If **index_simulation** is set to 2, values will be written : no realistic silly values 1 scan, 3 fov.

EXAMPLES
========

To run doctest:

.. code-block:: bash

   clear; reset; python -m doctest simul_amsu_t1.py

To simulate no realistic silly values 1 scan, 1 fov:

.. code-block:: bash

   index_simulation=1
   simul_amsu_t1.py --index_simulation ${index_simulation}
   tlogd.sh simul_amsu_t1_main

The same in debug

.. code-block:: bash

   python -m pdb ${PROJECT}/src/simul_amsu_t1.py --index_simulation ${index_simulation}

SEE ALSO
========

same output as :func:`read_amsua1c_py`

TODO
====

complete header and datset values (now minimal to try to plot something)

simulate missing values

trouver dans http://nbviewer.ipython.org/github/jrjohansson/scientific-python-lectures/blob/master/Lecture-2-Numpy.ipynb
peut-tre utile pour simuler lon lat d'un coup

.. code-block:: python

   In [18]:
   x, y = mgrid[0:5, 0:5] # similar to meshgrid in MATLAB
   In [19]:
   x
   Out[19]:
   array([[0, 0, 0, 0, 0],
          [1, 1, 1, 1, 1],
          [2, 2, 2, 2, 2],
          [3, 3, 3, 3, 3],
          [4, 4, 4, 4, 4]])
   In [20]:
   y
   Out[20]:
   array([[0, 1, 2, 3, 4],
          [0, 1, 2, 3, 4],
          [0, 1, 2, 3, 4],
          [0, 1, 2, 3, 4],
          [0, 1, 2, 3, 4]])

EVOLUTIONS
==========

$Id: simul_amsu_t1.py$

$URL$

- pinsard 20141002T102538Z climtest0.ipsl.polytechnique.fr (Linux)

  * fix missing scale factor

- pinsard 20141001T123055Z climtest0.ipsl.polytechnique.fr (Linux)

  * creation

"""

__all__ = ["simul_amsu_t1"]
__docformat__ = 'restructuredtext'

import os

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

from geolocation_to_string_py import geolocation_to_string_py

import numpy
import dateutil


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """simul AMSU_T1 dataset"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--index_simulation',
                      help='index of dataset simulation',
                      dest='index_simulation',
                      type="int",
                      metavar='<index_simulation>',
                      action='store')

    return parser


def simul_amsu_t1(index_simulation):

    """

    :returns: amsu_t1 structure if ok or -1 if error++
    :rtype: structure ++ dictinary

    # ++...        'lon': numpy.linspace(-25., +25., endpoint=True, num=nrows),
    # ++...        'lat': numpy.linspace(-12., +45., endpoint=True, num=nrows),

    >>> index_simulation = 1
    >>> amsu_t1 = simul_amsu_t1(index_simulation)
    >>> print('{0}'.format(amsu_t1))
    {'header': {'amb1c_h_instrument': 11, 'amb1c_h_satid': 15}, 'dataset': {'amb1c_btemps': array([[[15000]]], dtype=int32), 'amb1c_latlon': array([[[ 146927, -174466]]], dtype=int32)}}

    """

    status = -1

    logger = logging.getLogger()

    #
    # Dakar Coordinates: 1441#34#N 1726#48#W
    cities = {'Dakar': {'lat': 14.692778, 'lon': -17.446667}}

    if index_simulation == 1:
        logger.info('generation of amsu_t1 unrealistic data : 1 channel 1 scan 1 fov on Dakar')
        header = {'amb1c_h_satid': 15, 'amb1c_h_instrument': 11}
        n_scan = 1
        n_fov = 1
        n_ch = 1
        dataset = {'amb1c_btemps': numpy.zeros((n_scan, n_fov, n_ch), 'Int32'),
                   'amb1c_latlon': numpy.zeros((n_scan, n_fov, 2), 'Int32')}
        isc = 0
        ifov = 0
        dataset['amb1c_latlon'][isc, ifov, :] = (numpy.array(cities['Dakar'].values()) * 1.e4).astype(int)
        ich = 0
        dataset['amb1c_btemps'][isc, ifov, ich] = int(150. * 1.e2)
        amsu_t1 = {'header': header, 'dataset': dataset}

    elif index_simulation == 2:
        logger.info('generation of amsu_t1 unrealistic data 1 channel 1 scan 3 fov centered on Dakar')
        header = {'amb1c_h_satid': 15, 'amb1c_h_instrument': 11}
        n_scan = 1
        n_fov = 3
        n_ch = 1
        dataset = {'amb1c_btemps': numpy.zeros((n_scan, n_fov, n_ch), 'Int32'),
                   'amb1c_latlon': numpy.zeros((n_scan, n_fov, 2), 'Int32')}
        isc = 0
        ifov = 1
        dataset['amb1c_latlon'][isc, ifov, :] = (numpy.array(cities['Dakar'].values()) * 1.e4).astype(int)
        dataset['amb1c_latlon'][isc, ifov - 1, :] = dataset['amb1c_latlon'][isc, ifov, :] - (5.e4, 5.e4)
        dataset['amb1c_latlon'][isc, ifov + 1, :] = dataset['amb1c_latlon'][isc, ifov, :] + (5.e4, 5.e4)
        ich = 0
        dataset['amb1c_btemps'][isc, ifov, ich] = int(150. * 1.e2)
        dataset['amb1c_btemps'][isc, ifov - 1, ich] = dataset['amb1c_btemps'][isc, ifov, ich] - 10 * 1.e2
        dataset['amb1c_btemps'][isc, ifov + 1, ich] = dataset['amb1c_btemps'][isc, ifov, ich] + 10 * 1.e2
        amsu_t1 = {'header': header, 'dataset': dataset}
        logger.info(amsu_t1)

    elif index_simulation == 3:
        logger.info('generation of amsu_t1 unrealistic data 1 channel 1 scan 3 fov centered on Dakar with missing values')
        header = {'amb1c_h_satid': 15, 'amb1c_h_instrument': 11}
        n_scan = 1
        n_fov = 3
        n_ch = 1
        dataset = {'amb1c_btemps': numpy.zeros((n_scan, n_fov, n_ch), 'Int32'),
                   'amb1c_latlon': numpy.zeros((n_scan, n_fov, 2), 'Int32')}
        isc = 0
        ifov = 1
        dataset['amb1c_latlon'][isc, ifov, :] = (numpy.array(cities['Dakar'].values()) * 1.e4).astype(int)
        dataset['amb1c_latlon'][isc, ifov - 1, :] = dataset['amb1c_latlon'][isc, ifov, :] - (5.e4, 5.e4)
        dataset['amb1c_latlon'][isc, ifov + 1, :] = dataset['amb1c_latlon'][isc, ifov, :] + (5.e4, 5.e4)
        ich = 0
        dataset['amb1c_btemps'][isc, ifov, ich] = int(150. * 1.e2)
        dataset['amb1c_btemps'][isc, ifov - 1, ich] = dataset['amb1c_btemps'][isc, ifov, ich] - 10 * 1.e2
        dataset['amb1c_btemps'][isc, ifov + 1, ich] = dataset['amb1c_btemps'][isc, ifov, ich] + 10 * 1.e2
        dataset['amb1c_btemps'][isc, 0, ich] = -999999

        amsu_t1 = {'header': header, 'dataset': dataset}
        logger.info(amsu_t1)

    else:
        msg = 'index_simulation {index_simulation} unknown'.format(index_simulation=index_simulation)
        logger.error(msg)
        raise ValueError(msg)
    result = amsu_t1

    return result


def simul_amsu_t1_main():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['index_simulation']

    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    index_simulation = fromcli.index_simulation

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)
    if is_verbose:
        logging.basicConfig(filename=log, level=logging.DEBUG)
    else:
        logging.basicConfig(filename=log, level=logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))
    logger.info('[Parameters]')
    logger.info('index_simulation={0}'.format(index_simulation))

    try:
        amsu_t1 = simul_amsu_t1(index_simulation)
    except ValueError:
        logger.error('pb simul_amsu_t1')
        raise
    else:
        logger.info('simul_amsu_t1 reach end')

# Run main, if called from the command line
if __name__ == '__main__':
    simul_amsu_t1_main()
