#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
===================
traite_amsuab_py.py
===================

SYNOPSIS
========

.. code-block:: bash

   traite_amsuab_py.py --begin yyyymmdd --end yyyymmdd --lonmin lonmin --lonmax lonmax --latmin latmin --latmax latmax --numch numch

DESCRIPTION
===========

+todo+

.. option:: --begin beginning date <yyyymmdd>
.. option:: --end   end date <yyyymmdd>
.. option:: --lonmin <lonmin>
.. option:: --lonmax <lonmax>
.. option:: --latmin <latmin>
.. option:: --latmax <latmax>
.. option:: --numch <channel>

``traite_amsuab_py.`` read :file:`${PROJECT_ID}/AMSU/AMSU*/*/*.L1C``
and :file:`${PROJECT}/SRC/dataref_amsu/COR*.dat`++

Log file is written on
:file:`${PROJECT_LOG}/traite_amsuab_py.log.{YYYYMMDDTHHMMSSZ}`

.. only:: man

   Figure is visible on PDF and HTML documents only.

.. only:: html or latex

    .. graphviz::

       digraph traite_amsuab_py {

          amsu_t1 [shape=ellipse,
          fontname=Courier,
          label="${PROJECT_ID}/AMSU/AMSU*/L1C/yyyy/yyyy_mm_dd/*.L1C"];

          ++amsu_a_cor_land [shape=diamond,
          fontname=Courier,
          label="${PROJECT}/src/dataref_amsu/CORR_LAND_AMSUA5_DataFromJune2010_40deg.DAT"];
          ++amsu_a_cor_sea [shape=diamond,
           fontname=Courier,
          label="${PROJECT}/src/dataref_amsu/CORR_SEA_AMSUA5_DataFromJune2010_40deg.DA"];
          amsu_t2 [shape=ellipse,
          fontname=Courier,
          label="${PROJECT_ID}/yyyy/mm/amsua_amsub_yyyymmdd_geomin_geomax.dat"];
          amsu_t3 [shape=ellipse,
          fontname=Courier,
          label="${PROJECT_ID}/yyyy/mm/amsua_amsub_yyyymmdd_geomin_geomax_nadir.dat"];
          amsu_t4 [shape=ellipse,
          fontname=Courier,
          label="${PROJECT_ID}/yyyy/mm/amsua_amsub_yyyymmdd_geomin_geomax_nadir.nc"];
          mask [shape=ellipse,
          fontname=Courier,
          label="${PROJECT_ID}/MASK/ETOPO1_Ice_g_gmt4.nc"];

          log [shape=ellipse,
          fontname=Courier,
          label="${P}/traite_amsuab.log{YYYYMMDDTHHMMSSZ}"];

          traite_amsuab_py [shape=box,
          fontname=Courier,
          color=blue,
          URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/traite_amsuab_py.py",
          label="${PROJECT}/src/traite_amsuab_py.py"];

          {amsu_t1 amsu_a_cor_land amsu_a_cor_sea mask} -> {traite_amsuab} -> {log amsu_t2 amsu_t3 amsu_t4}

         }

EXAMPLES
========

You already used :ref:`get_amsu.sh` to get AMSU data including channel a5
during [20060801, 20060802[ and you want to produce NetCDF files:

.. code-block:: bash

   traite_amsuab_py.py --begin 20060801 --end 20060802 --lonmin -25 --lonmax 25 --latmin -5 --latmax 20 --numch a5

And look at log file with :

.. code-block:: bash

   tlogd.sh traite_amsuab_py

A more realistic example:

.. code-block:: bash

   traite_amsuab_py.py --begin 20060101 --end 20060630 --lonmin -32 --lonmax 32 --latmin -2 --latmax 42 --numch b3

and of course on files in ${PROJECT_ID}.

If you want to work on channel b5 of AMSU:

.. code-block:: bash

   traite_amsuab_py.py --begin 20060813 --end 20060814 --lonmin -25 --lonmax 25 --latmin -5 --latmax 20 --numch b5

SEE ALSO
========

:ref:`varamma_profile.sh`

:ref:`run_traite_amsuab.sh` to submit job++

Previous step : :ref:`get_amsu.sh`, ++ get correcb files

Use :
:func:`extract_amsu_py`
++:ref:`amsu2ncdf.pro`
:func:`search_amsufiles_py`

:func:`pixelsize_py`
:func:`geolocation_to_string_py.py`
:func:`file_correcb_to_mem_py`

TODO
====

finish shell and idl migration

trop de log (mieux que pas de log) : stdout + logfile (idem extract_amsu_py.py)

check correction values given to extract_amsu : I am not sure to give the right (sat, date)
couple because more than one sat possible during on day !!

EVOLUTIONS
==========

$Id$

$URL$

- pinsard 20141020T125240Z climtest0.ipsl.polytechnique.fr (Linux)

  * some fix for correction

- pinsard 20141007T140301Z loholt1.ipsl.polytechnique.fr (Linux)

  * some improvement for correction
  * remove hardcoded nomsat and nomcanal and yyyymmdd values

- pinsard 20140827T105543Z climtest0.ipsl.polytechnique.fr (Linux)

  * add usage of pixelsize and file_correcb_to_mem_py (but not yet ok)

- pinsard 20140731T125930Z climtest0.ipsl.polytechnique.fr (Linux)

  * creation from traite_amsuab.sh

"""

__all__ = ["traite_amsuab_py"]
__docformat__ = 'restructuredtext'

import os

import re

from optparse import OptionParser
import inspect
from datetime import datetime
import logging

from dateutil.rrule import rrule, DAILY
import dateutil.parser

from collections import OrderedDict

from search_amsufiles_py import search_amsufiles_py
from extract_amsu_py import extract_amsu_py
from file_correcb_to_mem_py import file_correcb_to_mem_py
from pixelsize_py import pixelsize_py


def get_option_parser():
    """parse CLI arguments

       :returns: parser
       :rtype: :class:`optparse.OptionParser`

    """
    desc = """process AMSU files"""

    parser = OptionParser(description=desc)

    parser.add_option('-V', '--verbose',
                      help='produce verbose output',
                      action='store_true',
                      default=False,
                      dest='is_verbose')

    parser.add_option('--version',
                      action='store_true',
                      dest='version',
                      help='show version number and exit')

    parser.add_option('-f', '--force',
                      help='force++',
                      action='store_true',
                      default=False,
                      dest='force')

    parser.add_option('-b', '--begin',
                      help='date (yyyymmdd) of the first date',
                      dest='yyyymmddb',
                      metavar='<BEGIN>',
                      action='store')

    parser.add_option('-e', '--end',
                      help='date (yyyymmdd) of the last date',
                      dest='yyyymmdde',
                      metavar='<END>',
                      action='store')

    parser.add_option('--lonmin',
                      help='min longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmin')

    parser.add_option('--lonmax',
                      help='max longitude, W < 0',
                      type='float',
                      default=None,
                      metavar='<LON>',
                      dest='lonmax')

    parser.add_option('--latmin',
                      help='min latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmin')

    parser.add_option('--latmax',
                      help='max latitude, N > 0',
                      type='float',
                      default=None,
                      metavar='<LAT>',
                      dest='latmax')

    parser.add_option('--numch',
                      help='name of the channel',
                      dest='numch',
                      metavar='<NUMCH>',
                      action='store')

    return parser


def traite_amsuab_py():
    """main

    """
    try:
        parser = get_option_parser()
        (fromcli, args) = parser.parse_args()
    except IOError, msg:
        parser.error(str(msg))

    # Making sure all mandatory options appeared.
    mandatories = ['numch', 'yyyymmddb', 'yyyymmdde',
                            'lonmin', 'lonmax',
                            'latmin', 'latmax']
    for m in mandatories:
        if not fromcli.__dict__[m]:
            logging.error('mandatory option {0} is missing'.format(m))
            parser.print_help()
            exit(-1)

    is_verbose = fromcli.is_verbose
    numch = fromcli.numch
    yyyymmddb = dateutil.parser.parse(fromcli.yyyymmddb, ignoretz=True)
    yyyymmdde = dateutil.parser.parse(fromcli.yyyymmdde, ignoretz=True)
    lonmin = fromcli.lonmin
    lonmax = fromcli.lonmax
    latmin = fromcli.latmin
    latmax = fromcli.latmax
    #
    # !! check consistency of those lines with equivalent in project_init.pro
    # !! when change ++
    yyyymmdd_min = dateutil.parser.parse('19991230', ignoretz=True)
    yyyymmdd_max = dateutil.parser.parse('20150102', ignoretz=True)

    netcdf_build = 1  # ++ parameter

    if yyyymmdd_min <= yyyymmddb <= yyyymmdd_max:
        logging.info('yyyymmddb is between {1} and {2}'.format(yyyymmddb, yyyymmdd_min, yyyymmdd_max))
    else:
        logging.error('yyyymmddb {0} must be between {1} and {2}'.format(yyyymmddb, yyyymmdd_min, yyyymmdd_max))
        parser.print_help()
        return False

    if yyyymmdd_min <= yyyymmdde <= yyyymmdd_max:
        logging.info('yyyymmdde is between {1} and {2}'.format(yyyymmdde, yyyymmdd_min, yyyymmdd_max))
    else:
        logging.error('yyyymmdde {0} must be between {1} and {2}'.format(yyyymmdde, yyyymmdd_min, yyyymmdd_max))
        parser.print_help()
        return False

    if yyyymmdde > yyyymmddb:
        logging.info('yyyymmdde {0} is after {1}'.format(yyyymmdde, yyyymmddb))
    else:
        logging.error('yyyymmdde {0} must be after {1}'.format(yyyymmdde, yyyymmddb))
        parser.print_help()
        return False

    # define log file
    command = '{0}'.format(inspect.stack()[0][3])
    log_date = datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    log = '{0}/{1}.log.{2}'.format(os.environ['PROJECT_LOG'], command, log_date)

    logger = logging.getLogger()
    handler = logging.FileHandler(log)
    logger.addHandler(handler)
    if is_verbose:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    logger = logging.getLogger()
    logger.info('[Context]')
    logger.info('PID={0}'.format(os.getpid()))
    logger.info('command={0}'.format(command))
    import socket
    logger.info('hostname={0}'.format(socket.getfqdn()))
    logger.info('runtime={0}'.format(log_date))
    logger.info('log={0}'.format(log))

    logger.info('[Parameters]')
    logger.info('numch={0}'.format(numch))
    logger.info('yyyymmddb={0}'.format(yyyymmddb))
    logger.info('yyyymmdde={0}'.format(yyyymmdde))
    logger.info('lonmin={0}'.format(lonmin))
    logger.info('lonmax={0}'.format(lonmax))
    logger.info('latmin={0}'.format(latmin))
    logger.info('latmax={0}'.format(latmax))

    dict_sat = {'AMSUBN16': {'nomsat': 'N16_AMSUB'},
                'AMSUBN15': {'nomsat': 'N15_AMSUB'}}

    nocanal = int(numch.split()[0][1])

    resol = 1
    # une boucle pour chaque jour
    list_dt = rrule(DAILY, dtstart=yyyymmddb, until=yyyymmdde)
    for dt in list_dt:
        logger.info('traitement du jour {0}'.format(dt))
        try:
            result_search_amsufiles = search_amsufiles_py(numch, dt, yyyymmdd_min, yyyymmdd_max, is_verbose)
        except:
            logger.error('pb search_amsufiles {0}'.format(dt))
            raise
        else:
            files_list = result_search_amsufiles
            #files_list = result_search_amsufiles[0:3]  # ++ limitation pour mise au point
            logger.info('files found for {0} : {1}'.format(dt, files_list))

        # build the list of satellites involved
        project_id_env = os.environ['PROJECT_ID']
        tmplist = [re.sub('{0}/AMSU/'.format(project_id_env), '', file) for file in files_list]
        tmp2list = [re.sub('/L1C.*$','', tmpitem) for tmpitem in tmplist]

        nomdirsat_list = list(OrderedDict.fromkeys(tmp2list))
        logger.info('satellites involved for {0} : {1}'.format(dt, nomdirsat_list))
        for nomdirsat in nomdirsat_list:
            nomsat = dict_sat[nomdirsat]['nomsat']
            # read only in result_file_correcb['nomsat': 'N16_AMSUB', 'yyyymmdd': dt] not
            # yet present
            # ++ test to be improved because not all dt are present
            # in correcb files may be add doy asked: 225 nearest found: 226.0 2006-08-14 00:00:00 in result
            try:
                result_file_correcb[nomsat]
            except NameError:
                try:
                    resultonesat_file_correcb = file_correcb_to_mem_py(nomsat, nocanal, dt, lonmin, lonmax, latmin, latmax)
                except:
                    logger.error('pb file_correcb_to_mem_py nomdirsat {0} nomsat {1}'.format(nomdirsat, nomsat))
                    raise
                else:
                    result_file_correcb = {nomsat: resultonesat_file_correcb}
                    logger.info('correction from nomsat {0} date {1} (first satellite call)'.format(result_file_correcb[nomsat]['nomsat'], result_file_correcb[nomsat]['yyyymmdd']))
            except KeyError:
                try:
                    resultonesat_file_correcb = file_correcb_to_mem_py(nomsat, nocanal, dt, lonmin, lonmax, latmin, latmax)
                except:
                    logger.error('pb file_correcb_to_mem_py nomdirsat {0} nomsat {1}'.format(nomdirsat, nomsat))
                    raise
                else:
                    result_file_correcb[nomsat] = resultonesat_file_correcb
                    logger.info('correction from nomsat {0} date {1} (new satellite call)'.format(result_file_correcb[nomsat]['nomsat'], result_file_correcb[nomsat]['yyyymmdd']))
            else:
                if not ('yyyymmdd', dt) and ('nomsat', nomsat) in result_file_correcb[nomsat].items():
                    try:
                        resultonesat_file_correcb = file_correcb_to_mem_py(nomsat, nocanal, dt, lonmin, lonmax, latmin, latmax)
                    except:
                        logger.error('pb file_correcb_to_mem_py nomdirsat {0} nomsat {1}'.format(nomdirsat, nomsat))
                        raise
                    else:
                        result_file_correcb = {nomsat: resultonesat_file_correcb}  # ++ autre jour
                        logger.info('correction from nomsat {0} date {1} (next call)'.format(result_file_correcb[nomsat]['nomsat'], result_file_correcb[nomsat]['yyyymmdd']))
                else:
                    logger.info('correction unchanged')

        # appel pixelsize pour avoir les dimensions des pixels dans la fauch~C©e
        (pixatot, pixbtot, alongatot, alongbtot) = pixelsize_py()
        swath = pixbtot
        try:
            result = extract_amsu_py(numch, files_list, dt, resol, lonmin, lonmax, latmin, latmax, result_file_correcb, pixbtot)
        except:
            logger.error('pb extract_amsu_py')
            raise
        else:
            logger.info('extract_amsu_py ok')

        # ++ migrationif [ ${netcdf_build} -eq 1 ]
        # ++ migration    then
        # ++ migration    cat >> ${PROJECT}/src/${randomnumber}_temp.pro << EOF
        # ++ migration    resol_grid = 0.1
        # ++ migration    amsu2ncdf, '${numch}', yyyy,mm,dd, ${lonmin}, ${lonmax}, ${latmin}, ${latmax}, resol_grid
        # ++ migration; compile users procedures to be scan by Profiler
        # ++ migration.compile interpol_correc
        # ++ migration.compile extract_amsu
        # ++ migration.compile mem_to_file_amsu_t2

# Run main, if called from the command line
if __name__ == '__main__':
    traite_amsuab_py()
