function [result] = visuMSGsystconv(TbMSG, Temps, Longitudes, Latitudes, timestep,Tb_SeuilSup)

%visuMSGsystconv permet de faire une image des données de températures de
%brillances
%pour les systèmes convectifs potentiels dont la température est
%inférieure, en tout point, à la valeur
%seuil définie.

%
%+
%
% =================
% visuMSGsystconv.m
% =================
%
% .. function:: visuMSGsystconv(Annee, Mois, Jour, timestep, fix(min(Longitudes)), ceil(max(Longitudes)), fix(min(Latitudes)), ceil(max(Latitudes)),Tb_SeuilSup)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * TbMSG : Données de température de brillance d'après MSG dans une
%     matrice de dimensions temps, longitudes, latitudes.
%   * Temps : Données de la dimension temporelle de la matrice en jour
%     julien (première dimension).
%   * Longitudes : Données de la première dimension spatiale de la matrice
%     (seconde dimension).
%   * Latitudes : Données de la seconde dimension spatiale de la matrice
%     (troisième dimension).
%   * timestep : Numéro du quart d'heure correspondant (de 1 pour 00:00 à
%     96 pour 23:45 pour des données complètes).
%   * Tb_SeuilSup : Seuil de température de brillance maximale délimitant
%     un système convectif.
%
% EXAMPLES
% ========
%
% Voir "demo".
%
% SEE ALSO
% ========
%
% :ref:`data_msg`
% :ref:`world_map`
%
% :func:`MSGbuildfullfilename`
% :func:`MSGread`
%
% :func:`extractedmatrixdata``
%
% :func:`ticklatlon`
%
% TODO
% ====
%
% coast dots are plot outside the frame on ClimServ and cratos matlab 7.4
%
% timestep vs extractedmartixdata : not the same unit but equivalent with
% mindim1,maxdim1
%
% no obvious difference between normal and extracted MSG figures.
%
% save figure
%
% demo with simulation
%
% trouver l'équivalent de 'alphadata'
%
% handling error
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/visuMSGsystconv.m $
%
% - fplod 20110830T074359Z cratos (Linux)
%
%   * suppress interactivity
%
% - fplod 20110804T104210Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add demo with MSG real file normal
%   * minimize decoration (font, label)
%   * add result
%
% - jaclod 2011-07-28
%
%   * Fonctionne sur le Matlab de cratos mais les côtes dépassent.
%
% - jaclod 2011-07-24
%
%   * Externalisation de la lecture et modification des arguments d'entrée.
%   * Modification en conséquence de la demo.
%
% - pinsard 2011-06-08T13:48:24Z loholt1.ipsl.polytechnique.fr (Linux)
%
%   * add box parameter
%
% - pinsard 2011-05-26T07:52:18Z loholt1.ipsl.polytechnique.fr
%
%   * suppress full screen
%   * image is a function (reserve word)
%
% - fplod 20110520T133138Z aedon.locean-ipsl.upmc.fr
%
%   * replace hardcoded clip values by pseudo parameter lat/lon min/max
%   * world map cmgco_traitdecote.mat (need varamma_startup)
%
% - fplod 20110520T110336Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * use MSGread
%
% - jaclod 2011-04-10
%
%   * Création
%
%-
global application
%
result=-1;
usage='[result] = visuMSGsystconv(TbMSG, Temps, Longitudes, Latitudes, timestep,Tb_SeuilSup)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% Récupération des données concernées (On transpose pour avoir latitudes,
% longitudes comme dimension pour 'data', en vue de l'imagesc plus loin).
data = transpose(squeeze(TbMSG(timestep,:,:)));

% On ne garde que les systèmes convectifs potentiels (dont les températures
% de brillance sont inférieurs au seuil définie).
data(data >= Tb_SeuilSup)=NaN;

%%%%%%%%%
% Figure
%%%%%%%%%

% Ouverture d'une figure
figure();

% Modification des bornes pour les axes.
axis([fix(min(Longitudes)) ceil(max(Longitudes)) fix(min(Latitudes)) ceil(max(Latitudes))]);

% Modification de la direction de l'axe Y (Latitudes).
set(gca,'Ydir','normal');

% Chargement des données côtières.
load('cmgco_traitdecote.mat','Xgco','Ygco');

% Ajout d'une barre de couleur, bornée entre le minimum et le maximum des
% températures de brillances.
colorbar
caxis([min(data(:)) max(data(:))]);

hold on;

% Normalisation de la taille de la figure pour obtenir un carré.
set(gca, 'DataAspectRatio', [1 1 1]);

% Passage du temps en jour julien (initialement en heures depuis le 1er
% janvier 1960).
Temps = Temps/24 + datenum(1960,01,01);

% Ajout d'un titre.
title(['Temperatures de brillance (K) d''apres MSG pour le ' datestr(Temps(timestep), 'dd mm yyyy a HH:MM')]);

% Aperçu des données de température de brillance pour les systèmes convectifs.
my_image=imagesc(Longitudes,Latitudes,data);
switch application
  case {'matlab'}
     set(my_image,'alphadata',~isnan(data));
  case {'octave'}
     disp(['www : no alphadata available using ', application]);
  otherwise
     error('eee : unknown application running');
end

% Ajout des côtes.
plot(Xgco,Ygco,'k.');

status_ticklatlon=ticklatlon();

% Ajout de titres pour les axes.
xlabel('Longitudes')
ylabel('Latitudes')

result=0;

end

%!demo
%! % From real file :file:`${PROJECT_ID}MSG/msg-tb108_2006-08-01_15min.nc` ::
%! clear all
%! close all
%! varamma_startup
%! more off
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! ncfilesset = 'normal';
%! [fullfilename] = MSGbuildfullfilename(ncfilesset, Annee, Mois, Jour);
%! [TbMSG, Temps, Longitudes, Latitudes] = MSGread(ncfilesset, fullfilename);
%! mindim1=min(Temps);
%! maxdim1=max(Temps);
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! [TbMSG_e, Temps_e, Longitudes_e, Latitudes_e] = extractedmatrixdata(TbMSG, Temps, Longitudes, Latitudes, mindim1, maxdim1, lonmin, lonmax, latmin, latmax);
%! timestep = 75;
%! Tb_SeuilSup = 235;
%! result = visuMSGsystconv(TbMSG_e, Temps_e, Longitudes_e, Latitudes_e, timestep, Tb_SeuilSup)

%!demo
%! clear all
%! close all
%! varamma_startup;
%! more off
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! [fullfilename] = MSGbuildfullfilename('extracted', Annee, Mois, Jour);
%! [TbMSG, Temps, Longitudes, Latitudes] = MSGread('extracted', fullfilename);
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! [TbMSG_e, Temps_e, Longitudes_e, Latitudes_e] = extractedmatrixdata(TbMSG, Temps, Longitudes, Latitudes, Temps(1), Temps(size(Temps,1)), lonmin, lonmax, latmin, latmax);
%! timestep = 75;
%! Tb_SeuilSup = 235;
%! result = visuMSGsystconv(TbMSG_e, Temps_e, Longitudes_e, Latitudes_e, timestep, Tb_SeuilSup)
