function [result] = visumodele2(TbMSG, Longitudes, Latitudes, Tb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)

%VISUMODELE2 permet de visualiser le résultat de l'analyse des systèmes
%convectifs avec le modèle "2 couches", à savoir les intensités de
%l'activité convective pour une journée sur le domaine d'étude donné

%
%+
%
% =============
% visumodele2.m
% =============
%
% .. function:: visumodele2(TbMSG, Longitudes, Latitudes, Tb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)
%
% DESCRIPTION
% ===========
%
% - Données d'entrées :
%
%   * TbMSG : Données de température de brillance d'après MSG dans une
%     matrice de dimensions temps, longitudes, latitudes.
%   * Temps : Données de la dimension temporelle de la matrice en jour
%     julien (première dimension).
%   * Longitudes : Données de la première dimension spatiale de la matrice
%     (seconde dimension).
%   * Latitudes : Données de la seconde dimension spatiale de la matrice
%     (troisième dimension).
%   * Tb_SeuilSup : Seuil de température de brillance maximale délimitant
%     un système convectif (en K).
%   * Tb_LimNoyau : Seuil de température de brillance maximale délimitant
%     le noyau convectif.
%   * Tps_SeuilInf : Seuil de temps pour définir un système convectif (en
%     nombre de quart d'heure).
%
% EXAMPLES
% ========
%
% Voir "demo".
%
% SEE ALSO
% ========
%
% :ref:`data_msg`
% :ref:`world_map`
%
% :func:`MSGread`
%
% TODO
% ====
%
% add demo simul
%
% describe demo
%
% make it work when matlab is running : now 20110704::
%
%   Warning: Divide by zero.
%
% coast dots are plot outside the frame
%
% save figure
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/visumodele2.m $
%
% - fplod 20110805T104442Z cratos (Linux)
%
%   * when used as a substitution on bwlabeln, bwlabel must be followed by a reshape
%   * statistics package must be installed in octave to provide nansum
%
% - fplod 20110804T140543Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add result
%
% - jaclod 2011-07-28
%
%   * Externalisation de la lecture et modification des arguments d'entrée.
%   * Modification en conséquence de la demo.
%   * Fonctionne sur Matlab cratos.
%
% - fplod 20110704T104826Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * bwlabel from local (fplod) image package when octave running
%   * DetectionNuage dim1 time
%
% - pinsard 2011-06-08T15:36:59Z loholt1.ipsl.polytechnique.fr (Linux)
%
%   * add box parameter
%
% - fplod 20110520T133138Z aedon.locean-ipsl.upmc.fr
%
%   * add pseudo parameters  lat/lon min/max
%   * replace hardcoded clip values by pseudo parameter lat/lon min/max
%   * world map cmgco_traitdecote.mat (need varamma_startup)
%
% - jaclod 2011-04-13
%
%   * Creation.
%
%-
%
global application;
%
result=-1;
usage='[result] = visumodele2(TbMSG, Longitudes, Latitudes, Tb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% dimension 1 : temps, dimension 2 : lon,  dimension 3 : lat
idim_time=1;
idim_lon=2;
idim_lat=3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Labellisation des nuages
%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch application
   case {'matlab'}
      DetectionNuage = bwlabeln(TbMSG <= Tb_SeuilSup);
   case {'octave'}
      disp(['www : no bwlabeln available using ', application]);
      disp(['www : bwlabel used instead']);
      DetectionNuage_2D = bwlabel(TbMSG <= Tb_SeuilSup);
      DetectionNuage=reshape(DetectionNuage_2D,size(TbMSG,1),size(TbMSG,2),size(TbMSG,3));
      clear DetectionNuage_2D

  otherwise
      error('unknown application running');
end

numax = max(DetectionNuage(:));

% Boucle sur les nuages, pour éliminer les nuages trop brefs.
for num_nuage = 1 : numax;

    % On cherche à quel quart d'heure le nuage apparait pour la première
    % fois.
    timestep = 1;
    while size(find(DetectionNuage(timestep,:,:) == num_nuage), 1) == 0 && timestep ~= size(TbMSG,idim_time)-(Tps_SeuilInf-1);
        timestep = timestep + 1;
    end

    % On vérifie que le nuage est présent, depuis le temps o� il
    % apparait jusqu'à atteindre le seuil définit précédemment.
    Elimination = 0;
    timestepSup = 0;
    while timestepSup ~= Tps_SeuilInf-1 && Elimination == 0;
        % Si parmi les quarts d'heure o� le nuage doit être présent, il
        % n'est pas détecté, alors il ne satisfait pas le critère de
        % temps et n'est plus pris en compte.
        if size(find(DetectionNuage(timestep+timestepSup,:,:) == num_nuage), 1) == 0;
            Elimination = 1;
            DetectionNuage(DetectionNuage == num_nuage) = 0;
        end

        timestepSup = timestepSup + 1;

    end

end

% On labellise une nouvelle fois.
switch application
     case {'matlab'}
        DetectionNuage = bwlabeln(DetectionNuage);
     case {'octave'}
        disp(['www : no bwlabeln available using ', application]);
        disp(['www : bwlabel used instead']);
        DetectionNuage_logical=logical(DetectionNuage);
        DetectionNuage_2D = bwlabel(DetectionNuage_logical,8);
        DetectionNuage=reshape(DetectionNuage_2D,size(DetectionNuage_logical,1),size(DetectionNuage_logical,2),size(DetectionNuage_logical,3));
        clear DetectionNuage_2D
     otherwise
        error('unknown application running');
end

numax = max(DetectionNuage(:));

%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modèle de nuages à deux couches
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Création d'une matrice qui contiendra les valeurs des Tb dans ce modèle.
Tb2couches = zeros(size(TbMSG));

% Boucle sur les nuages.
for num_nuage = 1 : numax;

    % On garde la Tb moyenne du noyau dans sa zone.
    ZoneNoyau = find(TbMSG <= Tb_LimNoyau & DetectionNuage == num_nuage);
    Tb2couches(ZoneNoyau) = mean(TbMSG(ZoneNoyau));

    % De même pour la zone extérieure.
    ZoneExt = find(TbMSG > Tb_LimNoyau & TbMSG <= Tb_SeuilSup & DetectionNuage == num_nuage);
    Tb2couches(ZoneExt) = mean(TbMSG(ZoneExt));

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%
% Intensités
%%%%%%%%%%%%%

Tb2couches(Tb2couches==0)=NaN;

% On définit l'intensité pour chaque point comme la somme de la différence
% entre la température de brillance de référence (Tb_SeuilSup, valeur seuil
% du nuage) et la température de brillance du nuage à chaque pas de temps.
Intensity = Tb_SeuilSup - Tb2couches;
ij_intensity = nansum(Intensity,idim_time);

%%%%%%%%%%%%

%%%%%%%%%
% Figure
%%%%%%%%%

ij_intensity(ij_intensity==0)=NaN;

% Chargement des données côtières.
load('cmgco_traitdecote.mat','Xgco','Ygco');

% Ouverture d'une figure
ifigure=gcf();
figure(ifigure);
set(gca, 'DataAspectRatio', [1 1 1]);

hold on

% Modification des bornes sur les axes.
axis([fix(min(Longitudes)) ceil(max(Longitudes)) fix(min(Latitudes)) ceil(max(Latitudes))])

% Visualisation des données (avec transparence là où il n'y a pas eu de
% système détecté).
ij_intensity = squeeze(ij_intensity);
image = imagesc(Longitudes,Latitudes,ij_intensity');
set(image,'alphadata',~isnan(ij_intensity'));

% Ajout des côtes.
coast=plot(Xgco,Ygco,'k.');

% Ajout de titres et modifications sur la figure.
title('(b)','FontSize',18,'FontName','Times');
set(gca,'Xtick',fix(min(Longitudes)):2:ceil(max(Longitudes)));
set(gca,'Ytick',fix(min(Latitudes)):2:ceil(max(Latitudes)));
xlabel('Longitudes','FontSize',18,'FontName','Times')
ylabel('Latitudes','FontSize',18,'FontName','Times')
set(gca,'FontSize',16,'FontName','Times');
set(gca,'TickDir','in');
set(gca,'Box','on');

% Ajout d'un cadre.
set(gca,'Box','on');

% Ajout d'une barre de couleur.
colorbar

result=0;

end

%!demo
%! clear all
%! close all
%! varamma_startup;
%! more off
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! latmin = 10;
%! latmax = 20;
%! lonmin = -20;
%! lonmax = -10;
%! Tb_SeuilSup = 235;
%! Tb_LimNoyau = 210;
%! Tps_SeuilInf = 3;
%! [fullfilename] = MSGbuildfullfilename('extracted', Annee, Mois, Jour);
%! [TbMSG, Temps, Longitudes, Latitudes] = MSGread('extracted', fullfilename);
%! [TbMSG, Temps, Longitudes, Latitudes] = extractedmatrixdata(TbMSG, Temps, Longitudes, Latitudes, Temps(1), Temps(size(Temps,1)), lonmin, lonmax, latmin, latmax);
%! result = visumodele2(TbMSG, Longitudes, Latitudes, Tb_SeuilSup, Tb_LimNoyau, Tps_SeuilInf);
