#!/usr/bin/env python
# -*- coding: utf-8 -*-

__docformat__ = 'reStructuredText'
__autodoc__ = ['write_msg_xrit_nc']

import os
import sys

from datetime import datetime

import time
import numpy
from netCDF4 import Dataset
from netCDF4 import num2date, date2num


def write_msg_xrit_nc(values, product, yyyy, mm, dd, hh, mn):

    """

    =================
    write_msg_xrit_nc
    =================

    DESCRIPTION
    ===========

    Write in a file :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/H-000-MSG1__-MSG1________-{product}____-CYCLE____-{yyyy}{mm}{dd}{hh}{mn}.write_msg_xrit_nc.nc`

    Parameters
    ----------

    values :
        values

    yyyy : yyyy
        year

    Returns
    -------

    result : int
       1 if ok

    Raises
    ------

    Missing parameters

    EXAMPLES
    ========

    From scratch to write some raw simulated data in
    :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_nc.nc`

    >>> from simul_msg_xrit_raw import simul_msg_xrit_raw
    >>> index_simulation = 1
    >>> (product, values, missing_value) = simul_msg_xrit_raw(index_simulation)
    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> result = write_msg_xrit_nc(values, product, yyyy, mm, dd, hh, mn)
    iii : opening for writing /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_nc.nc

    From real file :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw`

    >>> from read_msg_xrit_raw import read_msg_xrit_raw
    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> product = 'IR108'
    >>> values = read_msg_xrit_raw(product, yyyy, mm, dd, hh, mn)
    iii : opening for reading /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw
    >>> result = write_msg_xrit_nc(values, product, yyyy, mm, dd, hh, mn)
    iii : opening for writing /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_nc.nc

    SEE ALSO
    ========

    :ref:`msg_data`

    :func:`simul_msg_xrit_raw`
    :func:`read_msg_xrit_raw`

    :ref:`create_nc.jnl`
    :ref:`create_nc.sh`
    for structure (dimension, variables), missing values, attributes, etc. of the NetCDF files

    TODO
    ====

    scale factor /offset::

        values[0,0] -999.000000
        values[0,0]*inv_scale_factor = -999.000000*100.000002 = -99900.002233
        data[0,0,0] implicit conversion -285.279999
        data[0,0,0] explicit conversion -285

    pas bon !! et on notera le 100.000002 au lieu de 100. tout rond

    missing value

    make it work

    validation

    lat, lon value

    attributes

    coding rules

    avoid over writing

    write read_msg_xrit_nc.py to check

    compare file in doctest

    EVOLUTIONS
    ==========

    $Id$

    $URL$

    - pinsard 20130801T084854Z loholt1.ipsl.polytechnique.fr (Linux)

      * creation
        trying to be close as much as possible to
        :file:`/homedata/pinsard/varamma_d/MSG/2006/08/01/200608010000_msg-tb108_map_15min.nc`
        created by the suite :ref:`create_nc.sh` and :ref:`create_nc.jnl`

        thanks to http://www.unidata.ucar.edu/software/netcdf/examples/programs/simple_xy_wr.py
        and :samp:`pydoc netCDF4`

        avoid NETCDF4 in :samp:`fid = Dataset(fullfilename, 'w', format='NETCDF4')`
        because of :

        .. code-block:: bash

           ncdump -h /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_nc.nc
           ncdump: /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_nc.nc: NetCDF: Unknown file format

        and:

        .. code-block:: bash

           ncks -M /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_nc.nc
           ERROR NC_ENOTNC Not a netCDF file
           HINT: NC_ENOTNC errors will occur when NCO operators linked to the netCDF3 library attempt to read netCDF4 files.  Are your input files netCDF4 format?  (http://nco.sf.net/nco.html#fmt_inq shows how to tell.) If so then installing or re-building a netCDF4-compatible version of NCO should solve this problem.
           nco_err_exit(): ERROR Short NCO-generated message (usually name of function that triggered error): nco_open()
           nco_err_exit(): ERROR Error code is -51. Translation into English with nc_strerror(-51) is "NetCDF: Unknown file format"
           nco_err_exit(): ERROR NCO will now exit with system call exit(EXIT_FAILURE)

           with NETCDF3_CLASSIC, no such troubles !

        thanks to http://netcdf4-python.googlecode.com/svn/trunk/docs/netCDF3-module.html for time conversion
        in HOURS since ... using date2num

    """

    result = -1

    usage = 'result = write_msg_xrit_nc(values, product, yyyy, mm, dd, hh, mn)'

    project_id_env = os.environ['PROJECT_ID']

    # build filename to be written
    yyyymmddhhmn = datetime(yyyy, mm, dd, hh, mn, 00)
    yyyy = yyyymmddhhmn.strftime('%Y')
    mm = yyyymmddhhmn.strftime('%m')
    dd = yyyymmddhhmn.strftime('%d')
    hh = yyyymmddhhmn.strftime('%H')
    mn = yyyymmddhhmn.strftime('%M')
    template_dir = "MSG/{yyyy}/{mm}/{dd}/"
    relativedir = template_dir.format(yyyy=yyyy, mm=mm, dd=dd)
    # ++ replace 'write_msg_xrit_nc' by the name of this function
    template_filename = "H-000-MSG1__-MSG1________-{product}____-CYCLE____-{yyyy}{mm}{dd}{hh}{mn}.write_msg_xrit_nc.nc"
    fullfilename = project_id_env + relativedir + template_filename.format(product=product, yyyy=yyyy, mm=mm, dd=dd, hh=hh, mn=mn)

    # open a new file
    try:
        fid = Dataset(fullfilename, 'w',  format='NETCDF3_CLASSIC')
        #fid = Dataset(fullfilename, 'w')
    except IOError, err:
        sys.stderr.write("Error: %r. %s.\n" % (err.filename, err.strerror))
        sys.exit(1)

    print('iii : opening for writing %s' % (fullfilename))

    missing_value_nc = numpy.int16(32767)

    # create the time, x and y dimensions
    fid.createDimension('TIME', None)
    fid.createDimension('LATITUDE', values.shape[1])
    fid.createDimension('LONGITUDE', values.shape[0])
    fid.createDimension('x', values.shape[0])
    fid.createDimension('y', values.shape[1])

    # create the variable
    data = fid.createVariable('DATA', numpy.dtype('int16').char, ('TIME', 'x', 'y'), fill_value=missing_value_nc)  # ++TIME,LONGITUDE,LONGITUDE
    latitudes = fid.createVariable('LATITUDE', numpy.dtype('float32').char, ('LATITUDE'))
    longitudes = fid.createVariable('LONGITUDE', numpy.dtype('float32').char, ('LONGITUDE'))
    times = fid.createVariable('TIME', numpy.dtype('float32').char, ('TIME',))
    #
    lats = numpy.arange(values.shape[1], dtype=numpy.float32)  # ++ bad values
    lons = numpy.arange(values.shape[0], dtype=numpy.float32)  # ++ bad values
    latitudes[:] = lats
    longitudes[:] = lons

    # create variables attributes
    longitudes.unit = "DEGREES EAST"
    longitudes.point_spacing = "even"
    longitudes.axis = "X"
    longitudes.modulo = 360.
    latitudes.unit = "DEGREES NORTH"
    latitudes.point_spacing = "even"
    latitudes.axis = "Y"
    if product == 'IR108':
        data.units = "++"
        data.long_name = "++"
        data.missing_value = missing_value_nc
        data.scale_factor = numpy.float32(0.01)
        print data.scale_factor
        inv_scale_factor = 1./data.scale_factor
        data.add_offset = 10.  # ++
    else:
        print ('eee : unknown product %s' % (product))
        sys.exit(1)

    time0 = datetime(1960, 01, 01, 00, 00, 00)
    times.units = time0.strftime('HOURS since %Y-%m-%d %H:%M:%S')
    times.axis = 'T'
    times.origin = time0.strftime('%d-%b-%Y %H:%M:%S').upper()
    times.calendar = 'gregorian'

    # create global attributes
    fid.history = 'Created ' + datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
    fid.source = "this program"  # ++

    # write data to variable
    # ++ missing value vs missing_value_nc
    print('values[0,0] %f' % values[0, 0])
    print('values[0,0]*inv_scale_factor = %f*%f = %f' % (values[0, 0], inv_scale_factor, values[0, 0] * inv_scale_factor))
    data[0, :] = (values * inv_scale_factor)
    print('data[0, 0, 0] implicit conversion %f ' % data[0, 0, 0])
    data[0, :] = numpy.int16(values * inv_scale_factor)
    print('data[0, 0, 0] explicit conversion %d ' % data[0, 0, 0])

    # time unlimited 1 value
    times[0] = date2num(yyyymmddhhmn, units=times.units, calendar=times.calendar)
    # control
    # décommenter pour debuger ou ajouter dans le log debug
    #dates = num2date(times[0],units=times.units,calendar=times.calendar)
    #print 'dates corresponding to time values:\n',dates

    # close file
    fid.close()

    result = 0

# Run main, if called from the command line
if __name__ == '__main__':
    import doctest
    doctest.testmod()
