#!/usr/bin/env python
# -*- coding: utf-8 -*-

__docformat__ = 'reStructuredText'
__autodoc__ = ['write_msg_xrit_raw']

import os
import sys

import datetime


def write_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn):

    """

    ==================
    write_msg_xrit_raw
    ==================

    DESCRIPTION
    ===========

    Write in a file :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/H-000-MSG1__-MSG1________-{product}____-CYCLE____-{yyyy}{mm}{dd}{hh}{mn}.write_msg_xrit_raw.raw`

    Parameters
    ----------

    values :
        values

    yyyy : yyyy
        year

    Returns
    -------

    result : int
       1 if ok

    Raises
    ------

    Missing parameters

    EXAMPLES
    ========

    From scratch to write some simulated data in
    :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_raw.raw`

    >>> from simul_msg_xrit_raw import simul_msg_xrit_raw
    >>> index_simulation = 1
    >>> (product, values, missing_value) = simul_msg_xrit_raw(index_simulation)
    >>> type(values)
    <type 'numpy.ndarray'>
    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> result = write_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn)
    iii : opening for writing /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_raw.raw

    From real file :file:`${PROJECT_ID}/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw`

    >>> from read_msg_xrit_raw import read_msg_xrit_raw
    >>> yyyy = 2006
    >>> mm = 8
    >>> dd = 1
    >>> hh = 0
    >>> mn = 0
    >>> product = 'IR108'
    >>> values = read_msg_xrit_raw(product, yyyy, mm, dd, hh, mn)
    iii : opening for reading /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.raw
    >>> result = write_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn)
    iii : opening for writing /homedata/pinsard/varamma_d/MSG/2006/08/01/H-000-MSG1__-MSG1________-IR108____-CYCLE____-200608010000.write_msg_xrit_raw.raw

    SEE ALSO
    ========

    :ref:`msg_data`

    :func:`simul_msg_xrit_raw`
    :func:`read_msg_xrit_raw`

    TODO
    ====

    coding rules

    avoid over writing

    compare file in doctest

    EVOLUTIONS
    ==========

    $Id$

    $URL$

    - pinsard 20130730T144650Z loholt1.ipsl.polytechnique.fr (Linux)

      * creation

    """

    result = -1
    #
    usage = 'result = write_msg_xrit_raw(values, product, yyyy, mm, dd, hh, mn)'
    #
    project_id_env = os.environ['PROJECT_ID']

    # build filename to be written
    # ++ replace 'write_msg_xrit_raw' by the name of this function
    yyyymmddhhmn = datetime.datetime(yyyy, mm, dd, hh, mn, 00)
    yyyy = yyyymmddhhmn.strftime('%Y')
    mm = yyyymmddhhmn.strftime('%m')
    dd = yyyymmddhhmn.strftime('%d')
    hh = yyyymmddhhmn.strftime('%H')
    mn = yyyymmddhhmn.strftime('%M')
    template_dir = "MSG/{yyyy}/{mm}/{dd}/"
    relativedir = template_dir.format(yyyy=yyyy, mm=mm, dd=dd)
    template_filename = "H-000-MSG1__-MSG1________-{product}____-CYCLE____-{yyyy}{mm}{dd}{hh}{mn}.write_msg_xrit_raw.raw"
    fullfilename = project_id_env + relativedir + template_filename.format(product=product, yyyy=yyyy, mm=mm, dd=dd, hh=hh, mn=mn)

    try:
        fid = open(fullfilename, "wb")
    except IOError, err:
        sys.stderr.write("Error: %r. %s.\n" % (err.filename, err.strerror))
        sys.exit(1)

    print('iii : opening for writing %s' % (fullfilename))
    #
    # write data
    fid.write(values)
    #
    fid.close()
    #
    result = 0
    #

# Run main, if called from the command line
if __name__ == '__main__':
    import doctest
    doctest.testmod()
